@@ -25,6 +25,7 @@ from nova import compute
 import nova.conf
 from nova import exception
 from nova.i18n import _
+from nova import network
 from nova.policies import evacuate as evac_policies
 from nova import utils
 
@@ -36,6 +37,7 @@ class EvacuateController(wsgi.Controller):
         super(EvacuateController, self).__init__(*args, **kwargs)
         self.compute_api = compute.API()
         self.host_api = compute.HostAPI()
+        self.network_api = network.API()
 
     def _get_on_shared_storage(self, req, evacuate_body):
         if api_version_request.is_supported(req, min_version='2.14'):
@@ -73,7 +75,8 @@ class EvacuateController(wsgi.Controller):
     @wsgi.action('evacuate')
     @validation.schema(evacuate.evacuate, "2.0", "2.13")
     @validation.schema(evacuate.evacuate_v214, "2.14", "2.28")
-    @validation.schema(evacuate.evacuate_v2_29, "2.29")
+    @validation.schema(evacuate.evacuate_v2_29, "2.29", "2.67")
+    @validation.schema(evacuate.evacuate_v2_68, "2.68")
     def _evacuate(self, req, id, body):
         """Permit admins to evacuate a server from a failed host
         to a new one.
@@ -114,6 +117,18 @@ class EvacuateController(wsgi.Controller):
             msg = _("The target host can't be the same one.")
             raise exc.HTTPBadRequest(explanation=msg)
 
+        # We could potentially move this check to conductor and avoid the
+        # extra API call to neutron when we support move operations with ports
+        # having resource requests.
+        if (common.instance_has_port_with_resource_request(
+                    context, instance.uuid, self.network_api) and not
+                common.supports_port_resource_request_during_move(req)):
+            msg = _("The evacuate action on a server with ports having "
+                    "resource requests, like a port with a QoS minimum "
+                    "bandwidth policy, is not supported with this "
+                    "microversion")
+            raise exc.HTTPBadRequest(explanation=msg)
+
         try:
             self.compute_api.evacuate(context, instance, host,
                                       on_shared_storage, password, force)
