@@ -12,8 +12,6 @@
 
 import webob
 
-from oslo_log import log as logging
-
 from nova.api.openstack import api_version_request
 from nova.api.openstack.compute.schemas import flavor_manage
 from nova.api.openstack.compute.views import flavors as flavors_view
@@ -22,12 +20,10 @@ from nova.api import validation
 from nova.compute import flavors
 from nova import exception
 from nova import objects
-from nova.policies import base
+from nova.policies import flavor_extra_specs as fes_policies
 from nova.policies import flavor_manage as fm_policies
-from nova import policy
 
 
-LOG = logging.getLogger(__name__)
 ALIAS = "os-flavor-manage"
 
 
@@ -46,15 +42,7 @@ class FlavorManageController(wsgi.Controller):
     @wsgi.action("delete")
     def _delete(self, req, id):
         context = req.environ['nova.context']
-        # TODO(rb560u): remove this check in future release
-        using_old_action = \
-            policy.verify_deprecated_policy(fm_policies.BASE_POLICY_NAME,
-                fm_policies.POLICY_ROOT % 'delete',
-                base.RULE_ADMIN_API,
-                context)
-
-        if not using_old_action:
-            context.can(fm_policies.POLICY_ROOT % 'delete')
+        context.can(fm_policies.POLICY_ROOT % 'delete')
 
         flavor = objects.Flavor(context=context, flavorid=id)
         try:
@@ -72,15 +60,7 @@ class FlavorManageController(wsgi.Controller):
                        flavors_view.FLAVOR_DESCRIPTION_MICROVERSION)
     def _create(self, req, body):
         context = req.environ['nova.context']
-        # TODO(rb560u): remove this check in future release
-        using_old_action = \
-            policy.verify_deprecated_policy(fm_policies.BASE_POLICY_NAME,
-                fm_policies.POLICY_ROOT % 'create',
-                base.RULE_ADMIN_API,
-                context)
-
-        if not using_old_action:
-            context.can(fm_policies.POLICY_ROOT % 'create')
+        context.can(fm_policies.POLICY_ROOT % 'create')
 
         vals = body['flavor']
 
@@ -108,12 +88,21 @@ class FlavorManageController(wsgi.Controller):
                                     description=description)
             # NOTE(gmann): For backward compatibility, non public flavor
             # access is not being added for created tenant. Ref -bug/1209101
-            req.cache_db_flavor(flavor)
         except (exception.FlavorExists,
                 exception.FlavorIdExists) as err:
             raise webob.exc.HTTPConflict(explanation=err.format_message())
 
-        return self._view_builder.show(req, flavor, include_description)
+        include_extra_specs = False
+        if api_version_request.is_supported(
+                req, flavors_view.FLAVOR_EXTRA_SPECS_MICROVERSION):
+            include_extra_specs = context.can(
+                fes_policies.POLICY_ROOT % 'index', fatal=False)
+            # NOTE(yikun): This empty extra_spec only for keeping consistent
+            # with other related flavor api.
+            flavor.extra_specs = {}
+
+        return self._view_builder.show(req, flavor, include_description,
+                                       include_extra_specs=include_extra_specs)
 
     @wsgi.Controller.api_version(flavors_view.FLAVOR_DESCRIPTION_MICROVERSION)
     @wsgi.action('update')
@@ -133,7 +122,10 @@ class FlavorManageController(wsgi.Controller):
         except exception.FlavorNotFound as e:
             raise webob.exc.HTTPNotFound(explanation=e.format_message())
 
-        # Cache the flavor so the flavor_access and flavor_rxtx extensions
-        # can add stuff to the response.
-        req.cache_db_flavor(flavor)
-        return self._view_builder.show(req, flavor, include_description=True)
+        include_extra_specs = False
+        if api_version_request.is_supported(
+                req, flavors_view.FLAVOR_EXTRA_SPECS_MICROVERSION):
+            include_extra_specs = context.can(
+                fes_policies.POLICY_ROOT % 'index', fatal=False)
+        return self._view_builder.show(req, flavor, include_description=True,
+                                       include_extra_specs=include_extra_specs)
