@@ -76,9 +76,7 @@ class LimitsController(wsgi.Controller):
         context.can(limits_policies.BASE_POLICY_NAME)
         project_id = req.params.get('tenant_id', context.project_id)
         quotas = QUOTAS.get_project_quotas(context, project_id,
-                                           usages=False)
-        abs_limits = {k: v['limit'] for k, v in quotas.items()}
-
+                                           usages=True)
         builder = limits_views.ViewBuilder()
-        return builder.build(abs_limits, filtered_limits=filtered_limits,
+        return builder.build(req, quotas, filtered_limits=filtered_limits,
                              max_image_meta=max_image_meta)
