@@ -14,11 +14,138 @@
 
 import copy
 
-from nova.api.openstack.compute.schemas import user_data
 from nova.api.validation import parameter_types
 from nova.api.validation.parameter_types import multi_params
 from nova.objects import instance
 
+legacy_block_device_mapping = {
+    'type': 'object',
+    'properties': {
+        'virtual_name': {
+            'type': 'string', 'maxLength': 255,
+        },
+        'volume_id': parameter_types.volume_id,
+        'snapshot_id': parameter_types.image_id,
+        'volume_size': parameter_types.volume_size,
+        # Do not allow empty device names and number values and
+        # containing spaces(defined in nova/block_device.py:from_api())
+        'device_name': {
+            'type': 'string', 'minLength': 1, 'maxLength': 255,
+            'pattern': '^[a-zA-Z0-9._-r/]*$',
+        },
+        # Defined as boolean in nova/block_device.py:from_api()
+        'delete_on_termination': parameter_types.boolean,
+        'no_device': {},
+        # Defined as mediumtext in column "connection_info" in table
+        # "block_device_mapping"
+        'connection_info': {
+            'type': 'string', 'maxLength': 16777215
+        },
+    },
+    'additionalProperties': False
+}
+
+block_device_mapping_v2_new_item = {
+    # defined in nova/block_device.py:from_api()
+    # NOTE: Client can specify the Id with the combination of
+    # source_type and uuid, or a single attribute like volume_id/
+    # image_id/snapshot_id.
+    'source_type': {
+        'type': 'string',
+        'enum': ['volume', 'image', 'snapshot', 'blank'],
+    },
+    'uuid': {
+        'type': 'string', 'minLength': 1, 'maxLength': 255,
+        'pattern': '^[a-zA-Z0-9._-]*$',
+    },
+    'image_id': parameter_types.image_id,
+    'destination_type': {
+        'type': 'string',
+        'enum': ['local', 'volume'],
+    },
+    # Defined as varchar(255) in column "guest_format" in table
+    # "block_device_mapping"
+    'guest_format': {
+        'type': 'string', 'maxLength': 255,
+    },
+    # Defined as varchar(255) in column "device_type" in table
+    # "block_device_mapping"
+    'device_type': {
+        'type': 'string', 'maxLength': 255,
+    },
+    # Defined as varchar(255) in column "disk_bus" in table
+    # "block_device_mapping"
+    'disk_bus': {
+        'type': 'string', 'maxLength': 255,
+    },
+    # Defined as integer in nova/block_device.py:from_api()
+    # NOTE(mriedem): boot_index=None is also accepted for backward
+    # compatibility with the legacy v2 API.
+    'boot_index': {
+        'type': ['integer', 'string', 'null'],
+        'pattern': '^-?[0-9]+$',
+    },
+}
+
+block_device_mapping_v2 = copy.deepcopy(legacy_block_device_mapping)
+block_device_mapping_v2['properties'].update(block_device_mapping_v2_new_item)
+
+_hints = {
+    'type': 'object',
+    'properties': {
+        'group': {
+            'type': 'string',
+            'format': 'uuid'
+        },
+        'different_host': {
+            # NOTE: The value of 'different_host' is the set of server
+            # uuids where a new server is scheduled on a different host.
+            # A user can specify one server as string parameter and should
+            # specify multiple servers as array parameter instead.
+            'oneOf': [
+                {
+                    'type': 'string',
+                    'format': 'uuid'
+                },
+                {
+                    'type': 'array',
+                    'items': parameter_types.server_id
+                }
+            ]
+        },
+        'same_host': {
+            # NOTE: The value of 'same_host' is the set of server
+            # uuids where a new server is scheduled on the same host.
+            'type': ['string', 'array'],
+            'items': parameter_types.server_id
+        },
+        'query': {
+            # NOTE: The value of 'query' is converted to dict data with
+            # jsonutils.loads() and used for filtering hosts.
+            'type': ['string', 'object'],
+        },
+        # NOTE: The value of 'target_cell' is the cell name what cell
+        # a new server is scheduled on.
+        'target_cell': parameter_types.name,
+        'different_cell': {
+            'type': ['string', 'array'],
+            'items': {
+                'type': 'string'
+            }
+        },
+        'build_near_host_ip': parameter_types.ip_address,
+        'cidr': {
+            'type': 'string',
+            'pattern': '^\/[0-9a-f.:]+$'
+        },
+    },
+    # NOTE: As this Mail:
+    # http://lists.openstack.org/pipermail/openstack-dev/2015-June/067996.html
+    # pointed out the limit the scheduler-hints in the API is problematic. So
+    # relax it.
+    'additionalProperties': True
+}
+
 base_create = {
     'type': 'object',
     'properties': {
@@ -55,10 +182,44 @@ base_create = {
                 'accessIPv6': parameter_types.accessIPv6,
                 'personality': parameter_types.personality,
                 'availability_zone': parameter_types.name,
+                'block_device_mapping': {
+                    'type': 'array',
+                    'items': legacy_block_device_mapping
+                },
+                'block_device_mapping_v2': {
+                    'type': 'array',
+                    'items': block_device_mapping_v2
+                },
+                'config_drive': parameter_types.boolean,
+                'key_name': parameter_types.name,
+                'min_count': parameter_types.positive_integer,
+                'max_count': parameter_types.positive_integer,
+                'return_reservation_id': parameter_types.boolean,
+                'security_groups': {
+                    'type': 'array',
+                    'items': {
+                        'type': 'object',
+                        'properties': {
+                            # NOTE(oomichi): allocate_for_instance() of
+                            # neutronv2/api.py gets security_group names
+                            # or UUIDs from this parameter.
+                            # parameter_types.name allows both format.
+                            'name': parameter_types.name,
+                        },
+                        'additionalProperties': False,
+                    }
+                },
+                'user_data': {
+                    'type': 'string',
+                    'format': 'base64',
+                    'maxLength': 65535
+                }
             },
             'required': ['name', 'flavorRef'],
             'additionalProperties': False,
         },
+        'os:scheduler_hints': _hints,
+        'OS-SCH-HNT:scheduler_hints': _hints,
     },
     'required': ['server'],
     'additionalProperties': False,
@@ -70,24 +231,54 @@ base_create_v20['properties']['server'][
     'properties']['name'] = parameter_types.name_with_leading_trailing_spaces
 base_create_v20['properties']['server']['properties'][
     'availability_zone'] = parameter_types.name_with_leading_trailing_spaces
-
+base_create_v20['properties']['server']['properties'][
+    'key_name'] = parameter_types.name_with_leading_trailing_spaces
+base_create_v20['properties']['server']['properties'][
+    'security_groups']['items']['properties']['name'] = (
+    parameter_types.name_with_leading_trailing_spaces)
+base_create_v20['properties']['server']['properties'][
+    'user_data'] = {
+        'oneOf': [{'type': 'string', 'format': 'base64', 'maxLength': 65535},
+                  {'type': 'null'},
+        ],
+    }
 
 base_create_v219 = copy.deepcopy(base_create)
 base_create_v219['properties']['server'][
     'properties']['description'] = parameter_types.description
 
-
 base_create_v232 = copy.deepcopy(base_create_v219)
 base_create_v232['properties']['server'][
     'properties']['networks']['items'][
     'properties']['tag'] = parameter_types.tag
+base_create_v232['properties']['server'][
+    'properties']['block_device_mapping_v2']['items'][
+    'properties']['tag'] = parameter_types.tag
 
+# NOTE(artom) the following conditional was merged as
+# "if version == '2.32'" The intent all along was to check whether
+# version was greater than or equal to 2.32. In other words, we wanted
+# to support tags in versions 2.32 and up, but ended up supporting them
+# in version 2.32 only. Since we need a new microversion to add request
+# body attributes, tags have been re-added in version 2.42.
+
+# NOTE(gmann) Below schema 'base_create_v233' is added (builds on 2.19 schema)
+# to keep the above mentioned behavior while merging the extension schema code
+# into server schema file. Below is the ref code where BDM tag was originally
+# got added for 2.32 microversion *only*.
+# Ref- https://github.com/openstack/nova/blob/
+#      9882a60e69a5ab8da314a199a56defc05098b743/nova/api/
+#      openstack/compute/block_device_mapping.py#L71
+base_create_v233 = copy.deepcopy(base_create_v219)
+base_create_v233['properties']['server'][
+    'properties']['networks']['items'][
+    'properties']['tag'] = parameter_types.tag
 
 # 2.37 builds on 2.32 and makes the following changes:
 # 1. server.networks is required
 # 2. server.networks is now either an enum or a list
 # 3. server.networks.uuid is now required to be a uuid
-base_create_v237 = copy.deepcopy(base_create_v232)
+base_create_v237 = copy.deepcopy(base_create_v233)
 base_create_v237['properties']['server']['required'].append('networks')
 base_create_v237['properties']['server']['properties']['networks'] = {
     'oneOf': [
@@ -131,6 +322,9 @@ base_create_v242['properties']['server']['properties']['networks'] = {
         },
         {'type': 'string', 'enum': ['none', 'auto']},
     ]}
+base_create_v242['properties']['server'][
+    'properties']['block_device_mapping_v2']['items'][
+    'properties']['tag'] = parameter_types.tag
 
 
 # 2.52 builds on 2.42 and makes the following changes:
@@ -148,6 +342,20 @@ base_create_v257 = copy.deepcopy(base_create_v252)
 base_create_v257['properties']['server']['properties'].pop('personality')
 
 
+# 2.63 builds on 2.57 and makes the following changes:
+# Allowing adding trusted certificates to instances when booting
+base_create_v263 = copy.deepcopy(base_create_v257)
+base_create_v263['properties']['server']['properties'][
+    'trusted_image_certificates'] = parameter_types.trusted_certs
+
+
+# Add volume type in block_device_mapping_v2.
+base_create_v267 = copy.deepcopy(base_create_v263)
+base_create_v267['properties']['server']['properties'][
+    'block_device_mapping_v2']['items'][
+    'properties']['volume_type'] = parameter_types.volume_type
+
+
 base_update = {
     'type': 'object',
     'properties': {
@@ -219,11 +427,17 @@ base_rebuild_v257 = copy.deepcopy(base_rebuild_v254)
 base_rebuild_v257['properties']['rebuild']['properties'].pop('personality')
 base_rebuild_v257['properties']['rebuild']['properties']['user_data'] = ({
     'oneOf': [
-        user_data.common_user_data,
+        {'type': 'string', 'format': 'base64', 'maxLength': 65535},
         {'type': 'null'}
     ]
 })
 
+# 2.63 builds on 2.57 and makes the following changes:
+# Allowing adding trusted certificates to instances when rebuilding
+base_rebuild_v263 = copy.deepcopy(base_rebuild_v257)
+base_rebuild_v263['properties']['rebuild']['properties'][
+    'trusted_image_certificates'] = parameter_types.trusted_certs
+
 resize = {
     'type': 'object',
     'properties': {
@@ -379,6 +593,7 @@ query_params_v21 = {
         'sort_key': multi_params(VALID_SORT_KEYS),
         'sort_dir': parameter_types.common_query_param,
         'all_tenants': parameter_types.common_query_param,
+        'soft_deleted': parameter_types.common_query_param,
         'deleted': parameter_types.common_query_param,
         'status': parameter_types.common_query_param,
         'changes-since': multi_params({'type': 'string',
@@ -413,3 +628,9 @@ query_params_v226['properties'].update({
     'not-tags': parameter_types.common_query_regex_param,
     'not-tags-any': parameter_types.common_query_regex_param,
 })
+
+query_params_v266 = copy.deepcopy(query_params_v226)
+query_params_v266['properties'].update({
+    'changes-before': multi_params({'type': 'string',
+                                    'format': 'date-time'}),
+})
