@@ -16,15 +16,36 @@ Common parameter types for validating request Body.
 
 """
 import copy
+import functools
 import re
 import unicodedata
 
 import six
 
-from nova import db
 from nova.i18n import _
 from nova.objects import tag
 
+_REGEX_RANGE_CACHE = {}
+
+
+def memorize(func):
+
+    @functools.wraps(func)
+    def memorizer(*args, **kwargs):
+        global _REGEX_RANGE_CACHE
+        key = "%s:%s:%s" % (func.__name__, hash(str(args)), hash(str(kwargs)))
+        value = _REGEX_RANGE_CACHE.get(key)
+        if value is None:
+            value = func(*args, **kwargs)
+            _REGEX_RANGE_CACHE[key] = value
+        return value
+    return memorizer
+
+
+def _reset_cache():
+    global _REGEX_RANGE_CACHE
+    _REGEX_RANGE_CACHE = {}
+
 
 def single_param(schema):
     """Macro function for use in JSONSchema to support query parameters that
@@ -84,6 +105,7 @@ def _get_all_chars():
 # constraint fails and this causes issues for some unittests when
 # PYTHONHASHSEED is set randomly.
 
+@memorize
 def _build_regex_range(ws=True, invert=False, exclude=None):
     """Build a range regex for a set of characters in utf8.
 
@@ -352,6 +374,11 @@ volume_id = {
 }
 
 
+volume_type = {
+    'type': ['string', 'null'], 'minLength': 0, 'maxLength': 255
+}
+
+
 network_id = {
     'type': 'string', 'format': 'uuid'
 }
@@ -427,7 +454,9 @@ volume_size = {
     'type': ['integer', 'string'],
     'pattern': '^[0-9]+$',
     'minimum': 1,
-    'maximum': db.MAX_INT
+    # maximum's value is limited to db constant's MAX_INT
+    # (in nova/db/constants.py)
+    'maximum': 0x7FFFFFFF
 }
 
 disk_config = {
@@ -476,3 +505,16 @@ pagination_parameters = {
     'limit': multi_params(non_negative_integer),
     'marker': multi_params({'type': 'string'})
 }
+
+# The trusted_certs list is restricted to a maximum of 50 IDs.
+# "null" is allowed to unset/reset trusted certs during rebuild.
+trusted_certs = {
+    "type": ["array", "null"],
+    "minItems": 1,
+    "maxItems": 50,
+    "uniqueItems": True,
+    "items": {
+        "type": "string",
+        "minLength": 1,
+    }
+}
