@@ -26,7 +26,6 @@ import nova.conf
 from nova import config
 from nova import objects
 from nova import service
-from nova import utils
 from nova import version
 
 CONF = nova.conf.CONF
@@ -36,7 +35,6 @@ LOG = logging.getLogger('nova.cells')
 def main():
     config.parse_args(sys.argv)
     logging.setup(CONF, 'nova')
-    utils.monkey_patch()
     objects.register_all()
     gmr_opts.set_defaults(CONF)
 
