@@ -19,18 +19,7 @@ from oslo_db import options as oslo_db_options
 from nova.conf import paths
 
 _DEFAULT_SQL_CONNECTION = 'sqlite:///' + paths.state_path_def('nova.sqlite')
-
-
-# NOTE(sdague): we know of at least 1 instance of out of tree usage
-# for this config in RAX. They used this because of performance issues
-# with some queries. We think the right path forward is fixing the
-# SQLA queries to be more performant for everyone.
-db_driver_opt = cfg.StrOpt(
-        'db_driver',
-        default='nova.db',
-        deprecated_for_removal=True,
-        deprecated_since='13.0.0',
-        help='The driver to use for database access')
+_ENRICHED = False
 
 
 # NOTE(markus_z): We cannot simply do:
@@ -42,8 +31,8 @@ db_driver_opt = cfg.StrOpt(
 # group here. See commit ba407e3 ("Add support for multiple database engines")
 # for more details.
 api_db_group = cfg.OptGroup('api_database',
-                            title='API Database Options',
-                            help="""
+    title='API Database Options',
+    help="""
 The *Nova API Database* is a separate database which is used for information
 which is used across *cells*. This database is mandatory since the Mitaka
 release (13.0.0).
@@ -52,49 +41,52 @@ release (13.0.0).
 api_db_opts = [
     # TODO(markus_z): This should probably have a required=True attribute
     cfg.StrOpt('connection',
-               secret=True,
-               help=''),
+        secret=True,
+        help=''),
+    cfg.StrOpt('connection_parameters',
+        default='',
+        help=''),
     cfg.BoolOpt('sqlite_synchronous',
-                default=True,
-                help=''),
+        default=True,
+        help=''),
     cfg.StrOpt('slave_connection',
-               secret=True,
-               help=''),
+        secret=True,
+        help=''),
     cfg.StrOpt('mysql_sql_mode',
-               default='TRADITIONAL',
-               help=''),
+        default='TRADITIONAL',
+        help=''),
     cfg.IntOpt('connection_recycle_time',
-               default=3600,
-               deprecated_name='idle_timeout',
-               help=''),
+        default=3600,
+        deprecated_name='idle_timeout',
+        help=''),
     # TODO(markus_z): We should probably default this to 5 to not rely on the
     # SQLAlchemy default. Otherwise we wouldn't provide a stable default.
     cfg.IntOpt('max_pool_size',
-               help=''),
+        help=''),
     cfg.IntOpt('max_retries',
-               default=10,
-               help=''),
+        default=10,
+        help=''),
     # TODO(markus_z): This should have a minimum attribute of 0
     cfg.IntOpt('retry_interval',
-               default=10,
-               help=''),
+        default=10,
+        help=''),
     # TODO(markus_z): We should probably default this to 10 to not rely on the
     # SQLAlchemy default. Otherwise we wouldn't provide a stable default.
     cfg.IntOpt('max_overflow',
-               help=''),
+        help=''),
     # TODO(markus_z): This should probably make use of the "choices" attribute.
     # "oslo.db" uses only the values [<0, 0, 50, 100] see module
     # /oslo_db/sqlalchemy/engines.py method "_setup_logging"
     cfg.IntOpt('connection_debug',
-               default=0,
-               help=''),
+        default=0,
+        help=''),
     cfg.BoolOpt('connection_trace',
-                default=False,
-                help=''),
+        default=False,
+        help=''),
     # TODO(markus_z): We should probably default this to 30 to not rely on the
     # SQLAlchemy default. Otherwise we wouldn't provide a stable default.
     cfg.IntOpt('pool_timeout',
-               help='')
+        help='')
 ]  # noqa
 
 
@@ -113,11 +105,61 @@ def enrich_help_text(alt_db_opts):
                 # texts here if needed.
                 alt_db_opt.help = db_opt.help + alt_db_opt.help
 
+# NOTE(cdent): See the note above on api_db_group. The same issues
+# apply here.
+
+placement_db_group = cfg.OptGroup('placement_database',
+                                  title='Placement API database options',
+                                  help="""
+The *Placement API Database* is a separate database which can be used with the
+placement service. This database is optional: if the connection option is not
+set, the nova api database will be used instead.
+""")
+
+placement_db_opts = [
+    cfg.StrOpt('connection',
+        help='',
+        secret=True),
+    cfg.StrOpt('connection_parameters',
+        default='',
+        help=''),
+    cfg.BoolOpt('sqlite_synchronous',
+        default=True,
+        help=''),
+    cfg.StrOpt('slave_connection',
+        secret=True,
+        help=''),
+    cfg.StrOpt('mysql_sql_mode',
+        default='TRADITIONAL',
+        help=''),
+    cfg.IntOpt('connection_recycle_time',
+        default=3600,
+        help=''),
+    cfg.IntOpt('max_pool_size',
+        help=''),
+    cfg.IntOpt('max_retries',
+        default=10,
+        help=''),
+    cfg.IntOpt('retry_interval',
+        default=10,
+        help=''),
+    cfg.IntOpt('max_overflow',
+        help=''),
+    cfg.IntOpt('connection_debug',
+        default=0,
+        help=''),
+    cfg.BoolOpt('connection_trace',
+        default=False,
+        help=''),
+    cfg.IntOpt('pool_timeout',
+        help=''),
+]  # noqa
+
 
 def register_opts(conf):
     oslo_db_options.set_defaults(conf, connection=_DEFAULT_SQL_CONNECTION)
-    conf.register_opt(db_driver_opt)
     conf.register_opts(api_db_opts, group=api_db_group)
+    conf.register_opts(placement_db_opts, group=placement_db_group)
 
 
 def list_opts():
@@ -128,7 +170,12 @@ def list_opts():
     # As I think it is useful to have the "oslo.db" namespace information
     # in the "sample.conf" file, I omit the listing of the "oslo_db_options"
     # here.
-    enrich_help_text(api_db_opts)
-    return {'DEFAULT': [db_driver_opt],
-            api_db_group: api_db_opts,
-            }
+    global _ENRICHED
+    if not _ENRICHED:
+        enrich_help_text(api_db_opts)
+        enrich_help_text(placement_db_opts)
+        _ENRICHED = True
+    return {
+        api_db_group: api_db_opts,
+        placement_db_group: placement_db_opts,
+    }
