@@ -100,8 +100,9 @@ class ConsoleAuthManager(manager.Manager):
 
         self.mc_instance.set(instance_uuid.encode('UTF-8'),
                              jsonutils.dumps(tokens))
-        token_dict['token'] = '***'
-        LOG.info("Received Token: %(token_dict)s", {'token_dict': token_dict})
+
+        LOG.info("Received Token: %(token)s, %(token_dict)s",
+                 {'token': token, 'token_dict': token_dict})
 
     def _validate_token(self, context, token):
         instance_uuid = token['instance_uuid']
@@ -129,8 +130,8 @@ class ConsoleAuthManager(manager.Manager):
     def check_token(self, context, token):
         token_str = self.mc.get(token.encode('UTF-8'))
         token_valid = (token_str is not None)
-        LOG.info("Checking that token is known: %(token_valid)s",
-                 {'token_valid': token_valid})
+        LOG.info("Checking Token: %(token)s, %(token_valid)s",
+                 {'token': token, 'token_valid': token_valid})
         if token_valid:
             token = jsonutils.loads(token_str)
             if self._validate_token(context, token):
