@@ -85,7 +85,7 @@ import_translation_for_log_or_exception = re.compile(
     r"(.)*(from\snova.i18n\simport)\s_")
 # We need this for cases where they have created their own _ function.
 custom_underscore_check = re.compile(r"(.)*_\s*=\s*(.)*")
-api_version_re = re.compile(r"@.*api_version")
+api_version_re = re.compile(r"@.*\bapi_version\b")
 dict_constructor_with_list_copy_re = re.compile(r".*\bdict\((\[)?(\(|\[)")
 decorator_re = re.compile(r"@.*")
 http_not_implemented_re = re.compile(r"raise .*HTTPNotImplemented\(")
@@ -98,6 +98,10 @@ log_remove_context = re.compile(
     r"(.)*LOG\.(.*)\(.*(context=[_a-zA-Z0-9].*)+.*\)")
 return_not_followed_by_space = re.compile(r"^\s*return(?:\(|{|\"|'|#).*$")
 uuid4_re = re.compile(r"uuid4\(\)($|[^\.]|\.hex)")
+redundant_import_alias_re = re.compile(r"import (?:.*\.)?(.+) as \1$")
+yield_not_followed_by_space = re.compile(r"^\s*yield(?:\(|{|\[|\"|').*$")
+asse_regexpmatches = re.compile(
+    r"(assertRegexpMatches|assertNotRegexpMatches)\(")
 
 
 class BaseASTChecker(ast.NodeVisitor):
@@ -152,8 +156,8 @@ def import_no_db_in_virt(logical_line, filename):
     N307
     """
     if "nova/virt" in filename and not filename.endswith("fake.py"):
-        if logical_line.startswith("from nova import db"):
-            yield (0, "N307: nova.db import not allowed in nova/virt/*")
+        if logical_line.startswith("from nova.db import api"):
+            yield (0, "N307: nova.db.api import not allowed in nova/virt/*")
 
 
 def no_db_session_in_public_api(logical_line, filename):
@@ -234,7 +238,7 @@ def capital_cfg_help(logical_line, tokens):
             if tokens[t][1] == "help":
                 txt = tokens[t + 2][1]
                 if len(txt) > 1 and txt[1].islower():
-                    yield(0, msg)
+                    yield (0, msg)
 
 
 def assert_true_instance(logical_line):
@@ -257,8 +261,8 @@ def assert_equal_type(logical_line):
 
 def check_python3_xrange(logical_line):
     if re.search(r"\bxrange\s*\(", logical_line):
-        yield(0, "N327: Do not use xrange(). 'xrange()' is not compatible "
-              "with Python 3. Use range() or six.moves.range() instead.")
+        yield (0, "N327: Do not use xrange(). 'xrange()' is not compatible "
+                  "with Python 3. Use range() or six.moves.range() instead.")
 
 
 def no_translate_debug_logs(logical_line, filename):
@@ -275,7 +279,7 @@ def no_translate_debug_logs(logical_line, filename):
     N319
     """
     if logical_line.startswith("LOG.debug(_("):
-        yield(0, "N319 Don't translate debug level logs")
+        yield (0, "N319 Don't translate debug level logs")
 
 
 def no_import_translation_in_tests(logical_line, filename):
@@ -285,7 +289,7 @@ def no_import_translation_in_tests(logical_line, filename):
     if 'nova/tests/' in filename:
         res = import_translation_for_log_or_exception.match(logical_line)
         if res:
-            yield(0, "N337 Don't import translation in tests")
+            yield (0, "N337 Don't import translation in tests")
 
 
 def no_setting_conf_directly_in_tests(logical_line, filename):
@@ -328,7 +332,7 @@ def check_explicit_underscore_import(logical_line, filename):
         UNDERSCORE_IMPORT_FILES.append(filename)
     elif (translated_log.match(logical_line) or
          string_translation.match(logical_line)):
-        yield(0, "N323: Found use of _() without explicit import of _ !")
+        yield (0, "N323: Found use of _() without explicit import of _ !")
 
 
 def use_jsonutils(logical_line, filename):
@@ -358,7 +362,7 @@ def check_api_version_decorator(logical_line, previous_logical, blank_before,
            " on a method.")
     if blank_before == 0 and re.match(api_version_re, logical_line) \
            and re.match(decorator_re, previous_logical):
-        yield(0, msg)
+        yield (0, msg)
 
 
 class CheckForStrUnicodeExc(BaseASTChecker):
@@ -559,7 +563,7 @@ def check_http_not_implemented(logical_line, physical_line, filename):
     if ("nova/api/openstack/compute" not in filename):
         return
     if re.match(http_not_implemented_re, logical_line):
-        yield(0, msg)
+        yield (0, msg)
 
 
 def check_greenthread_spawns(logical_line, physical_line, filename):
@@ -586,7 +590,7 @@ def check_no_contextlib_nested(logical_line, filename):
            "more information. nova.test.nested() is an alternative as well.")
 
     if contextlib_nested.match(logical_line):
-        yield(0, msg)
+        yield (0, msg)
 
 
 def check_config_option_in_central_place(logical_line, filename):
@@ -615,21 +619,24 @@ def check_config_option_in_central_place(logical_line, filename):
         return
 
     if cfg_opt_re.match(logical_line):
-        yield(0, msg)
+        yield (0, msg)
 
 
 def check_policy_registration_in_central_place(logical_line, filename):
-    msg = ('N350: Policy registration should be in the central location '
-           '"/nova/policies/*".')
+    msg = ('N350: Policy registration should be in the central location(s) '
+           '"/nova/policies/*" or "nova/api/openstack/placement/policies/*".')
     # This is where registration should happen
-    if "nova/policies/" in filename:
+    if ("nova/policies/" in filename or
+            "nova/api/openstack/placement/policies/" in filename):
         return
     # A couple of policy tests register rules
-    if "nova/tests/unit/test_policy.py" in filename:
+    if ("nova/tests/unit/test_policy.py" in filename or
+            "nova/tests/unit/api/openstack/placement/test_policy.py" in
+            filename):
         return
 
     if rule_default_re.match(logical_line):
-        yield(0, msg)
+        yield (0, msg)
 
 
 def check_policy_enforce(logical_line, filename):
@@ -647,7 +654,7 @@ def check_policy_enforce(logical_line, filename):
            'Use the authorize() method instead.')
 
     if policy_enforce_re.match(logical_line):
-        yield(0, msg)
+        yield (0, msg)
 
 
 def check_doubled_words(physical_line, filename):
@@ -667,21 +674,21 @@ def check_python3_no_iteritems(logical_line):
     msg = ("N344: Use items() instead of dict.iteritems().")
 
     if re.search(r".*\.iteritems\(\)", logical_line):
-        yield(0, msg)
+        yield (0, msg)
 
 
 def check_python3_no_iterkeys(logical_line):
     msg = ("N345: Use six.iterkeys() instead of dict.iterkeys().")
 
     if re.search(r".*\.iterkeys\(\)", logical_line):
-        yield(0, msg)
+        yield (0, msg)
 
 
 def check_python3_no_itervalues(logical_line):
     msg = ("N346: Use six.itervalues() instead of dict.itervalues().")
 
     if re.search(r".*\.itervalues\(\)", logical_line):
-        yield(0, msg)
+        yield (0, msg)
 
 
 def no_os_popen(logical_line):
@@ -694,8 +701,8 @@ def no_os_popen(logical_line):
     """
 
     if 'os.popen(' in logical_line:
-        yield(0, 'N348 Deprecated library function os.popen(). '
-                 'Replace it using subprocess module. ')
+        yield (0, 'N348 Deprecated library function os.popen(). '
+                  'Replace it using subprocess module. ')
 
 
 def no_log_warn(logical_line):
@@ -728,11 +735,11 @@ def check_context_log(logical_line, physical_line, filename):
         return
 
     if log_remove_context.match(logical_line):
-        yield(0,
-              "N353: Nova is using oslo.context's RequestContext "
-              "which means the context object is in scope when "
-              "doing logging using oslo.log, so no need to pass it as"
-              "kwarg.")
+        yield (0,
+               "N353: Nova is using oslo.context's RequestContext "
+               "which means the context object is in scope when "
+               "doing logging using oslo.log, so no need to pass it as "
+               "kwarg.")
 
 
 def no_assert_equal_true_false(logical_line):
@@ -792,13 +799,56 @@ def return_followed_by_space(logical_line):
     if there are unnecessary parentheses in the written code.
 
     Not correct: return(42), return(a, b)
-    Correct: return 42, return (a, b), return a, b
+    Correct: return, return 42, return (a, b), return a, b
 
     N358
     """
     if return_not_followed_by_space.match(logical_line):
         yield (0,
-               "N357: Return keyword should be followed by a space.")
+               "N358: Return keyword should be followed by a space.")
+
+
+def no_redundant_import_alias(logical_line):
+    """Check for redundant import aliases.
+
+    Imports should not be in the forms below.
+
+    from x import y as y
+    import x as x
+    import x.y as y
+
+    N359
+    """
+    if re.search(redundant_import_alias_re, logical_line):
+        yield (0, "N359: Import alias should not be redundant.")
+
+
+def yield_followed_by_space(logical_line):
+    """Yield should be followed by a space.
+
+    Yield should be followed by a space to clarify that yield is
+    not a function. Adding a space may force the developer to rethink
+    if there are unnecessary parentheses in the written code.
+
+    Not correct: yield(x), yield(a, b)
+    Correct: yield x, yield (a, b), yield a, b
+
+    N360
+    """
+    if yield_not_followed_by_space.match(logical_line):
+        yield (0,
+               "N360: Yield keyword should be followed by a space.")
+
+
+def assert_regexpmatches(logical_line):
+    """Check for usage of deprecated assertRegexpMatches/assertNotRegexpMatches
+
+    N361
+    """
+    res = asse_regexpmatches.search(logical_line)
+    if res:
+        yield (0, "N361: assertRegex/assertNotRegex must be used instead "
+                  "of assertRegexpMatches/assertNotRegexpMatches.")
 
 
 def factory(register):
@@ -842,3 +892,6 @@ def factory(register):
     register(no_assert_true_false_is_not)
     register(check_uuid4)
     register(return_followed_by_space)
+    register(no_redundant_import_alias)
+    register(yield_followed_by_space)
+    register(assert_regexpmatches)
