@@ -33,7 +33,7 @@ from nova import manager
 from nova import objects
 from nova.objects import host_mapping as host_mapping_obj
 from nova import quota
-from nova.scheduler import client as scheduler_client
+from nova.scheduler.client import report
 from nova.scheduler import request_filter
 from nova.scheduler import utils
 
@@ -53,8 +53,7 @@ class SchedulerManager(manager.Manager):
     _sentinel = object()
 
     def __init__(self, scheduler_driver=None, *args, **kwargs):
-        client = scheduler_client.SchedulerClient()
-        self.placement_client = client.reportclient
+        self.placement_client = report.SchedulerReportClient()
         if not scheduler_driver:
             scheduler_driver = CONF.scheduler.driver
         self.driver = driver.DriverManager(
@@ -147,10 +146,10 @@ class SchedulerManager(manager.Manager):
                 (alloc_reqs, provider_summaries,
                             allocation_request_version) = res
             if not alloc_reqs:
-                LOG.debug("Got no allocation candidates from the Placement "
-                          "API. This could be due to insufficient resources "
-                          "or a temporary occurrence as compute nodes start "
-                          "up.")
+                LOG.info("Got no allocation candidates from the Placement "
+                         "API. This could be due to insufficient resources "
+                         "or a temporary occurrence as compute nodes start "
+                         "up.")
                 raise exception.NoValidHost(reason="")
             else:
                 # Build a dict of lists of allocation requests, keyed by
