@@ -243,7 +243,8 @@ class ResourceRequest(object):
     def _clean_empties(self):
         """Get rid of any empty ResourceGroup instances."""
         for ident, rg in list(self._rg_by_id.items()):
-            if not any((rg.resources, rg.required_traits)):
+            if not any((rg.resources, rg.required_traits,
+                        rg.forbidden_traits)):
                 self._rg_by_id.pop(ident)
 
     def strip_zeros(self):
@@ -490,6 +491,12 @@ def resources_from_request_spec(spec_obj):
     if 'force_nodes' in spec_obj and spec_obj.force_nodes:
         res_req._limit = None
 
+    # Don't limit allocation candidates when using affinity/anti-affinity.
+    if ('scheduler_hints' in spec_obj and any(
+        key in ['group', 'same_host', 'different_host']
+        for key in spec_obj.scheduler_hints)):
+        res_req._limit = None
+
     return res_req
 
 
