@@ -43,6 +43,9 @@ class TestInstanceNotificationSampleWithMultipleCompute(
         self.useFixture(fixtures.AllServicesCurrent())
 
     def test_multiple_compute_actions(self):
+        # There are not going to be real network-vif-plugged events coming
+        # so don't wait for them.
+        self.flags(live_migration_wait_for_vif_plug=False, group='compute')
         server = self._boot_a_server(
             extra_params={'networks': [{'port': self.neutron.port_1['id']}]})
         self._wait_for_notification('instance.create.end')
@@ -76,102 +79,119 @@ class TestInstanceNotificationSampleWithMultipleCompute(
             'os-migrateLive': {
                 'host': 'host2',
                 'block_migration': True,
-                'force': True,
             }
         }
         self.admin_api.post_server_action(server['id'], post)
         self._wait_for_notification(
             'instance.live_migration_rollback_dest.end')
 
-        # 0. instance.live_migration_rollback.start
-        # 1. instance.live_migration_rollback.end
-        # 2. instance.live_migration_rollback_dest.start
-        # 3. instance.live_migration_rollback_dest.end
-        self.assertEqual(4, len(fake_notifier.VERSIONED_NOTIFICATIONS))
+        # 0. scheduler.select_destinations.start
+        # 1. scheduler.select_destinations.end
+        # 2. instance.live_migration_rollback.start
+        # 3. instance.live_migration_rollback.end
+        # 4. instance.live_migration_rollback_dest.start
+        # 5. instance.live_migration_rollback_dest.end
+        self.assertEqual(6, len(fake_notifier.VERSIONED_NOTIFICATIONS),
+                         [x['event_type'] for x in
+                          fake_notifier.VERSIONED_NOTIFICATIONS])
         self._verify_notification(
             'instance-live_migration_rollback-start',
             replacements={
                 'reservation_id': server['reservation_id'],
                 'uuid': server['id']},
-            actual=fake_notifier.VERSIONED_NOTIFICATIONS[0])
+            actual=fake_notifier.VERSIONED_NOTIFICATIONS[2])
         self._verify_notification(
             'instance-live_migration_rollback-end',
             replacements={
                 'reservation_id': server['reservation_id'],
                 'uuid': server['id']},
-            actual=fake_notifier.VERSIONED_NOTIFICATIONS[1])
+            actual=fake_notifier.VERSIONED_NOTIFICATIONS[3])
         self._verify_notification(
             'instance-live_migration_rollback_dest-start',
             replacements={
                 'reservation_id': server['reservation_id'],
                 'uuid': server['id']},
-            actual=fake_notifier.VERSIONED_NOTIFICATIONS[2])
+            actual=fake_notifier.VERSIONED_NOTIFICATIONS[4])
         self._verify_notification(
             'instance-live_migration_rollback_dest-end',
             replacements={
                 'reservation_id': server['reservation_id'],
                 'uuid': server['id']},
-            actual=fake_notifier.VERSIONED_NOTIFICATIONS[3])
+            actual=fake_notifier.VERSIONED_NOTIFICATIONS[5])
 
     def _test_live_migration_success(self, server):
         post = {
             'os-migrateLive': {
                 'host': 'host2',
                 'block_migration': True,
-                'force': True,
             }
         }
         self.admin_api.post_server_action(server['id'], post)
         self._wait_for_notification('instance.live_migration_pre.end')
-        self.assertEqual(2, len(fake_notifier.VERSIONED_NOTIFICATIONS))
+        # 0. scheduler.select_destinations.start
+        # 1. scheduler.select_destinations.end
+        # 2. instance.live_migration_pre.start
+        # 3. instance.live_migration_pre.end
+        self.assertEqual(4, len(fake_notifier.VERSIONED_NOTIFICATIONS),
+                         [x['event_type'] for x in
+                          fake_notifier.VERSIONED_NOTIFICATIONS])
         self._verify_notification(
             'instance-live_migration_pre-start',
             replacements={
                 'reservation_id': server['reservation_id'],
                 'uuid': server['id']},
-            actual=fake_notifier.VERSIONED_NOTIFICATIONS[0])
+            actual=fake_notifier.VERSIONED_NOTIFICATIONS[2])
         self._verify_notification(
             'instance-live_migration_pre-end',
             replacements={
                 'reservation_id': server['reservation_id'],
                 'uuid': server['id']},
-            actual=fake_notifier.VERSIONED_NOTIFICATIONS[1])
+            actual=fake_notifier.VERSIONED_NOTIFICATIONS[3])
         migrations = self.admin_api.get_active_migrations(server['id'])
         self.assertEqual(1, len(migrations))
 
         self._wait_for_notification('instance.live_migration_post.end')
-        self.assertEqual(6, len(fake_notifier.VERSIONED_NOTIFICATIONS))
+        # 0. scheduler.select_destinations.start
+        # 1. scheduler.select_destinations.end
+        # 2. instance.live_migration_pre.start
+        # 3. instance.live_migration_pre.end
+        # 4. instance.live_migration_post.start
+        # 5. instance.live_migration_post_dest.start
+        # 6. instance.live_migration_post_dest.end
+        # 7. instance.live_migration_post.end
+        self.assertEqual(8, len(fake_notifier.VERSIONED_NOTIFICATIONS),
+                         [x['event_type'] for x in
+                          fake_notifier.VERSIONED_NOTIFICATIONS])
         self._verify_notification(
             'instance-live_migration_post-start',
             replacements={
                 'reservation_id': server['reservation_id'],
                 'uuid': server['id']},
-            actual=fake_notifier.VERSIONED_NOTIFICATIONS[2])
+            actual=fake_notifier.VERSIONED_NOTIFICATIONS[4])
         self._verify_notification(
             'instance-live_migration_post_dest-start',
             replacements={
                 'reservation_id': server['reservation_id'],
                 'uuid': server['id']},
-            actual=fake_notifier.VERSIONED_NOTIFICATIONS[3])
+            actual=fake_notifier.VERSIONED_NOTIFICATIONS[5])
         self._verify_notification(
             'instance-live_migration_post_dest-end',
             replacements={
                 'reservation_id': server['reservation_id'],
                 'uuid': server['id']},
-            actual=fake_notifier.VERSIONED_NOTIFICATIONS[4])
+            actual=fake_notifier.VERSIONED_NOTIFICATIONS[6])
         self._verify_notification(
             'instance-live_migration_post-end',
             replacements={
                 'reservation_id': server['reservation_id'],
                 'uuid': server['id']},
-            actual=fake_notifier.VERSIONED_NOTIFICATIONS[5])
+            actual=fake_notifier.VERSIONED_NOTIFICATIONS[7])
 
     def _test_live_migration_abort(self, server):
         post = {
             "os-migrateLive": {
                 "host": "host2",
                 "block_migration": False,
-                "force": True
             }
         }
 
@@ -188,43 +208,54 @@ class TestInstanceNotificationSampleWithMultipleCompute(
         # wait for the rollback to ensure we can assert both notifications
         # below
         self._wait_for_notification('instance.live_migration_rollback.end')
-        self.assertEqual(6, len(fake_notifier.VERSIONED_NOTIFICATIONS))
+
+        # 0. scheduler.select_destinations.start
+        # 1. scheduler.select_destinations.end
+        # 2. instance.live_migration_pre.start
+        # 3. instance.live_migration_pre.end
+        # 4. instance.live_migration_abort.start
+        # 5. instance.live_migration_abort.end
+        # 6. instance.live_migration_rollback.start
+        # 7. instance.live_migration_rollback.end
+        self.assertEqual(8, len(fake_notifier.VERSIONED_NOTIFICATIONS),
+                         [x['event_type'] for x in
+                          fake_notifier.VERSIONED_NOTIFICATIONS])
         self._verify_notification(
             'instance-live_migration_pre-start',
             replacements={
                 'reservation_id': server['reservation_id'],
                 'uuid': server['id']},
-            actual=fake_notifier.VERSIONED_NOTIFICATIONS[0])
+            actual=fake_notifier.VERSIONED_NOTIFICATIONS[2])
         self._verify_notification(
             'instance-live_migration_pre-end',
             replacements={
                 'reservation_id': server['reservation_id'],
                 'uuid': server['id']},
-            actual=fake_notifier.VERSIONED_NOTIFICATIONS[1])
+            actual=fake_notifier.VERSIONED_NOTIFICATIONS[3])
         self._verify_notification(
             'instance-live_migration_abort-start',
             replacements={
                 'reservation_id': server['reservation_id'],
                 'uuid': server['id']},
-            actual=fake_notifier.VERSIONED_NOTIFICATIONS[2])
+            actual=fake_notifier.VERSIONED_NOTIFICATIONS[4])
         self._verify_notification(
             'instance-live_migration_abort-end',
             replacements={
                 'reservation_id': server['reservation_id'],
                 'uuid': server['id']},
-            actual=fake_notifier.VERSIONED_NOTIFICATIONS[3])
+            actual=fake_notifier.VERSIONED_NOTIFICATIONS[5])
         self._verify_notification(
             'instance-live_migration_rollback-start',
             replacements={
                 'reservation_id': server['reservation_id'],
                 'uuid': server['id']},
-            actual=fake_notifier.VERSIONED_NOTIFICATIONS[4])
+            actual=fake_notifier.VERSIONED_NOTIFICATIONS[6])
         self._verify_notification(
             'instance-live_migration_rollback-end',
             replacements={
                 'reservation_id': server['reservation_id'],
                 'uuid': server['id']},
-            actual=fake_notifier.VERSIONED_NOTIFICATIONS[5])
+            actual=fake_notifier.VERSIONED_NOTIFICATIONS[7])
 
     def _test_evacuate_server(self, server):
         services = self.admin_api.get_services(host='host2',
@@ -234,7 +265,6 @@ class TestInstanceNotificationSampleWithMultipleCompute(
         evacuate = {
             'evacuate': {
                 'host': 'compute',
-                'force': True
             }
         }
 
@@ -259,7 +289,6 @@ class TestInstanceNotificationSampleWithMultipleCompute(
             'os-migrateLive': {
                 'host': 'host2',
                 'block_migration': True,
-                'force': True,
             }
         }
         self.admin_api.post_server_action(server['id'], post)
@@ -273,23 +302,25 @@ class TestInstanceNotificationSampleWithMultipleCompute(
         self._wait_for_notification(
             'instance.live_migration_force_complete.end')
 
-        # 0. instance.live_migration_pre.start
-        # 1. instance.live_migration_pre.end
-        # 2. instance.live_migration_force_complete.start
-        # 3. instance.live_migration_force_complete.end
-        self.assertEqual(4, len(fake_notifier.VERSIONED_NOTIFICATIONS))
+        # 0. scheduler.select_destinations.start
+        # 1. scheduler.select_destinations.end
+        # 2. instance.live_migration_pre.start
+        # 3. instance.live_migration_pre.end
+        # 4. instance.live_migration_force_complete.start
+        # 5. instance.live_migration_force_complete.end
+        self.assertEqual(6, len(fake_notifier.VERSIONED_NOTIFICATIONS))
         self._verify_notification(
             'instance-live_migration_force_complete-start',
             replacements={
                 'reservation_id': server['reservation_id'],
                 'uuid': server['id']},
-            actual=fake_notifier.VERSIONED_NOTIFICATIONS[2])
+            actual=fake_notifier.VERSIONED_NOTIFICATIONS[4])
         self._verify_notification(
             'instance-live_migration_force_complete-end',
             replacements={
                 'reservation_id': server['reservation_id'],
                 'uuid': server['id']},
-            actual=fake_notifier.VERSIONED_NOTIFICATIONS[3])
+            actual=fake_notifier.VERSIONED_NOTIFICATIONS[5])
 
 
 class TestInstanceNotificationSampleWithMultipleComputeOldAttachFlow(
@@ -428,9 +459,13 @@ class TestInstanceNotificationSample(
                           'tags': ['tag'],
                           'trusted_image_certificates': fake_trusted_certs})
 
-        self.assertEqual(2, len(fake_notifier.VERSIONED_NOTIFICATIONS))
+        # 0. scheduler.select_destinations.start
+        # 1. scheduler.select_destinations.end
+        # 2. instance-create-start
+        # 3. instance-create-error
+        self.assertEqual(4, len(fake_notifier.VERSIONED_NOTIFICATIONS))
 
-        tb = fake_notifier.VERSIONED_NOTIFICATIONS[1]['payload'][
+        tb = fake_notifier.VERSIONED_NOTIFICATIONS[3]['payload'][
             'nova_object.data']['fault']['nova_object.data']['traceback']
         self.assertIn('raise exception.FlavorDiskTooSmall()', tb)
 
@@ -439,13 +474,32 @@ class TestInstanceNotificationSample(
             replacements={
                 'reservation_id': server['reservation_id'],
                 'uuid': server['id']},
-            actual=fake_notifier.VERSIONED_NOTIFICATIONS[0])
+            actual=fake_notifier.VERSIONED_NOTIFICATIONS[2])
         self._verify_notification(
             'instance-create-error',
             replacements={
                 'reservation_id': server['reservation_id'],
                 'uuid': server['id'],
                 'fault.traceback': self.ANY},
+            actual=fake_notifier.VERSIONED_NOTIFICATIONS[3])
+
+        fake_notifier.reset()
+
+        self.api.delete_server(server['id'])
+        self._wait_until_deleted(server)
+
+        self.assertEqual(2, len(fake_notifier.VERSIONED_NOTIFICATIONS))
+        self._verify_notification(
+            'instance-delete-start_not_scheduled',
+            replacements={
+                'reservation_id': server['reservation_id'],
+                'uuid': server['id']},
+            actual=fake_notifier.VERSIONED_NOTIFICATIONS[0])
+        self._verify_notification(
+            'instance-delete-end_not_scheduled',
+            replacements={
+                'reservation_id': server['reservation_id'],
+                'uuid': server['id']},
             actual=fake_notifier.VERSIONED_NOTIFICATIONS[1])
 
     def test_instance_exists_usage_audit(self):
@@ -488,6 +542,36 @@ class TestInstanceNotificationSample(
             },
             actual=notifications[0])
 
+    def test_delete_server_while_compute_is_down(self):
+
+        server = self._boot_a_server(
+            expected_status='ACTIVE',
+            extra_params={'networks': [{'port': self.neutron.port_1['id']}]})
+        self._attach_volume_to_server(server, self.cinder.SWAP_OLD_VOL)
+
+        service_id = self.api.get_service_id('nova-compute')
+        self.admin_api.put_service_force_down(service_id, True)
+        fake_notifier.reset()
+
+        self.api.delete_server(server['id'])
+        self._wait_until_deleted(server)
+
+        self.assertEqual(2, len(fake_notifier.VERSIONED_NOTIFICATIONS))
+        self._verify_notification(
+            'instance-delete-start_compute_down',
+            replacements={
+                'reservation_id': server['reservation_id'],
+                'uuid': server['id']},
+            actual=fake_notifier.VERSIONED_NOTIFICATIONS[0])
+        self._verify_notification(
+            'instance-delete-end_compute_down',
+            replacements={
+                'reservation_id': server['reservation_id'],
+                'uuid': server['id']},
+            actual=fake_notifier.VERSIONED_NOTIFICATIONS[1])
+
+        self.admin_api.put_service_force_down(service_id, False)
+
     def _verify_instance_update_steps(self, steps, notifications,
                                       initial=None):
         replacements = {}
@@ -752,7 +836,6 @@ class TestInstanceNotificationSample(
 
         self.api.post_server_action(server['id'], {'unshelve': None})
         self._wait_for_state_change(self.api, server, 'ACTIVE')
-        self._wait_for_notification('instance.unshelve.end')
 
     def _test_unshelve_server(self, server):
         # setting the shelved_offload_time to 0 should set the
@@ -770,20 +853,19 @@ class TestInstanceNotificationSample(
         post = {'unshelve': None}
         self.api.post_server_action(server['id'], post)
         self._wait_for_state_change(self.admin_api, server, 'ACTIVE')
-        self._wait_for_notification('instance.unshelve.end')
-        self.assertEqual(7, len(fake_notifier.VERSIONED_NOTIFICATIONS))
+        self.assertEqual(9, len(fake_notifier.VERSIONED_NOTIFICATIONS))
         self._verify_notification(
             'instance-unshelve-start',
             replacements={
                 'reservation_id': server['reservation_id'],
                 'uuid': server['id']},
-            actual=fake_notifier.VERSIONED_NOTIFICATIONS[5])
+            actual=fake_notifier.VERSIONED_NOTIFICATIONS[7])
         self._verify_notification(
             'instance-unshelve-end',
             replacements={
                 'reservation_id': server['reservation_id'],
                 'uuid': server['id']},
-            actual=fake_notifier.VERSIONED_NOTIFICATIONS[6])
+            actual=fake_notifier.VERSIONED_NOTIFICATIONS[8])
 
     def _test_suspend_resume_server(self, server):
         post = {'suspend': {}}
@@ -866,6 +948,27 @@ class TestInstanceNotificationSample(
                 'uuid': server['id']},
             actual=fake_notifier.VERSIONED_NOTIFICATIONS[3])
 
+    def _build_destination_payload(self):
+        cell1 = self.cell_mappings.get('cell1')
+        return {
+            'nova_object.version': '1.0',
+            'nova_object.namespace': 'nova',
+            'nova_object.name': 'DestinationPayload',
+            'nova_object.data': {
+                'aggregates': None,
+                'cell': {
+                    'nova_object.version': '2.0',
+                    'nova_object.namespace': 'nova',
+                    'nova_object.name': 'CellMappingPayload',
+                    'nova_object.data': {
+                        'disabled': False,
+                        'name': u'cell1',
+                        'uuid': cell1.uuid
+                    }
+                }
+            }
+        }
+
     def _test_resize_and_revert_server(self, server):
         self.flags(allow_resize_to_same_host=True)
         other_flavor_body = {
@@ -894,9 +997,19 @@ class TestInstanceNotificationSample(
         self.api.post_server_action(server['id'], post)
         self._wait_for_state_change(self.api, server, 'VERIFY_RESIZE')
 
+        self._pop_and_verify_dest_select_notification(server['id'],
+            replacements={
+                'ignore_hosts': [],
+                'flavor.memory_mb': other_flavor_body['flavor']['ram'],
+                'flavor.name': other_flavor_body['flavor']['name'],
+                'flavor.flavorid': other_flavor_id,
+                'flavor.extra_specs': extra_specs['extra_specs'],
+                'requested_destination': self._build_destination_payload()})
+
         self.assertEqual(7, len(fake_notifier.VERSIONED_NOTIFICATIONS))
         # ignore instance.exists
         fake_notifier.VERSIONED_NOTIFICATIONS.pop(0)
+
         # This list needs to be in order.
         expected_notifications = [
             'instance-resize_prep-start',
@@ -971,6 +1084,13 @@ class TestInstanceNotificationSample(
         mock_prep_resize.side_effect = _build_resources
         self.api.post_server_action(server['id'], post)
         self._wait_for_notification('instance.resize.error')
+        self._pop_and_verify_dest_select_notification(server['id'],
+            replacements={
+                'ignore_hosts': [],
+                'flavor.name': other_flavor_body['flavor']['name'],
+                'flavor.flavorid': other_flavor_id,
+                'flavor.extra_specs': {},
+                'requested_destination': self._build_destination_payload()})
         # 0: instance-exists
         # 1: instance-resize_prep-start
         # 2: instance-resize-error
@@ -1035,7 +1155,8 @@ class TestInstanceNotificationSample(
         self.api.post_server_action(server['id'], post)
         self._wait_for_state_change(self.api, server, expected_status='ERROR')
         self._wait_for_notification('compute.exception')
-        # There should be the following notifications.
+        # There should be the following notifications after scheduler's
+        # select_destination notifications:
         # 0: instance-exists
         # 1: instance-resize_prep-start
         # 2: instance-resize-error
@@ -1043,6 +1164,13 @@ class TestInstanceNotificationSample(
         # 4: compute.exception
         #    (via the wrap_exception decorator on
         #    the ComputeManager.prep_resize method.)
+        self._pop_and_verify_dest_select_notification(server['id'],
+            replacements={
+                'ignore_hosts': [],
+                'flavor.name': other_flavor_body['flavor']['name'],
+                'flavor.flavorid': other_flavor_id,
+                'flavor.extra_specs': {},
+                'requested_destination': self._build_destination_payload()})
         self.assertEqual(5, len(fake_notifier.VERSIONED_NOTIFICATIONS),
                          'Unexpected number of notifications: %s' %
                          fake_notifier.VERSIONED_NOTIFICATIONS)
@@ -1092,10 +1220,11 @@ class TestInstanceNotificationSample(
 
         fake_notifier.reset()
 
+        image_ref = 'a2459075-d96c-40d5-893e-577ff92e721c'
         post = {
             'rebuild': {
-                'imageRef': 'a2459075-d96c-40d5-893e-577ff92e721c',
-                'metadata': {},
+                'imageRef': image_ref,
+                'metadata': {}
             }
         }
         self.api.post_server_action(server['id'], post)
@@ -1106,6 +1235,22 @@ class TestInstanceNotificationSample(
         self._wait_for_state_change(self.api, server,
                                     expected_status='ACTIVE')
 
+        self._pop_and_verify_dest_select_notification(server['id'],
+            replacements={
+                'image.container_format': 'ami',
+                'image.disk_format': 'ami',
+                'image.id': image_ref,
+                'image.properties': {
+                    'nova_object.data': {},
+                    'nova_object.name': 'ImageMetaPropsPayload',
+                    'nova_object.namespace': 'nova',
+                    'nova_object.version': u'1.0'},
+                'image.size': 58145823,
+                'image.tags': [],
+                'scheduler_hints': {'_nova_check_type': ['rebuild']},
+                'force_hosts': 'compute',
+                'force_nodes': 'fake-mini'})
+
         # 0. instance.rebuild_scheduled
         # 1. instance.exists
         # 2. instance.rebuild.start
@@ -1168,10 +1313,11 @@ class TestInstanceNotificationSample(
 
         fake_notifier.reset()
 
+        image_ref = 'a2459075-d96c-40d5-893e-577ff92e721c'
         rebuild_trusted_certs = ['rebuild-cert-id-1', 'rebuild-cert-id-2']
         post = {
             'rebuild': {
-                'imageRef': 'a2459075-d96c-40d5-893e-577ff92e721c',
+                'imageRef': image_ref,
                 'metadata': {},
                 'trusted_image_certificates': rebuild_trusted_certs,
             }
@@ -1184,6 +1330,22 @@ class TestInstanceNotificationSample(
         self._wait_for_state_change(self.api, server,
                                     expected_status='ACTIVE')
 
+        self._pop_and_verify_dest_select_notification(server['id'],
+            replacements={
+                'image.container_format': 'ami',
+                'image.disk_format': 'ami',
+                'image.id': image_ref,
+                'image.properties': {
+                    'nova_object.data': {},
+                    'nova_object.name': 'ImageMetaPropsPayload',
+                    'nova_object.namespace': 'nova',
+                    'nova_object.version': u'1.0'},
+                'image.size': 58145823,
+                'image.tags': [],
+                'scheduler_hints': {'_nova_check_type': ['rebuild']},
+                'force_hosts': 'compute',
+                'force_nodes': 'fake-mini'})
+
         # 0. instance.rebuild_scheduled
         # 1. instance.exists
         # 2. instance.rebuild.start
