@@ -12,11 +12,13 @@
 
 import mock
 
+from nova.conductor import api as conductor_api
 from nova import context as nova_context
 from nova import objects
 from nova.scheduler import weights
 from nova import test
 from nova.tests import fixtures as nova_fixtures
+from nova.tests.functional import fixtures as func_fixtures
 from nova.tests.functional import integrated_helpers
 from nova.tests.unit.image import fake as fake_image
 from nova.virt import fake as fake_virt
@@ -59,7 +61,7 @@ class MissingReqSpecInstanceGroupUUIDTestCase(
         super(MissingReqSpecInstanceGroupUUIDTestCase, self).setUp()
         # Stub out external dependencies.
         self.useFixture(nova_fixtures.NeutronFixture(self))
-        self.useFixture(nova_fixtures.PlacementFixture())
+        self.useFixture(func_fixtures.PlacementFixture())
         fake_image.stub_out_image_service(self)
         self.addCleanup(fake_image.FakeImageService_reset)
         # Configure the API to allow resizing to the same host so we can keep
@@ -110,6 +112,25 @@ class MissingReqSpecInstanceGroupUUIDTestCase(
         reqspec = objects.RequestSpec.get_by_instance_uuid(ctxt, server['id'])
         self.assertEqual(group_id, reqspec.instance_group.uuid)
 
+        # Stub out the ComputeTaskAPI.resize_instance method to not pass the
+        # request spec from compute to conductor during a reschedule.
+        # This simulates the pre-Stein reschedule behavior.
+        original_resize_instance = conductor_api.ComputeTaskAPI.resize_instance
+
+        def stub_resize_instance(_self, context, instance,
+                                 extra_instance_updates, scheduler_hint,
+                                 *args, **kwargs):
+            # Only remove the request spec if we know we're rescheduling
+            # which we can determine from the filter_properties retry dict.
+            filter_properties = scheduler_hint['filter_properties']
+            if filter_properties.get('retry', {}).get('exc'):
+                kwargs.pop('request_spec', None)
+            return original_resize_instance(
+                _self, context, instance, extra_instance_updates,
+                scheduler_hint, *args, **kwargs)
+        self.stub_out('nova.conductor.api.ComputeTaskAPI.resize_instance',
+                      stub_resize_instance)
+
         # Now cold migrate the server. Because of allow_resize_to_same_host and
         # the weigher, the scheduler will pick host1 first. The FakeDriver
         # actually allows migrating to the same host so we need to stub that
