@@ -13,8 +13,11 @@
 #    License for the specific language governing permissions and limitations
 #    under the License.
 
+import time
+
 import mock
 from oslo_config import cfg
+import six
 
 from nova import context
 from nova.db import api as db
@@ -22,6 +25,7 @@ from nova.db.sqlalchemy import api as db_api
 from nova import test
 from nova.tests import fixtures as nova_fixtures
 from nova.tests.functional.api import client
+from nova.tests.functional import fixtures as func_fixtures
 from nova.tests.functional import integrated_helpers
 from nova.tests.unit import policy_fixture
 from nova import utils
@@ -47,6 +51,10 @@ class ServerGroupTestBase(test.TestCase,
                         + ['ServerGroupAntiAffinityFilter',
                            'ServerGroupAffinityFilter'])
 
+    # Override servicegroup parameters to make the tests run faster
+    _service_down_time = 10
+    _report_interval = 1
+
     anti_affinity = {'name': 'fake-name-1', 'policies': ['anti-affinity']}
     affinity = {'name': 'fake-name-2', 'policies': ['affinity']}
 
@@ -62,11 +70,13 @@ class ServerGroupTestBase(test.TestCase,
         self.flags(disk_allocation_ratio=9999.0)
         self.flags(weight_classes=self._get_weight_classes(),
                    group='filter_scheduler')
+        self.flags(service_down_time=self._service_down_time)
+        self.flags(report_interval=self._report_interval)
 
         self.useFixture(policy_fixture.RealPolicyFixture())
         self.useFixture(nova_fixtures.NeutronFixture(self))
 
-        self.useFixture(nova_fixtures.PlacementFixture())
+        self.useFixture(func_fixtures.PlacementFixture())
         api_fixture = self.useFixture(nova_fixtures.OSAPIFixture(
             api_version='v2.1'))
 
@@ -135,13 +145,13 @@ class ServerGroupTestV21(ServerGroupTestBase):
         self.stub_out('nova.virt.driver.load_compute_driver',
                       _fake_load_compute_driver)
         fake.set_nodes(['compute'])
+        self.addCleanup(fake.restore_nodes)
         self.compute = self.start_service('compute', host='compute')
 
         # NOTE(gibi): start a second compute host to be able to test affinity
         # NOTE(sbauza): Make sure the FakeDriver returns a different nodename
         # for the second compute node.
         fake.set_nodes(['host2'])
-        self.addCleanup(fake.restore_nodes)
         self.compute2 = self.start_service('compute', host='host2')
 
     def test_get_no_groups(self):
@@ -404,20 +414,16 @@ class ServerGroupTestV21(ServerGroupTestBase):
 
         return host
 
-    def _set_forced_down(self, service, forced_down):
-        # Use microversion 2.53 for PUT /os-services/{service_id} force down.
-        with utils.temporary_mutation(self.admin_api, microversion='2.53'):
-            self.admin_api.put_service_force_down(service.service_ref.uuid,
-                                                  forced_down)
-
     def test_evacuate_with_anti_affinity(self):
         created_group = self.api.post_server_groups(self.anti_affinity)
         servers = self._boot_servers_to_group(created_group)
 
         host = self._get_compute_service_by_host_name(
             servers[1]['OS-EXT-SRV-ATTR:host'])
-        # Set forced_down on the host to ensure nova considers the host down.
-        self._set_forced_down(host, True)
+        host.stop()
+        # Need to wait service_down_time amount of seconds to ensure
+        # nova considers the host down
+        time.sleep(self._service_down_time)
 
         # Start additional host to test evacuation
         fake.set_nodes(['host3'])
@@ -436,14 +442,18 @@ class ServerGroupTestV21(ServerGroupTestBase):
         self.assertNotEqual(evacuated_server['OS-EXT-SRV-ATTR:host'],
                             servers[0]['OS-EXT-SRV-ATTR:host'])
 
+        host.start()
+
     def test_evacuate_with_anti_affinity_no_valid_host(self):
         created_group = self.api.post_server_groups(self.anti_affinity)
         servers = self._boot_servers_to_group(created_group)
 
         host = self._get_compute_service_by_host_name(
             servers[1]['OS-EXT-SRV-ATTR:host'])
-        # Set forced_down on the host to ensure nova considers the host down.
-        self._set_forced_down(host, True)
+        host.stop()
+        # Need to wait service_down_time amount of seconds to ensure
+        # nova considers the host down
+        time.sleep(self._service_down_time)
 
         post = {'evacuate': {'onSharedStorage': False}}
         self.admin_api.post_server_action(servers[1]['id'], post)
@@ -456,14 +466,18 @@ class ServerGroupTestV21(ServerGroupTestBase):
         self.assertEqual(server_after_failed_evac['OS-EXT-SRV-ATTR:host'],
                          servers[1]['OS-EXT-SRV-ATTR:host'])
 
+        host.start()
+
     def test_evacuate_with_affinity_no_valid_host(self):
         created_group = self.api.post_server_groups(self.affinity)
         servers = self._boot_servers_to_group(created_group)
 
         host = self._get_compute_service_by_host_name(
             servers[1]['OS-EXT-SRV-ATTR:host'])
-        # Set forced_down on the host to ensure nova considers the host down.
-        self._set_forced_down(host, True)
+        host.stop()
+        # Need to wait service_down_time amount of seconds to ensure
+        # nova considers the host down
+        time.sleep(self._service_down_time)
 
         post = {'evacuate': {'onSharedStorage': False}}
         self.admin_api.post_server_action(servers[1]['id'], post)
@@ -476,6 +490,8 @@ class ServerGroupTestV21(ServerGroupTestBase):
         self.assertEqual(server_after_failed_evac['OS-EXT-SRV-ATTR:host'],
                          servers[1]['OS-EXT-SRV-ATTR:host'])
 
+        host.start()
+
     def test_soft_affinity_not_supported(self):
         ex = self.assertRaises(client.OpenStackApiException,
                                self.api.post_server_groups,
@@ -601,8 +617,10 @@ class ServerGroupTestV215(ServerGroupTestV21):
 
         host = self._get_compute_service_by_host_name(
             servers[1]['OS-EXT-SRV-ATTR:host'])
-        # Set forced_down on the host to ensure nova considers the host down.
-        self._set_forced_down(host, True)
+        host.stop()
+        # Need to wait service_down_time amount of seconds to ensure
+        # nova considers the host down
+        time.sleep(self._service_down_time)
 
         # Start additional host to test evacuation
         fake.set_nodes(['host3'])
@@ -622,6 +640,7 @@ class ServerGroupTestV215(ServerGroupTestV21):
                             servers[0]['OS-EXT-SRV-ATTR:host'])
 
         compute3.kill()
+        host.start()
 
     def test_evacuate_with_anti_affinity_no_valid_host(self):
         created_group = self.api.post_server_groups(self.anti_affinity)
@@ -629,8 +648,10 @@ class ServerGroupTestV215(ServerGroupTestV21):
 
         host = self._get_compute_service_by_host_name(
             servers[1]['OS-EXT-SRV-ATTR:host'])
-        # Set forced_down on the host to ensure nova considers the host down.
-        self._set_forced_down(host, True)
+        host.stop()
+        # Need to wait service_down_time amount of seconds to ensure
+        # nova considers the host down
+        time.sleep(self._service_down_time)
 
         post = {'evacuate': {}}
         self.admin_api.post_server_action(servers[1]['id'], post)
@@ -643,14 +664,18 @@ class ServerGroupTestV215(ServerGroupTestV21):
         self.assertEqual(server_after_failed_evac['OS-EXT-SRV-ATTR:host'],
                          servers[1]['OS-EXT-SRV-ATTR:host'])
 
+        host.start()
+
     def test_evacuate_with_affinity_no_valid_host(self):
         created_group = self.api.post_server_groups(self.affinity)
         servers = self._boot_servers_to_group(created_group)
 
         host = self._get_compute_service_by_host_name(
             servers[1]['OS-EXT-SRV-ATTR:host'])
-        # Set forced_down on the host to ensure nova considers the host down.
-        self._set_forced_down(host, True)
+        host.stop()
+        # Need to wait service_down_time amount of seconds to ensure
+        # nova considers the host down
+        time.sleep(self._service_down_time)
 
         post = {'evacuate': {}}
         self.admin_api.post_server_action(servers[1]['id'], post)
@@ -663,6 +688,8 @@ class ServerGroupTestV215(ServerGroupTestV21):
         self.assertEqual(server_after_failed_evac['OS-EXT-SRV-ATTR:host'],
                          servers[1]['OS-EXT-SRV-ATTR:host'])
 
+        host.start()
+
     def _check_group_format(self, group, created_group):
         self.assertEqual(group['policies'], created_group['policies'])
         self.assertEqual({}, created_group['metadata'])
@@ -791,8 +818,10 @@ class ServerGroupTestV215(ServerGroupTestV21):
 
         host = self._get_compute_service_by_host_name(
             servers[1]['OS-EXT-SRV-ATTR:host'])
-        # Set forced_down on the host to ensure nova considers the host down.
-        self._set_forced_down(host, True)
+        host.stop()
+        # Need to wait service_down_time amount of seconds to ensure
+        # nova considers the host down
+        time.sleep(self._service_down_time)
 
         post = {'evacuate': {}}
         self.admin_api.post_server_action(servers[1]['id'], post)
@@ -805,6 +834,8 @@ class ServerGroupTestV215(ServerGroupTestV21):
         # new host later
         evacuated_server = self.admin_api.get_server(evacuated_server['id'])
 
+        host.start()
+
         return [evacuated_server['OS-EXT-SRV-ATTR:host'],
                 servers[0]['OS-EXT-SRV-ATTR:host']]
 
@@ -880,7 +911,6 @@ class ServerGroupTestMultiCell(ServerGroupTestBase):
         self.compute1 = self.start_service('compute', host='host1',
                                            cell='cell1')
         fake.set_nodes(['host2'])
-        self.addCleanup(fake.restore_nodes)
         self.compute2 = self.start_service('compute', host='host2',
                                            cell='cell2')
         # This is needed to find a server that is still booting with multiple
@@ -942,3 +972,96 @@ class ServerGroupTestMultiCell(ServerGroupTestBase):
         # because group members found in cell2 should violate the policy.
         self._boot_a_server_to_group(created_group, az='cell1',
                                      expected_status='ERROR')
+
+
+class TestAntiAffinityLiveMigration(test.TestCase,
+                                    integrated_helpers.InstanceHelperMixin):
+
+    def setUp(self):
+        super(TestAntiAffinityLiveMigration, self).setUp()
+        # Setup common fixtures.
+        self.useFixture(policy_fixture.RealPolicyFixture())
+        self.useFixture(nova_fixtures.NeutronFixture(self))
+        self.useFixture(func_fixtures.PlacementFixture())
+        # Setup API.
+        api_fixture = self.useFixture(nova_fixtures.OSAPIFixture(
+            api_version='v2.1'))
+        self.api = api_fixture.api
+        self.admin_api = api_fixture.admin_api
+        # Fake out glance.
+        nova.tests.unit.image.fake.stub_out_image_service(self)
+        self.addCleanup(nova.tests.unit.image.fake.FakeImageService_reset)
+        # Start conductor, scheduler and two computes.
+        self.start_service('conductor')
+        self.start_service('scheduler')
+        self.addCleanup(fake.restore_nodes)
+        for host in ('host1', 'host2'):
+            fake.set_nodes([host])
+            self.start_service('compute', host=host)
+
+    def test_serial_no_valid_host_then_pass_with_third_host(self):
+        """Creates 2 servers in order (not a multi-create request) in an
+        anti-affinity group so there will be 1 server on each host. Then
+        attempts to live migrate the first server which will fail because the
+        only other available host will be full. Then starts up a 3rd compute
+        service and retries the live migration which should then pass.
+        """
+        # Create the anti-affinity group used for the servers.
+        group = self.api.post_server_groups(
+            {'name': 'test_serial_no_valid_host_then_pass_with_third_host',
+             'policies': ['anti-affinity']})
+        servers = []
+        for x in range(2):
+            server = self._build_minimal_create_server_request(
+                self.api,
+                'test_serial_no_valid_host_then_pass_with_third_host-%d' % x,
+                networks='none')
+            # Add the group hint so the server is created in our group.
+            server_req = {
+                'server': server,
+                'os:scheduler_hints': {'group': group['id']}
+            }
+            # Use microversion 2.37 for passing networks='none'.
+            with utils.temporary_mutation(self.api, microversion='2.37'):
+                server = self.api.post_server(server_req)
+                servers.append(
+                    self._wait_for_state_change(
+                        self.admin_api, server, 'ACTIVE'))
+
+        # Make sure each server is on a unique host.
+        hosts = set([svr['OS-EXT-SRV-ATTR:host'] for svr in servers])
+        self.assertEqual(2, len(hosts))
+
+        # And make sure the group has 2 members.
+        members = self.api.get_server_group(group['id'])['members']
+        self.assertEqual(2, len(members))
+
+        # Now attempt to live migrate one of the servers which should fail
+        # because we don't have a free host. Since we're using microversion 2.1
+        # the scheduling will be synchronous and we should get back a 400
+        # response for the NoValidHost error.
+        body = {
+            'os-migrateLive': {
+                'host': None,
+                'block_migration': False,
+                'disk_over_commit': False
+            }
+        }
+        # Specifically use the first server since that was the first member
+        # added to the group.
+        server = servers[0]
+        ex = self.assertRaises(client.OpenStackApiException,
+                               self.admin_api.post_server_action,
+                               server['id'], body)
+        self.assertEqual(400, ex.response.status_code)
+        self.assertIn('No valid host', six.text_type(ex))
+
+        # Now start up a 3rd compute service and retry the live migration which
+        # should work this time.
+        fake.set_nodes(['host3'])
+        self.start_service('compute', host='host3')
+        self.admin_api.post_server_action(server['id'], body)
+        server = self._wait_for_state_change(self.admin_api, server, 'ACTIVE')
+        # Now the server should be on host3 since that was the only available
+        # host for the live migration.
+        self.assertEqual('host3', server['OS-EXT-SRV-ATTR:host'])
