@@ -18,14 +18,11 @@ import datetime
 from oslo_utils import fixture as utils_fixture
 
 from nova.api.openstack.compute import instance_usage_audit_log as v21_ial
-from nova.api.openstack.compute.legacy_v2.contrib \
-        import instance_usage_audit_log as ial
 from nova import context
 from nova import exception
 from nova import test
 from nova.tests.unit.api.openstack import fakes
 from nova.tests.unit.objects import test_service
-from nova import utils
 
 
 service_base = test_service.fake_service
@@ -123,10 +120,10 @@ class InstanceUsageAuditLogTestV21(test.NoDBTestCase):
             self.assertIsNone(disabled)
             return TEST_COMPUTE_SERVICES
 
-        self.stubs.Set(utils, 'last_completed_audit_period',
-                            fake_last_completed_audit_period)
-        self.stub_out('nova.db.service_get_all', fake_service_get_all)
-        self.stub_out('nova.db.task_log_get_all', fake_task_log_get_all)
+        self.stub_out('nova.utils.last_completed_audit_period',
+                      fake_last_completed_audit_period)
+        self.stub_out('nova.db.api.service_get_all', fake_service_get_all)
+        self.stub_out('nova.db.api.task_log_get_all', fake_task_log_get_all)
 
         self.req = fakes.HTTPRequest.blank('')
 
@@ -188,25 +185,6 @@ class InstanceUsageAuditLogTestV21(test.NoDBTestCase):
                          logs['overall_status'])
 
 
-class InstanceUsageAuditLogTest(InstanceUsageAuditLogTestV21):
-    def setUp(self):
-        super(InstanceUsageAuditLogTest, self).setUp()
-        self.req = fakes.HTTPRequest.blank('', use_admin_context=True)
-        self.non_admin_req = fakes.HTTPRequest.blank('')
-
-    def _set_up_controller(self):
-        self.controller = ial.InstanceUsageAuditLogController()
-
-    def test_index_non_admin(self):
-        self.assertRaises(exception.PolicyNotAuthorized,
-                          self.controller.index, self.non_admin_req)
-
-    def test_show_non_admin(self):
-        self.assertRaises(exception.PolicyNotAuthorized,
-                          self.controller.show, self.non_admin_req,
-                          '2012-07-05 10:00:00')
-
-
 class InstanceUsageAuditPolicyEnforcementV21(test.NoDBTestCase):
 
     def setUp(self):
