@@ -17,6 +17,7 @@ import copy
 import datetime
 
 import mock
+from oslo_utils.fixture import uuidsentinel as uuids
 import six
 import webob
 
@@ -26,7 +27,7 @@ from nova import objects
 from nova.objects import base
 from nova import test
 from nova.tests.unit.api.openstack import fakes
-from nova.tests import uuidsentinel as uuids
+
 
 SERVER_UUID = uuids.server_uuid
 
@@ -179,7 +180,8 @@ class ServerMigrationsTestsV223(ServerMigrationsTestsV21):
         self.assertEqual(migrations_in_progress, response)
 
         m_get_instance.assert_called_once_with(self.context, SERVER_UUID,
-                                               expected_attrs=None)
+                                               expected_attrs=None,
+                                               cell_down_support=False)
 
     @mock.patch('nova.compute.api.API.get')
     def test_index_invalid_instance(self, m_get_instance):
@@ -189,7 +191,8 @@ class ServerMigrationsTestsV223(ServerMigrationsTestsV21):
                           self.req, SERVER_UUID)
 
         m_get_instance.assert_called_once_with(self.context, SERVER_UUID,
-                                               expected_attrs=None)
+                                               expected_attrs=None,
+                                               cell_down_support=False)
 
     @mock.patch('nova.compute.api.API.get_migration_by_id_and_instance')
     @mock.patch('nova.compute.api.API.get')
@@ -201,7 +204,8 @@ class ServerMigrationsTestsV223(ServerMigrationsTestsV21):
         self.assertEqual(migrations[0], response['migration'])
 
         m_get_instance.assert_called_once_with(self.context, SERVER_UUID,
-                                               expected_attrs=None)
+                                               expected_attrs=None,
+                                               cell_down_support=False)
 
     @mock.patch('nova.compute.api.API.get_migration_by_id_and_instance')
     @mock.patch('nova.compute.api.API.get')
@@ -215,7 +219,8 @@ class ServerMigrationsTestsV223(ServerMigrationsTestsV21):
                           non_progress_mig.id)
 
         m_get_instance.assert_called_once_with(self.context, SERVER_UUID,
-                                               expected_attrs=None)
+                                               expected_attrs=None,
+                                               cell_down_support=False)
 
     @mock.patch('nova.compute.api.API.get_migration_by_id_and_instance')
     @mock.patch('nova.compute.api.API.get')
@@ -230,7 +235,8 @@ class ServerMigrationsTestsV223(ServerMigrationsTestsV21):
                           non_progress_mig.id)
 
         m_get_instance.assert_called_once_with(self.context, SERVER_UUID,
-                                               expected_attrs=None)
+                                               expected_attrs=None,
+                                               cell_down_support=False)
 
     @mock.patch('nova.compute.api.API.get_migration_by_id_and_instance')
     @mock.patch('nova.compute.api.API.get')
@@ -244,7 +250,8 @@ class ServerMigrationsTestsV223(ServerMigrationsTestsV21):
                           migrations_obj[0].id)
 
         m_get_instance.assert_called_once_with(self.context, SERVER_UUID,
-                                               expected_attrs=None)
+                                               expected_attrs=None,
+                                               cell_down_support=False)
 
     @mock.patch('nova.compute.api.API.get')
     def test_show_migration_invalid_instance(self, m_get_instance):
@@ -255,7 +262,8 @@ class ServerMigrationsTestsV223(ServerMigrationsTestsV21):
                           migrations_obj[0].id)
 
         m_get_instance.assert_called_once_with(self.context, SERVER_UUID,
-                                               expected_attrs=None)
+                                               expected_attrs=None,
+                                               cell_down_support=False)
 
 
 class ServerMigrationsTestsV224(ServerMigrationsTestsV21):
