@@ -19,6 +19,7 @@ import datetime
 import iso8601
 import mock
 from oslo_utils import fixture as utils_fixture
+from oslo_utils.fixture import uuidsentinel
 import six
 import webob.exc
 
@@ -35,7 +36,7 @@ from nova import test
 from nova.tests import fixtures
 from nova.tests.unit.api.openstack import fakes
 from nova.tests.unit.objects import test_service
-from nova.tests import uuidsentinel
+
 
 # This is tied into the os-services API samples functional tests.
 FAKE_UUID_COMPUTE_HOST1 = 'e81d66a4-ddd3-4aba-8a84-171d1cb4d339'
@@ -120,7 +121,7 @@ fake_services_list = [
 
 def fake_service_get_all(services):
     def service_get_all(context, filters=None, set_zones=False,
-                        all_cells=False):
+                        all_cells=False, cell_down_support=False):
         if set_zones or 'availability_zone' in filters:
             return availability_zones.set_availability_zones(context,
                                                              services)
@@ -712,33 +713,25 @@ class ServicesTestV21(test.TestCase):
         """Tests that we are still able to successfully delete a nova-compute
         service even if the HostMapping is not found.
         """
-        @mock.patch('nova.objects.ComputeNodeList.get_all_by_host',
-                    return_value=objects.ComputeNodeList(objects=[
-                        objects.ComputeNode(host='host1',
-                                            hypervisor_hostname='node1'),
-                        objects.ComputeNode(host='host1',
-                                            hypervisor_hostname='node2')]))
         @mock.patch.object(self.controller.host_api, 'service_get_by_id',
                            return_value=objects.Service(
-                               host='host1', binary='nova-compute'))
+                               host='host1', binary='nova-compute',
+                               compute_node=objects.ComputeNode()))
         @mock.patch.object(self.controller.aggregate_api,
                            'get_aggregates_by_host',
                            return_value=objects.AggregateList())
         @mock.patch.object(self.controller.placementclient,
                            'delete_resource_provider')
         def _test(delete_resource_provider,
-                  get_aggregates_by_host, service_get_by_id,
-                  cn_get_all_by_host):
+                  get_aggregates_by_host, service_get_by_id):
             self.controller.delete(self.req, 2)
             ctxt = self.req.environ['nova.context']
             service_get_by_id.assert_called_once_with(ctxt, 2)
             get_instances.assert_called_once_with(ctxt, 'host1')
             get_aggregates_by_host.assert_called_once_with(ctxt, 'host1')
-            self.assertEqual(2, delete_resource_provider.call_count)
-            nodes = cn_get_all_by_host.return_value
-            delete_resource_provider.assert_has_calls([
-                mock.call(ctxt, node, cascade=True) for node in nodes
-            ], any_order=True)
+            delete_resource_provider.assert_called_once_with(
+                ctxt, service_get_by_id.return_value.compute_node,
+                cascade=True)
             get_hm.assert_called_once_with(ctxt, 'host1')
             service_delete.assert_called_once_with()
         _test()
