@@ -13,11 +13,20 @@
 #    License for the specific language governing permissions and limitations
 #    under the License.
 
+import fixtures
+
+
 from nova.compute import resource_tracker
 
 
 class FakeResourceTracker(resource_tracker.ResourceTracker):
     """Version without a DB requirement."""
 
-    def _update(self, context, compute_node):
+    def _update(self, context, compute_node, startup=False):
         pass
+
+
+class RTMockMixin(object):
+    def _mock_rt(self, **kwargs):
+        return self.useFixture(fixtures.MockPatchObject(
+            self.compute, 'rt', **kwargs)).mock
