@@ -17,9 +17,13 @@
 """Tests For miscellaneous util methods used with compute."""
 
 import copy
+import datetime
 import string
+import traceback
 
 import mock
+from oslo_serialization import jsonutils
+from oslo_utils.fixture import uuidsentinel as uuids
 from oslo_utils import uuidutils
 import six
 
@@ -28,6 +32,7 @@ from nova.compute import manager
 from nova.compute import power_state
 from nova.compute import task_states
 from nova.compute import utils as compute_utils
+from nova.compute import vm_states
 import nova.conf
 from nova import context
 from nova import exception
@@ -45,15 +50,13 @@ from nova.tests.unit import fake_instance
 from nova.tests.unit import fake_network
 from nova.tests.unit import fake_notifier
 from nova.tests.unit import fake_server_actions
-import nova.tests.unit.image.fake
 from nova.tests.unit.objects import test_flavor
-from nova.tests import uuidsentinel as uuids
 
 
-CONF = nova.conf.CONF
-
 FAKE_IMAGE_REF = uuids.image_ref
 
+CONF = nova.conf.CONF
+
 
 def create_instance(context, user_id='fake', project_id='fake', params=None):
     """Create a test instance."""
@@ -427,7 +430,7 @@ class UsageInfoTestCase(test.TestCase):
         instance.system_metadata.update(sys_metadata)
         instance.save()
         compute_utils.notify_usage_exists(
-            rpc.get_notifier('compute'), self.context, instance)
+            rpc.get_notifier('compute'), self.context, instance, 'fake-host')
         self.assertEqual(len(fake_notifier.NOTIFICATIONS), 1)
         msg = fake_notifier.NOTIFICATIONS[0]
         self.assertEqual(msg.priority, 'INFO')
@@ -448,12 +451,39 @@ class UsageInfoTestCase(test.TestCase):
             self.assertIn(attr, payload,
                           "Key %s not in payload" % attr)
         self.assertEqual(payload['image_meta'],
-                {'md_key1': 'val1', 'md_key2': 'val2'})
+                         {'md_key1': 'val1', 'md_key2': 'val2'})
         image_ref_url = "%s/images/%s" % (
             glance.generate_glance_url(self.context), uuids.fake_image_ref)
         self.assertEqual(payload['image_ref_url'], image_ref_url)
         self.compute.terminate_instance(self.context, instance, [])
 
+    def test_notify_usage_exists_emits_versioned(self):
+        # Ensure 'exists' notification generates appropriate usage data.
+        instance = create_instance(self.context)
+
+        compute_utils.notify_usage_exists(
+            rpc.get_notifier('compute'), self.context, instance, 'fake-host')
+        self.assertEqual(len(fake_notifier.VERSIONED_NOTIFICATIONS), 1)
+        msg = fake_notifier.VERSIONED_NOTIFICATIONS[0]
+        self.assertEqual(msg['priority'], 'INFO')
+        self.assertEqual(msg['event_type'], 'instance.exists')
+        payload = msg['payload']['nova_object.data']
+        self.assertEqual(payload['tenant_id'], self.project_id)
+        self.assertEqual(payload['user_id'], self.user_id)
+        self.assertEqual(payload['uuid'], instance['uuid'])
+        flavor = payload['flavor']['nova_object.data']
+        self.assertEqual(flavor['name'], 'm1.tiny')
+        flavorid = flavors.get_flavor_by_name('m1.tiny')['flavorid']
+        self.assertEqual(str(flavor['flavorid']), str(flavorid))
+
+        for attr in ('display_name', 'created_at', 'launched_at',
+                     'state', 'bandwidth', 'audit_period'):
+            self.assertIn(attr, payload,
+                          "Key %s not in payload" % attr)
+
+        self.assertEqual(payload['image_uuid'], uuids.fake_image_ref)
+        self.compute.terminate_instance(self.context, instance, [])
+
     def test_notify_usage_exists_deleted_instance(self):
         # Ensure 'exists' notification generates appropriate usage data.
         instance = create_instance(self.context)
@@ -465,7 +495,7 @@ class UsageInfoTestCase(test.TestCase):
         instance.save()
         self.compute.terminate_instance(self.context, instance, [])
         compute_utils.notify_usage_exists(
-            rpc.get_notifier('compute'), self.context, instance)
+            rpc.get_notifier('compute'), self.context, instance, 'fake-host')
         msg = fake_notifier.NOTIFICATIONS[-1]
         self.assertEqual(msg.priority, 'INFO')
         self.assertEqual(msg.event_type, 'compute.instance.exists')
@@ -669,7 +699,6 @@ class UsageInfoTestCase(test.TestCase):
 
         compute_utils.notify_about_volume_swap(
             self.context, instance, 'fake-compute',
-            fields.NotificationAction.VOLUME_SWAP,
             fields.NotificationPhase.START,
             uuids.old_volume_id, uuids.new_volume_id)
 
@@ -709,11 +738,11 @@ class UsageInfoTestCase(test.TestCase):
             # To get exception trace, raise and catch an exception
             raise test.TestingException('Volume swap error.')
         except Exception as ex:
+            tb = traceback.format_exc()
             compute_utils.notify_about_volume_swap(
                 self.context, instance, 'fake-compute',
-                fields.NotificationAction.VOLUME_SWAP,
                 fields.NotificationPhase.ERROR,
-                uuids.old_volume_id, uuids.new_volume_id, ex)
+                uuids.old_volume_id, uuids.new_volume_id, ex, tb)
 
         self.assertEqual(len(fake_notifier.VERSIONED_NOTIFICATIONS), 1)
         notification = fake_notifier.VERSIONED_NOTIFICATIONS[0]
@@ -753,6 +782,8 @@ class UsageInfoTestCase(test.TestCase):
                          exception_payload['function_name'])
         self.assertEqual('nova.tests.unit.compute.test_compute_utils',
                          exception_payload['module_name'])
+        self.assertIn('test_notify_about_volume_swap_with_error',
+                      exception_payload['traceback'])
 
     def test_notify_about_instance_rescue_action(self):
         instance = create_instance(self.context)
@@ -762,7 +793,6 @@ class UsageInfoTestCase(test.TestCase):
             instance,
             'fake-compute',
             uuids.rescue_image_ref,
-            action='rescue',
             phase='start')
 
         self.assertEqual(len(fake_notifier.VERSIONED_NOTIFICATIONS), 1)
@@ -830,7 +860,7 @@ class UsageInfoTestCase(test.TestCase):
         instance = create_instance(self.context)
         self.compute.terminate_instance(self.context, instance, [])
         compute_utils.notify_usage_exists(
-            rpc.get_notifier('compute'), self.context, instance)
+            rpc.get_notifier('compute'), self.context, instance, 'fake-host')
         msg = fake_notifier.NOTIFICATIONS[-1]
         self.assertEqual(msg.priority, 'INFO')
         self.assertEqual(msg.event_type, 'compute.instance.exists')
@@ -853,6 +883,44 @@ class UsageInfoTestCase(test.TestCase):
             glance.generate_glance_url(self.context), uuids.fake_image_ref)
         self.assertEqual(payload['image_ref_url'], image_ref_url)
 
+    def test_notify_about_volume_usage(self):
+        # Ensure 'volume.usage' notification generates appropriate usage data.
+        vol_usage = objects.VolumeUsage(
+            id=1, volume_id=uuids.volume, instance_uuid=uuids.instance,
+            project_id=self.project_id, user_id=self.user_id,
+            availability_zone='AZ1',
+            tot_last_refreshed=datetime.datetime(second=1, minute=1, hour=1,
+                                                 day=5, month=7, year=2018),
+            tot_reads=100, tot_read_bytes=100,
+            tot_writes=100, tot_write_bytes=100,
+            curr_last_refreshed=datetime.datetime(second=1, minute=1, hour=2,
+                                                  day=5, month=7, year=2018),
+            curr_reads=100, curr_read_bytes=100,
+            curr_writes=100, curr_write_bytes=100)
+
+        compute_utils.notify_about_volume_usage(self.context, vol_usage,
+                                                'fake-compute')
+
+        self.assertEqual(1, len(fake_notifier.VERSIONED_NOTIFICATIONS))
+        notification = fake_notifier.VERSIONED_NOTIFICATIONS[0]
+
+        self.assertEqual('INFO', notification['priority'])
+        self.assertEqual('volume.usage', notification['event_type'])
+        self.assertEqual('nova-compute:fake-compute',
+                         notification['publisher_id'])
+
+        payload = notification['payload']['nova_object.data']
+        self.assertEqual(uuids.volume, payload['volume_id'])
+        self.assertEqual(uuids.instance, payload['instance_uuid'])
+        self.assertEqual(self.project_id, payload['project_id'])
+        self.assertEqual(self.user_id, payload['user_id'])
+        self.assertEqual('AZ1', payload['availability_zone'])
+        self.assertEqual('2018-07-05T02:01:01Z', payload['last_refreshed'])
+        self.assertEqual(200, payload['read_bytes'])
+        self.assertEqual(200, payload['reads'])
+        self.assertEqual(200, payload['write_bytes'])
+        self.assertEqual(200, payload['writes'])
+
     def test_notify_about_instance_usage(self):
         instance = create_instance(self.context)
         # Set some system metadata
@@ -998,6 +1066,19 @@ class ComputeUtilsTestCase(test.NoDBTestCase):
         self.context = context.RequestContext(self.user_id,
                                               self.project_id)
 
+    @mock.patch.object(compute_utils, 'EventReporter')
+    def test_wrap_instance_event_without_host(self, mock_event):
+        inst = objects.Instance(uuid=uuids.instance)
+
+        @compute_utils.wrap_instance_event(prefix='compute')
+        def fake_event(self, context, instance):
+            pass
+
+        fake_event(self.compute, self.context, instance=inst)
+        # if the class doesn't include a self.host, the default host is None
+        mock_event.assert_called_once_with(self.context, 'compute_fake_event',
+                                           None, uuids.instance)
+
     @mock.patch.object(objects.InstanceActionEvent, 'event_start')
     @mock.patch.object(objects.InstanceActionEvent,
                        'event_finish_with_failure')
@@ -1059,10 +1140,12 @@ class ComputeUtilsTestCase(test.NoDBTestCase):
             self.assertEqual([], addresses)
         mock_ifaddresses.assert_called_once_with(iface)
 
+    @mock.patch('nova.compute.utils.notify_about_instance_action')
     @mock.patch('nova.compute.utils.notify_about_instance_usage')
     @mock.patch('nova.objects.Instance.destroy')
     def test_notify_about_instance_delete(self, mock_instance_destroy,
-                                          mock_notify_usage):
+                                          mock_notify_usage,
+                                          mock_notify_action):
         instance = fake_instance.fake_instance_obj(
             self.context, expected_attrs=('system_metadata',))
         with compute_utils.notify_about_instance_delete(
@@ -1072,9 +1155,64 @@ class ComputeUtilsTestCase(test.NoDBTestCase):
             mock.call(mock.sentinel.notifier, self.context, instance,
                       'delete.start'),
             mock.call(mock.sentinel.notifier, self.context, instance,
-                      'delete.end', system_metadata=instance.system_metadata)
+                      'delete.end')
         ]
         mock_notify_usage.assert_has_calls(expected_notify_calls)
+        mock_notify_action.assert_has_calls([
+            mock.call(self.context, instance,
+                      host='fake-mini', source='nova-api',
+                      action='delete', phase='start'),
+            mock.call(self.context, instance,
+                      host='fake-mini', source='nova-api',
+                      action='delete', phase='end'),
+        ])
+
+    def test_get_stashed_volume_connector_none(self):
+        inst = fake_instance.fake_instance_obj(self.context)
+        # connection_info isn't set
+        bdm = objects.BlockDeviceMapping(self.context)
+        self.assertIsNone(
+            compute_utils.get_stashed_volume_connector(bdm, inst))
+        # connection_info is None
+        bdm.connection_info = None
+        self.assertIsNone(
+            compute_utils.get_stashed_volume_connector(bdm, inst))
+        # connector is not set in connection_info
+        bdm.connection_info = jsonutils.dumps({})
+        self.assertIsNone(
+            compute_utils.get_stashed_volume_connector(bdm, inst))
+        # connector is set but different host
+        conn_info = {'connector': {'host': 'other_host'}}
+        bdm.connection_info = jsonutils.dumps(conn_info)
+        self.assertIsNone(
+            compute_utils.get_stashed_volume_connector(bdm, inst))
+
+    def test_may_have_ports_or_volumes(self):
+        inst = objects.Instance()
+        for vm_state, expected_result in ((vm_states.ERROR, True),
+                                          (vm_states.SHELVED_OFFLOADED, True),
+                                          (vm_states.BUILDING, False)):
+            inst.vm_state = vm_state
+            self.assertEqual(
+                expected_result, compute_utils.may_have_ports_or_volumes(inst),
+                vm_state)
+
+    def test_heal_reqspec_is_bfv_no_update(self):
+        reqspec = objects.RequestSpec(is_bfv=False)
+        with mock.patch.object(compute_utils, 'is_volume_backed_instance',
+                               new_callable=mock.NonCallableMock):
+            compute_utils.heal_reqspec_is_bfv(
+                self.context, reqspec, mock.sentinel.instance)
+
+    @mock.patch('nova.objects.RequestSpec.save')
+    def test_heal_reqspec_is_bfv_with_update(self, mock_save):
+        reqspec = objects.RequestSpec()
+        with mock.patch.object(compute_utils, 'is_volume_backed_instance',
+                               return_value=True):
+            compute_utils.heal_reqspec_is_bfv(
+                self.context, reqspec, mock.sentinel.instance)
+        self.assertTrue(reqspec.is_bfv)
+        mock_save.assert_called_once_with()
 
 
 class ServerGroupTestCase(test.TestCase):
@@ -1085,18 +1223,19 @@ class ServerGroupTestCase(test.TestCase):
         self.user_id = 'fake'
         self.project_id = 'fake'
         self.context = context.RequestContext(self.user_id, self.project_id)
+        self.group = objects.InstanceGroup(context=self.context,
+                                           id=1,
+                                           uuid=uuids.server_group,
+                                           user_id=self.user_id,
+                                           project_id=self.project_id,
+                                           name="test-server-group",
+                                           policy="anti-affinity",
+                                           policies=["anti-affinity"],
+                                           rules={"max_server_per_host": 3})
 
     def test_notify_about_server_group_action(self):
-        uuid = uuids.instance
-        group = objects.InstanceGroup(context=self.context,
-                                      id=1,
-                                      uuid=uuid,
-                                      user_id=self.user_id,
-                                      project_id=self.project_id,
-                                      name="test-server-group",
-                                      policies=["anti-affinity"])
         compute_utils.notify_about_server_group_action(self.context,
-                                                       group, 'create')
+                                                       self.group, 'create')
         self.assertEqual(len(fake_notifier.VERSIONED_NOTIFICATIONS), 1)
         notification = fake_notifier.VERSIONED_NOTIFICATIONS[0]
         expected = {'priority': 'INFO',
@@ -1106,15 +1245,49 @@ class ServerGroupTestCase(test.TestCase):
                         'nova_object.data': {
                             'name': u'test-server-group',
                             'policies': [u'anti-affinity'],
+                            'policy': u'anti-affinity',
+                            'rules': {"max_server_per_host": "3"},
                             'project_id': u'fake',
                             'user_id': u'fake',
-                            'uuid': uuid,
+                            'uuid': uuids.server_group,
                             'hosts': None,
                             'members': None
                         },
                         'nova_object.name': 'ServerGroupPayload',
                         'nova_object.namespace': 'nova',
-                        'nova_object.version': '1.0'
+                        'nova_object.version': '1.1'
+                   }
+            }
+        self.assertEqual(notification, expected)
+
+    @mock.patch.object(objects.InstanceGroup, 'get_by_uuid')
+    def test_notify_about_server_group_add_member(self, mock_get_by_uuid):
+        self.group.members = [uuids.instance]
+        mock_get_by_uuid.return_value = self.group
+        compute_utils.notify_about_server_group_add_member(
+            self.context, uuids.server_group)
+        mock_get_by_uuid.assert_called_once_with(self.context,
+                                                 uuids.server_group)
+        self.assertEqual(len(fake_notifier.VERSIONED_NOTIFICATIONS), 1)
+        notification = fake_notifier.VERSIONED_NOTIFICATIONS[0]
+        expected = {'priority': 'INFO',
+                    'event_type': u'server_group.add_member',
+                    'publisher_id': u'nova-api:fake-mini',
+                    'payload': {
+                        'nova_object.data': {
+                            'name': u'test-server-group',
+                            'policies': [u'anti-affinity'],
+                            'policy': u'anti-affinity',
+                            'rules': {"max_server_per_host": "3"},
+                            'project_id': u'fake',
+                            'user_id': u'fake',
+                            'uuid': uuids.server_group,
+                            'hosts': None,
+                            'members': [uuids.instance]
+                        },
+                        'nova_object.name': 'ServerGroupPayload',
+                        'nova_object.namespace': 'nova',
+                        'nova_object.version': '1.1'
                    }
             }
         self.assertEqual(notification, expected)
@@ -1134,38 +1307,7 @@ class ComputeUtilsQuotaTestCase(test.TestCase):
             'ram': new_flavor['memory_mb'] - old_flavor['memory_mb']
         }
 
-        deltas = compute_utils.upsize_quota_delta(self.context, new_flavor,
-                                                  old_flavor)
-        self.assertEqual(expected_deltas, deltas)
-
-    def test_downsize_quota_delta(self):
-        inst = create_instance(self.context, params=None)
-        inst.old_flavor = flavors.get_flavor_by_name('m1.medium')
-        inst.new_flavor = flavors.get_flavor_by_name('m1.tiny')
-
-        expected_deltas = {
-            'cores': (inst.new_flavor['vcpus'] -
-                      inst.old_flavor['vcpus']),
-            'ram': (inst.new_flavor['memory_mb'] -
-                    inst.old_flavor['memory_mb'])
-        }
-
-        deltas = compute_utils.downsize_quota_delta(self.context, inst)
-        self.assertEqual(expected_deltas, deltas)
-
-    def test_reverse_quota_delta(self):
-        inst = create_instance(self.context, params=None)
-        inst.old_flavor = flavors.get_flavor_by_name('m1.tiny')
-        inst.new_flavor = flavors.get_flavor_by_name('m1.medium')
-
-        expected_deltas = {
-            'cores': -1 * (inst.new_flavor['vcpus'] -
-                           inst.old_flavor['vcpus']),
-            'ram': -1 * (inst.new_flavor['memory_mb'] -
-                         inst.old_flavor['memory_mb'])
-        }
-
-        deltas = compute_utils.reverse_upsize_quota_delta(self.context, inst)
+        deltas = compute_utils.upsize_quota_delta(new_flavor, old_flavor)
         self.assertEqual(expected_deltas, deltas)
 
     @mock.patch('nova.objects.Quotas.count_as_dict')
@@ -1312,3 +1454,65 @@ class IsVolumeBackedInstanceTestCase(test.TestCase):
         self.assertFalse(
             compute_utils.is_volume_backed_instance(ctxt, instance, None))
         mock_bdms.assert_called_with(ctxt, instance.uuid)
+
+
+class TestComputeNodeToInventoryDict(test.NoDBTestCase):
+    def test_compute_node_inventory(self):
+        uuid = uuids.compute_node
+        name = 'computehost'
+        compute_node = objects.ComputeNode(uuid=uuid,
+                                           hypervisor_hostname=name,
+                                           vcpus=2,
+                                           cpu_allocation_ratio=16.0,
+                                           memory_mb=1024,
+                                           ram_allocation_ratio=1.5,
+                                           local_gb=10,
+                                           disk_allocation_ratio=1.0)
+
+        self.flags(reserved_host_memory_mb=1000)
+        self.flags(reserved_host_disk_mb=200)
+        self.flags(reserved_host_cpus=1)
+
+        result = compute_utils.compute_node_to_inventory_dict(compute_node)
+
+        expected = {
+            'VCPU': {
+                'total': compute_node.vcpus,
+                'reserved': CONF.reserved_host_cpus,
+                'min_unit': 1,
+                'max_unit': compute_node.vcpus,
+                'step_size': 1,
+                'allocation_ratio': compute_node.cpu_allocation_ratio,
+            },
+            'MEMORY_MB': {
+                'total': compute_node.memory_mb,
+                'reserved': CONF.reserved_host_memory_mb,
+                'min_unit': 1,
+                'max_unit': compute_node.memory_mb,
+                'step_size': 1,
+                'allocation_ratio': compute_node.ram_allocation_ratio,
+            },
+            'DISK_GB': {
+                'total': compute_node.local_gb,
+                'reserved': 1,  # this is ceil(1000/1024)
+                'min_unit': 1,
+                'max_unit': compute_node.local_gb,
+                'step_size': 1,
+                'allocation_ratio': compute_node.disk_allocation_ratio,
+            },
+        }
+        self.assertEqual(expected, result)
+
+    def test_compute_node_inventory_empty(self):
+        uuid = uuids.compute_node
+        name = 'computehost'
+        compute_node = objects.ComputeNode(uuid=uuid,
+                                           hypervisor_hostname=name,
+                                           vcpus=0,
+                                           cpu_allocation_ratio=16.0,
+                                           memory_mb=0,
+                                           ram_allocation_ratio=1.5,
+                                           local_gb=0,
+                                           disk_allocation_ratio=1.0)
+        result = compute_utils.compute_node_to_inventory_dict(compute_node)
+        self.assertEqual({}, result)
