@@ -19,6 +19,7 @@ import copy
 import fixtures
 import mock
 from oslo_serialization import jsonutils
+from oslo_utils.fixture import uuidsentinel as uuids
 import testtools
 
 from nova.api.openstack.compute import services
@@ -34,7 +35,6 @@ from nova.tests.unit.api.openstack import fakes
 from nova.tests.unit import fake_notifier
 from nova.tests.unit.objects import test_objects
 from nova.tests.unit.objects import test_service
-from nova.tests import uuidsentinel as uuids
 
 
 class ComputeHostAPITestCase(test.TestCase):
@@ -192,13 +192,49 @@ class ComputeHostAPITestCase(test.TestCase):
                                   host='host-%s' % uuids.cell1)
         mock_sg.return_value = {
             uuids.cell1: [service],
-            uuids.cell2: context.raised_exception_sentinel
+            uuids.cell2: context.did_not_respond_sentinel
         }
         services = self.host_api.service_get_all(self.ctxt, all_cells=True)
         # returns the results from cell1 and ignores cell2.
         self.assertEqual(['host-%s' % uuids.cell1],
                          [svc.host for svc in services])
 
+    @mock.patch('nova.objects.CellMappingList.get_all')
+    @mock.patch.object(objects.HostMappingList, 'get_by_cell_id')
+    @mock.patch('nova.context.scatter_gather_all_cells')
+    def test_service_get_all_cells_with_minimal_constructs(self, mock_sg,
+                                                           mock_get_hm,
+                                                           mock_cm_list):
+        service = objects.Service(binary='nova-compute',
+                                  host='host-%s' % uuids.cell0)
+        cells = [
+            objects.CellMapping(uuid=uuids.cell1, id=1),
+            objects.CellMapping(uuid=uuids.cell2, id=2),
+        ]
+        mock_cm_list.return_value = cells
+        context.load_cells()
+        # create two hms in cell1, which is the down cell in this test.
+        hm1 = objects.HostMapping(self.ctxt, host='host1-unavailable',
+            cell_mapping=cells[0])
+        hm1.create()
+        hm2 = objects.HostMapping(self.ctxt, host='host2-unavailable',
+            cell_mapping=cells[0])
+        hm2.create()
+        mock_sg.return_value = {
+            cells[0].uuid: [service],
+            cells[1].uuid: context.did_not_respond_sentinel,
+        }
+        mock_get_hm.return_value = [hm1, hm2]
+        services = self.host_api.service_get_all(self.ctxt, all_cells=True,
+            cell_down_support=True)
+        # returns the results from cell0 and minimal construct from cell1.
+        self.assertEqual(sorted(['host-%s' % uuids.cell0, 'host1-unavailable',
+                         'host2-unavailable']),
+                         sorted([svc.host for svc in services]))
+        mock_sg.assert_called_once_with(self.ctxt, objects.ServiceList.get_all,
+                                        None, set_zones=False)
+        mock_get_hm.assert_called_once_with(self.ctxt, cells[1].id)
+
     def test_service_get_all_no_zones(self):
         services = [dict(test_service.fake_service,
                          id=1, topic='compute', host='host1'),
@@ -541,6 +577,10 @@ class ComputeHostAPICellsTestCase(ComputeHostAPITestCase):
     def test_service_get_all_cells_with_failures(self):
         pass
 
+    @testtools.skip('cellsv1 does not use this')
+    def test_service_get_all_cells_with_minimal_constructs(self):
+        pass
+
     @testtools.skip('cellsv1 does not use this')
     def test_service_delete_ambiguous(self):
         pass
@@ -721,8 +761,12 @@ class ComputeAggregateAPITestCase(test.TestCase):
             fixtures.MockPatch('nova.objects.HostMapping.get_by_host'))
         self.useFixture(
             fixtures.MockPatch('nova.context.set_target_cell'))
-        self.useFixture(
-            fixtures.MockPatch('nova.objects.Service.get_by_compute_host'))
+        mock_service_get_by_compute_host = (
+            self.useFixture(
+                fixtures.MockPatch(
+                    'nova.objects.Service.get_by_compute_host')).mock)
+        mock_service_get_by_compute_host.return_value = (
+            objects.Service(host='fake-host'))
 
     @mock.patch('nova.scheduler.client.report.SchedulerReportClient.'
                 'aggregate_add_host')
