@@ -19,6 +19,7 @@ Tests for Consoleauth Code.
 """
 
 import mock
+from oslo_utils.fixture import uuidsentinel as uuids
 from oslo_utils import timeutils
 import six
 
@@ -26,7 +27,6 @@ from nova.consoleauth import manager
 from nova import context
 from nova import objects
 from nova import test
-from nova.tests import uuidsentinel as uuids
 
 
 class ConsoleauthTestCase(test.NoDBTestCase):
@@ -88,17 +88,6 @@ class ConsoleauthTestCase(test.NoDBTestCase):
         self.stub_out(self.rpcapi + 'validate_console_port',
                       fake_validate_console_port)
 
-    @mock.patch('nova.consoleauth.manager.LOG.info')
-    def test_authorize_does_not_log_token_secrete(self, mock_info):
-        self.manager_api.authorize_console(
-            self.context, 'secret', 'novnc', '127.0.0.1', '8080', 'host',
-            self.instance_uuid)
-
-        mock_info.assert_called_once_with(
-            'Received Token: %(token_dict)s', test.MatchType(dict))
-        self.assertEqual(
-            '***', mock_info.mock_calls[0][1][1]['token_dict']['token'])
-
     @mock.patch('nova.objects.instance.Instance.get_by_uuid')
     def test_multiple_tokens_for_instance(self, mock_get):
         mock_get.return_value = None
@@ -150,9 +139,8 @@ class ConsoleauthTestCase(test.NoDBTestCase):
             mock_delete.assert_called_once_with(
                 self.instance_uuid.encode('UTF-8'))
 
-    @mock.patch('nova.consoleauth.manager.LOG.info')
     @mock.patch('nova.objects.instance.Instance.get_by_uuid')
-    def test_wrong_token_has_port(self, mock_get, mock_log):
+    def test_wrong_token_has_port(self, mock_get):
         mock_get.return_value = None
 
         token = u'mytok'
@@ -163,13 +151,6 @@ class ConsoleauthTestCase(test.NoDBTestCase):
                                         '127.0.0.1', '8080', 'host',
                                         instance_uuid=self.instance_uuid)
         self.assertIsNone(self.manager_api.check_token(self.context, token))
-        mock_log.assert_has_calls([
-            mock.call(
-                'Received Token: %(token_dict)s', mock.ANY),
-            mock.call(
-                'Checking that token is known: %(token_valid)s',
-                {'token_valid': True}),
-        ])
 
     def test_delete_expired_tokens(self):
         self.useFixture(test.TimeOverride())
