@@ -17,6 +17,7 @@ import mock
 from neutronclient.common import exceptions as n_exc
 from neutronclient.neutron import v2_0 as neutronv20
 from neutronclient.v2_0 import client
+from oslo_utils.fixture import uuidsentinel as uuids
 from six.moves import range
 
 from nova import context
@@ -24,7 +25,6 @@ from nova import exception
 from nova.network.security_group import neutron_driver
 from nova import objects
 from nova import test
-from nova.tests import uuidsentinel as uuids
 
 
 class TestNeutronDriver(test.NoDBTestCase):
@@ -272,7 +272,7 @@ class TestNeutronDriver(test.NoDBTestCase):
         self.assertEqual(expected, result)
         self.mocked_client.list_security_groups.assert_called_once_with()
 
-    def test_instances_security_group_bindings(self, detailed=False):
+    def test_instances_security_group_bindings(self):
         server_id = 'c5a20e8d-c4b0-47cf-9dca-ebe4f758acb1'
         port1_id = '4c505aec-09aa-47bc-bcc0-940477e84dc0'
         port2_id = 'b3b31a53-6e29-479f-ae5c-00b7b71a6d44'
@@ -288,47 +288,23 @@ class TestNeutronDriver(test.NoDBTestCase):
         sg2 = {'id': sg2_id, 'name': 'eor'}
         security_groups_list = {'security_groups': [sg1, sg2]}
 
+        sg_bindings = {server_id: [{'name': 'wol'}, {'name': 'eor'}]}
+
         self.mocked_client.list_ports.return_value = port_list
         self.mocked_client.list_security_groups.return_value = (
             security_groups_list)
 
-        def stub_convert(sg):
-            if sg == sg1:
-                return mock.sentinel.sg1
-            if sg == sg2:
-                return mock.sentinel.sg2
-            raise Exception("Unexpected security group in "
-                            "_convert_to_nova_security_group_format: %s" % sg)
-
         sg_api = neutron_driver.SecurityGroupAPI()
-        with mock.patch.object(
-                sg_api, '_convert_to_nova_security_group_format',
-                side_effect=stub_convert) as convert:
-            result = sg_api.get_instances_security_groups_bindings(
-                                      self.context, servers, detailed=detailed)
-            if detailed:
-                convert.assert_has_calls([mock.call(sg1), mock.call(sg2)],
-                                         any_order=True)
-                sg_bindings = {server_id: [
-                    mock.sentinel.sg1, mock.sentinel.sg2
-                ]}
-            else:
-                convert.assert_not_called()
-                sg_bindings = {server_id: [{'name': 'wol'}, {'name': 'eor'}]}
+        result = sg_api.get_instances_security_groups_bindings(
+                                  self.context, servers)
         self.assertEqual(sg_bindings, result)
         self.mocked_client.list_ports.assert_called_once_with(
             device_id=[server_id])
-        expected_search_opts = {'id': mock.ANY}
-        if not detailed:
-            expected_search_opts['fields'] = ['id', 'name']
         self.mocked_client.list_security_groups.assert_called_once_with(
-            **expected_search_opts)
+            id=mock.ANY)
         self.assertEqual(sorted([sg1_id, sg2_id]),
             sorted(self.mocked_client.list_security_groups.call_args[1]['id']))
 
-    def test_instances_security_group_bindings_detailed(self):
-        self.test_instances_security_group_bindings(detailed=True)
-
     def test_instances_security_group_bindings_port_not_found(self):
         server_id = 'c5a20e8d-c4b0-47cf-9dca-ebe4f758acb1'
         servers = [{'id': server_id}]
@@ -379,7 +355,7 @@ class TestNeutronDriver(test.NoDBTestCase):
                                   self.context, servers)
         self.assertEqual(sg_bindings, result)
         self.mocked_client.list_security_groups.assert_called_once_with(
-            id=mock.ANY, fields=['id', 'name'])
+            id=mock.ANY)
         self.assertEqual(sorted([sg1_id, sg2_id]),
             sorted(self.mocked_client.list_security_groups.call_args[1]['id']))
         self.assertEqual(expected_args,
@@ -416,7 +392,7 @@ class TestNeutronDriver(test.NoDBTestCase):
         self.mocked_client.list_ports.assert_called_once_with(
             device_id=['server_1'])
         self.mocked_client.list_security_groups.assert_called_once_with(
-            id=mock.ANY, fields=['id', 'name'])
+            id=mock.ANY)
         self.assertEqual(['1', '2'],
             sorted(self.mocked_client.list_security_groups.call_args[1]['id']))
 
