@@ -14,12 +14,16 @@
 # License for the specific language governing permissions and limitations
 # under the License.
 
+import copy
 import sys
 
 import fixtures as fx
 import mock
 from oslo_config import cfg
+from oslo_db import exception as db_exc
 from oslo_log import log as logging
+from oslo_utils.fixture import uuidsentinel as uuids
+from oslo_utils import timeutils
 from oslo_utils import uuidutils
 import sqlalchemy
 import testtools
@@ -32,8 +36,10 @@ from nova import exception
 from nova import objects
 from nova.objects import base as obj_base
 from nova.objects import service as service_obj
+from nova import test
 from nova.tests import fixtures
 from nova.tests.unit import conf_fixture
+from nova.tests.unit import fake_instance
 from nova import utils
 
 CONF = cfg.CONF
@@ -470,20 +476,139 @@ class TestSingleCellSimpleFixture(testtools.TestCase):
             self.assertIs(mock.sentinel.context, c)
 
 
-class TestPlacementFixture(testtools.TestCase):
-    def test_responds_to_version(self):
-        """Ensure the Placement server responds to calls sensibly."""
-        placement_fixture = self.useFixture(fixtures.PlacementFixture())
+class TestWarningsFixture(test.TestCase):
+    def test_invalid_uuid_errors(self):
+        """Creating an oslo.versionedobject with an invalid UUID value for a
+        UUIDField should raise an exception.
+        """
+        valid_migration_kwargs = {
+                "created_at": timeutils.utcnow().replace(microsecond=0),
+                "updated_at": None,
+                "deleted_at": None,
+                "deleted": False,
+                "id": 123,
+                "uuid": uuids.migration,
+                "source_compute": "compute-source",
+                "dest_compute": "compute-dest",
+                "source_node": "node-source",
+                "dest_node": "node-dest",
+                "dest_host": "host-dest",
+                "old_instance_type_id": 42,
+                "new_instance_type_id": 84,
+                "instance_uuid": "fake-uuid",
+                "status": "migrating",
+                "migration_type": "resize",
+                "hidden": False,
+                "memory_total": 123456,
+                "memory_processed": 12345,
+                "memory_remaining": 111111,
+                "disk_total": 234567,
+                "disk_processed": 23456,
+                "disk_remaining": 211111,
+                }
 
-        # request the API root, which provides us the versions of the API
-        resp = placement_fixture._fake_get(None, '/')
-        self.assertEqual(200, resp.status_code)
+        # this shall not throw FutureWarning
+        objects.migration.Migration(**valid_migration_kwargs)
 
-        # request a known bad url, and we should get a 404
-        resp = placement_fixture._fake_get(None, '/foo')
-        self.assertEqual(404, resp.status_code)
+        invalid_migration_kwargs = copy.deepcopy(valid_migration_kwargs)
+        invalid_migration_kwargs["uuid"] = "fake_id"
+        self.assertRaises(FutureWarning, objects.migration.Migration,
+                          **invalid_migration_kwargs)
 
-        # unsets the token so we fake missing it
-        placement_fixture.token = None
-        resp = placement_fixture._fake_get(None, '/foo')
-        self.assertEqual(401, resp.status_code)
+
+class TestDownCellFixture(test.TestCase):
+
+    def test_fixture(self):
+        # The test setup creates two cell mappings (cell0 and cell1) by
+        # default. Let's first list servers across all cells while they are
+        # "up" to make sure that works as expected. We'll create a single
+        # instance in cell1.
+        ctxt = context.get_admin_context()
+        cell1 = self.cell_mappings[test.CELL1_NAME]
+        with context.target_cell(ctxt, cell1) as cctxt:
+            inst = fake_instance.fake_instance_obj(cctxt)
+            if 'id' in inst:
+                delattr(inst, 'id')
+            inst.create()
+
+        # Now list all instances from all cells (should get one back).
+        results = context.scatter_gather_all_cells(
+            ctxt, objects.InstanceList.get_all)
+        self.assertEqual(2, len(results))
+        self.assertEqual(0, len(results[objects.CellMapping.CELL0_UUID]))
+        self.assertEqual(1, len(results[cell1.uuid]))
+
+        # Now do the same but with the DownCellFixture which should result
+        # in exception results from both cells.
+        with fixtures.DownCellFixture():
+            results = context.scatter_gather_all_cells(
+                ctxt, objects.InstanceList.get_all)
+            self.assertEqual(2, len(results))
+            for result in results.values():
+                self.assertIsInstance(result, db_exc.DBError)
+
+    def test_fixture_when_explicitly_passing_down_cell_mappings(self):
+        # The test setup creates two cell mappings (cell0 and cell1) by
+        # default. We'll create one instance per cell and pass cell0 as
+        # the down cell. We should thus get db_exc.DBError for cell0 and
+        # correct InstanceList object from cell1.
+        ctxt = context.get_admin_context()
+        cell0 = self.cell_mappings['cell0']
+        cell1 = self.cell_mappings['cell1']
+        with context.target_cell(ctxt, cell0) as cctxt:
+            inst1 = fake_instance.fake_instance_obj(cctxt)
+            if 'id' in inst1:
+                delattr(inst1, 'id')
+            inst1.create()
+        with context.target_cell(ctxt, cell1) as cctxt:
+            inst2 = fake_instance.fake_instance_obj(cctxt)
+            if 'id' in inst2:
+                delattr(inst2, 'id')
+            inst2.create()
+        with fixtures.DownCellFixture([cell0]):
+            results = context.scatter_gather_all_cells(
+                ctxt, objects.InstanceList.get_all)
+            self.assertEqual(2, len(results))
+            for cell_uuid, result in results.items():
+                if cell_uuid == cell0.uuid:
+                    self.assertIsInstance(result, db_exc.DBError)
+                else:
+                    self.assertIsInstance(result, objects.InstanceList)
+                    self.assertEqual(1, len(result))
+                    self.assertEqual(inst2.uuid, result[0].uuid)
+
+    def test_fixture_for_an_individual_down_cell_targeted_call(self):
+        # We have cell0 and cell1 by default in the setup. We try targeting
+        # both the cells. We should get a db error for the down cell and
+        # the correct result for the up cell.
+        ctxt = context.get_admin_context()
+        cell0 = self.cell_mappings['cell0']
+        cell1 = self.cell_mappings['cell1']
+        with context.target_cell(ctxt, cell0) as cctxt:
+            inst1 = fake_instance.fake_instance_obj(cctxt)
+            if 'id' in inst1:
+                delattr(inst1, 'id')
+            inst1.create()
+        with context.target_cell(ctxt, cell1) as cctxt:
+            inst2 = fake_instance.fake_instance_obj(cctxt)
+            if 'id' in inst2:
+                delattr(inst2, 'id')
+            inst2.create()
+
+        def dummy_tester(ctxt, cell_mapping, uuid):
+            with context.target_cell(ctxt, cell_mapping) as cctxt:
+                return objects.Instance.get_by_uuid(cctxt, uuid)
+
+        # Scenario A: We do not pass any down cells, fixture automatically
+        # assumes the targeted cell is down whether its cell0 or cell1.
+        with fixtures.DownCellFixture():
+            self.assertRaises(
+                db_exc.DBError, dummy_tester, ctxt, cell1, inst2.uuid)
+        # Scenario B: We pass cell0 as the down cell.
+        with fixtures.DownCellFixture([cell0]):
+            self.assertRaises(
+                db_exc.DBError, dummy_tester, ctxt, cell0, inst1.uuid)
+            # Scenario C: We get the correct result from the up cell
+            # when targeted.
+            result = dummy_tester(ctxt, cell1, inst2.uuid)
+            self.assertEqual(inst2.uuid, result.uuid)
