@@ -34,6 +34,7 @@ from oslo_config import cfg
 from oslo_serialization import base64
 from oslo_serialization import jsonutils
 from oslo_utils import encodeutils
+from oslo_utils.fixture import uuidsentinel as uuids
 import requests
 import six
 import webob
@@ -54,8 +55,7 @@ from nova import test
 from nova.tests.unit.api.openstack import fakes
 from nova.tests.unit import fake_block_device
 from nova.tests.unit import fake_network
-from nova.tests.unit import test_identity
-from nova.tests import uuidsentinel as uuids
+from nova.tests.unit import fake_requests
 from nova import utils
 from nova.virt import netutils
 
@@ -168,6 +168,12 @@ def fake_metadata_objects():
         mac='00:00:00:00:00:00',
         tags=['foo']
     )
+    nic_vf_trusted_obj = metadata_obj.NetworkInterfaceMetadata(
+        bus=metadata_obj.PCIDeviceBus(address='0000:00:02.0'),
+        mac='00:11:22:33:44:55',
+        vf_trusted=True,
+        tags=['trusted']
+    )
     nic_vlans_obj = metadata_obj.NetworkInterfaceMetadata(
         bus=metadata_obj.PCIDeviceBus(address='0000:80:01.0'),
         mac='e3:a0:d0:12:c5:10',
@@ -200,11 +206,12 @@ def fake_metadata_objects():
     mdlist = metadata_obj.InstanceDeviceMetadata(
         instance_uuid='b65cee2f-8c69-4aeb-be2f-f79742548fc2',
         devices=[nic_obj, ide_disk_obj, scsi_disk_obj, usb_disk_obj,
-                 fake_device_obj, device_with_fake_bus_obj, nic_vlans_obj])
+                 fake_device_obj, device_with_fake_bus_obj, nic_vlans_obj,
+                 nic_vf_trusted_obj])
     return mdlist
 
 
-def fake_metadata_dicts(include_vlan=False):
+def fake_metadata_dicts(include_vlan=False, include_vf_trusted=False):
     nic_meta = {
         'type': 'nic',
         'bus': 'pci',
@@ -219,6 +226,13 @@ def fake_metadata_dicts(include_vlan=False):
         'mac': 'e3:a0:d0:12:c5:10',
         'vlan': 1000,
     }
+    vf_trusted_nic_meta = {
+        'type': 'nic',
+        'bus': 'pci',
+        'address': '0000:00:02.0',
+        'mac': '00:11:22:33:44:55',
+        'tags': ['trusted'],
+    }
     ide_disk_meta = {
         'type': 'disk',
         'bus': 'ide',
@@ -236,9 +250,15 @@ def fake_metadata_dicts(include_vlan=False):
     usb_disk_meta['bus'] = 'usb'
     usb_disk_meta['address'] = '05c8:021e'
 
-    dicts = [nic_meta, ide_disk_meta, scsi_disk_meta, usb_disk_meta]
+    dicts = [nic_meta, ide_disk_meta, scsi_disk_meta, usb_disk_meta,
+             vf_trusted_nic_meta]
     if include_vlan:
-        dicts += [vlan_nic_meta]
+        # NOTE(artom) Yeah, the order is important.
+        dicts.insert(len(dicts) - 1, vlan_nic_meta)
+    if include_vf_trusted:
+        nic_meta['vf_trusted'] = False
+        vlan_nic_meta['vf_trusted'] = False
+        vf_trusted_nic_meta['vf_trusted'] = True
     return dicts
 
 
@@ -337,7 +357,7 @@ class MetadataTestCase(test.TestCase):
                      'delete_on_termination': None,
                      'device_name': '/dev/sdb'})]
 
-        self.stub_out('nova.db.block_device_mapping_get_all_by_instance',
+        self.stub_out('nova.db.api.block_device_mapping_get_all_by_instance',
                       fake_bdm_get)
 
         expected = {'ami': 'sda1',
@@ -478,6 +498,11 @@ class MetadataTestCase(test.TestCase):
             'openstack/2017-02-22/vendor_data.json',
             'openstack/2017-02-22/network_data.json',
             'openstack/2017-02-22/vendor_data2.json',
+            'openstack/2018-08-27/meta_data.json',
+            'openstack/2018-08-27/user_data',
+            'openstack/2018-08-27/vendor_data.json',
+            'openstack/2018-08-27/network_data.json',
+            'openstack/2018-08-27/vendor_data2.json',
             'openstack/latest/meta_data.json',
             'openstack/latest/user_data',
             'openstack/latest/vendor_data.json',
@@ -567,6 +592,10 @@ class MetadataTestCase(test.TestCase):
         if md._check_os_version(base.NEWTON_ONE, os_version):
             expose_vlan = md._check_os_version(base.OCATA, os_version)
             expected_metadata['devices'] = fake_metadata_dicts(expose_vlan)
+        if md._check_os_version(base.OCATA, os_version):
+            expose_trusted = md._check_os_version(base.ROCKY, os_version)
+            expected_metadata['devices'] = fake_metadata_dicts(
+                True, expose_trusted)
         mock_cells_keypair.return_value = keypair
         md._metadata_as_json(os_version, 'non useless path parameter')
         if instance.key_name:
@@ -648,7 +677,7 @@ class OpenStackMetadataTestCase(test.TestCase):
         mdinst = fake_InstanceMetadata(self, inst)
         mdjson = mdinst.lookup("/openstack/latest/meta_data.json")
         mddict = jsonutils.loads(mdjson)
-        self.assertEqual(fake_metadata_dicts(True), mddict['devices'])
+        self.assertEqual(fake_metadata_dicts(True, True), mddict['devices'])
 
     def test_top_level_listing(self):
         # request for /openstack/<version>/ should show metadata.json
@@ -856,9 +885,11 @@ class OpenStackMetadataTestCase(test.TestCase):
 
     def _test_vendordata2_response_inner(self, request_mock, response_code,
                                          include_rest_result=True):
-        fake_response = test_identity.FakeResponse(response_code)
+        content = None
         if include_rest_result:
-            fake_response.content = '{"color": "blue"}'
+            content = '{"color": "blue"}'
+        fake_response = fake_requests.FakeResponse(response_code,
+                                                   content=content)
         request_mock.return_value = fake_response
 
         with utils.tempdir() as tmpdir:
@@ -1586,6 +1617,23 @@ class MetadataHandlerTestCase(test.TestCase):
         mock_get_im.assert_called_once_with(ctxt, 'foo')
         imd.assert_called_once_with(inst, 'bar')
 
+    @mock.patch.object(objects.Instance, 'get_by_uuid')
+    @mock.patch.object(objects.InstanceMapping, 'get_by_instance_uuid')
+    def test_get_metadata_by_instance_id_with_local_meta(self, mock_get_im,
+                                                         mock_get_inst):
+        # Test that if local_metadata_per_cell is set to True, we don't
+        # query API DB for instance mapping.
+        self.flags(local_metadata_per_cell=True, group='api')
+        ctxt = context.RequestContext()
+        inst = objects.Instance()
+        mock_get_inst.return_value = inst
+
+        with mock.patch.object(base, 'InstanceMetadata') as imd:
+            base.get_metadata_by_instance_id('foo', 'bar', ctxt=ctxt)
+
+        mock_get_im.assert_not_called()
+        imd.assert_called_once_with(inst, 'bar')
+
 
 class MetadataPasswordTestCase(test.TestCase):
     def setUp(self):
@@ -1624,7 +1672,10 @@ class MetadataPasswordTestCase(test.TestCase):
 
     @mock.patch('nova.objects.InstanceMapping.get_by_instance_uuid')
     @mock.patch('nova.objects.Instance.get_by_uuid')
-    def _try_set_password(self, get_by_uuid, get_mapping, val=b'bar'):
+    def _try_set_password(self, get_by_uuid, get_mapping, val=b'bar',
+                          use_local_meta=False):
+        if use_local_meta:
+            self.flags(local_metadata_per_cell=True, group='api')
         request = webob.Request.blank('')
         request.method = 'POST'
         request.body = val
@@ -1636,12 +1687,19 @@ class MetadataPasswordTestCase(test.TestCase):
             save.assert_called_once_with()
 
         self.assertIn('password_0', self.instance.system_metadata)
-        get_mapping.assert_called_once_with(mock.ANY, self.instance.uuid)
+        if use_local_meta:
+            get_mapping.assert_not_called()
+        else:
+            get_mapping.assert_called_once_with(mock.ANY, self.instance.uuid)
 
     def test_set_password(self):
         self.mdinst.password = ''
         self._try_set_password()
 
+    def test_set_password_local_meta(self):
+        self.mdinst.password = ''
+        self._try_set_password(use_local_meta=True)
+
     def test_conflict(self):
         self.mdinst.password = 'foo'
         self.assertRaises(webob.exc.HTTPConflict,
