@@ -86,18 +86,26 @@ class QuotaIntegrationTestCase(test.TestCase):
         nova.tests.unit.image.fake.FakeImageService_reset()
 
     def _create_instance(self, flavor_name='m1.large'):
-        """Create a test instance."""
-        inst = objects.Instance(context=self.context)
-        inst.image_id = 'cedef40a-ed67-4d10-800e-17455edce175'
-        inst.reservation_id = 'r-fakeres'
-        inst.user_id = self.user_id
-        inst.project_id = self.project_id
-        inst.flavor = flavors.get_flavor_by_name(flavor_name)
-        # This is needed for instance quota counting until we have the
-        # ability to count allocations in placement.
-        inst.vcpus = inst.flavor.vcpus
-        inst.memory_mb = inst.flavor.memory_mb
-        inst.create()
+        """Create a test instance in cell1 with an instance mapping."""
+        cell1 = self.cell_mappings[test.CELL1_NAME]
+        with context.target_cell(self.context, cell1) as cctxt:
+            inst = objects.Instance(context=cctxt)
+            inst.image_id = 'cedef40a-ed67-4d10-800e-17455edce175'
+            inst.reservation_id = 'r-fakeres'
+            inst.user_id = self.user_id
+            inst.project_id = self.project_id
+            inst.flavor = flavors.get_flavor_by_name(flavor_name)
+            # This is needed for instance quota counting until we have the
+            # ability to count allocations in placement.
+            inst.vcpus = inst.flavor.vcpus
+            inst.memory_mb = inst.flavor.memory_mb
+            inst.create()
+        # Create the related instance mapping which will be used in
+        # _instances_cores_ram_count().
+        inst_map = objects.InstanceMapping(
+            self.context, instance_uuid=inst.uuid, project_id=inst.project_id,
+            cell_mapping=cell1)
+        inst_map.create()
         return inst
 
     def test_too_many_instances(self):
@@ -145,7 +153,8 @@ class QuotaIntegrationTestCase(test.TestCase):
         for i in range(3):
             self._create_instance(flavor_name='m1.xlarge')
 
-    def test_too_many_addresses(self):
+    @mock.patch('nova.privsep.linux_net.bind_ip')
+    def test_too_many_addresses(self, mock_bind_ip):
         # This test is specifically relying on nova-network.
         self.flags(use_neutron=False,
                    network_manager='nova.network.manager.FlatDHCPManager')
@@ -164,7 +173,8 @@ class QuotaIntegrationTestCase(test.TestCase):
                           self.project_id)
         db.floating_ip_destroy(context.get_admin_context(), address)
 
-    def test_auto_assigned(self):
+    @mock.patch('nova.privsep.linux_net.bind_ip')
+    def test_auto_assigned(self, mock_bind_ip):
         # This test is specifically relying on nova-network.
         self.flags(use_neutron=False,
                    network_manager='nova.network.manager.FlatDHCPManager')
@@ -281,51 +291,27 @@ class FakeDriver(object):
         self.by_class = by_class or {}
         self.reservations = reservations or []
 
-    def get_by_project_and_user(self, context, project_id, user_id, resource):
-        self.called.append(('get_by_project_and_user',
-                            context, project_id, user_id, resource))
-        try:
-            return self.by_user[user_id][resource]
-        except KeyError:
-            raise exception.ProjectUserQuotaNotFound(project_id=project_id,
-                                                     user_id=user_id)
-
-    def get_by_project(self, context, project_id, resource):
-        self.called.append(('get_by_project', context, project_id, resource))
-        try:
-            return self.by_project[project_id][resource]
-        except KeyError:
-            raise exception.ProjectQuotaNotFound(project_id=project_id)
-
-    def get_by_class(self, context, quota_class, resource):
-        self.called.append(('get_by_class', context, quota_class, resource))
-        try:
-            return self.by_class[quota_class][resource]
-        except KeyError:
-            raise exception.QuotaClassNotFound(class_name=quota_class)
-
     def get_defaults(self, context, resources):
         self.called.append(('get_defaults', context, resources))
         return resources
 
-    def get_class_quotas(self, context, resources, quota_class,
-                         defaults=True):
+    def get_class_quotas(self, context, resources, quota_class):
         self.called.append(('get_class_quotas', context, resources,
-                            quota_class, defaults))
+                            quota_class))
         return resources
 
     def get_user_quotas(self, context, resources, project_id, user_id,
-                        quota_class=None, defaults=True, usages=True):
+                        quota_class=None, usages=True):
         self.called.append(('get_user_quotas', context, resources,
-                            project_id, user_id, quota_class, defaults,
+                            project_id, user_id, quota_class,
                             usages))
         return resources
 
     def get_project_quotas(self, context, resources, project_id,
-                           quota_class=None, defaults=True, usages=True,
+                           quota_class=None, usages=True,
                            remains=False):
         self.called.append(('get_project_quotas', context, resources,
-                            project_id, quota_class, defaults, usages,
+                            project_id, quota_class, usages,
                             remains))
         return resources
 
@@ -340,13 +326,6 @@ class FakeDriver(object):
         self.called.append(('limit_check_project_and_user', context, resources,
                             project_values, user_values, project_id, user_id))
 
-    def destroy_all_by_project_and_user(self, context, project_id, user_id):
-        self.called.append(('destroy_all_by_project_and_user', context,
-                            project_id, user_id))
-
-    def destroy_all_by_project(self, context, project_id):
-        self.called.append(('destroy_all_by_project', context, project_id))
-
 
 class BaseResourceTestCase(test.TestCase):
     def test_no_flag(self):
@@ -373,77 +352,6 @@ class BaseResourceTestCase(test.TestCase):
         self.assertEqual(resource.flag, 'instances')
         self.assertEqual(resource.default, -1)
 
-    def test_quota_no_project_no_class(self):
-        self.flags(instances=10, group='quota')
-        resource = quota.BaseResource('test_resource', 'instances')
-        driver = FakeDriver()
-        context = FakeContext(None, None)
-        quota_value = resource.quota(driver, context)
-
-        self.assertEqual(quota_value, 10)
-
-    def test_quota_with_project_no_class(self):
-        self.flags(instances=10, group='quota')
-        resource = quota.BaseResource('test_resource', 'instances')
-        driver = FakeDriver(by_project=dict(
-                test_project=dict(test_resource=15),
-                ))
-        context = FakeContext('test_project', None)
-        quota_value = resource.quota(driver, context)
-
-        self.assertEqual(quota_value, 15)
-
-    def test_quota_no_project_with_class(self):
-        self.flags(instances=10, group='quota')
-        resource = quota.BaseResource('test_resource', 'instances')
-        driver = FakeDriver(by_class=dict(
-                test_class=dict(test_resource=20),
-                ))
-        context = FakeContext(None, 'test_class')
-        quota_value = resource.quota(driver, context)
-
-        self.assertEqual(quota_value, 20)
-
-    def test_quota_with_project_with_class(self):
-        self.flags(instances=10, group='quota')
-        resource = quota.BaseResource('test_resource', 'instances')
-        driver = FakeDriver(by_project=dict(
-                test_project=dict(test_resource=15),
-                ),
-                            by_class=dict(
-                test_class=dict(test_resource=20),
-                ))
-        context = FakeContext('test_project', 'test_class')
-        quota_value = resource.quota(driver, context)
-
-        self.assertEqual(quota_value, 15)
-
-    def test_quota_override_project_with_class(self):
-        self.flags(instances=10, group='quota')
-        resource = quota.BaseResource('test_resource', 'instances')
-        driver = FakeDriver(by_project=dict(
-                test_project=dict(test_resource=15),
-                override_project=dict(test_resource=20),
-                ))
-        context = FakeContext('test_project', 'test_class')
-        quota_value = resource.quota(driver, context,
-                                     project_id='override_project')
-
-        self.assertEqual(quota_value, 20)
-
-    def test_quota_with_project_override_class(self):
-        self.flags(instances=10, group='quota')
-        resource = quota.BaseResource('test_resource', 'instances')
-        driver = FakeDriver(by_class=dict(
-                test_class=dict(test_resource=15),
-                override_class=dict(test_resource=20),
-                ))
-        context = FakeContext('test_project', 'test_class')
-        quota_value = resource.quota(driver, context,
-                                     quota_class='override_class')
-
-        self.assertEqual(quota_value, 20)
-
     def test_valid_method_call_check_invalid_input(self):
         resources = {'dummy': 1}
 
@@ -485,14 +393,10 @@ class BaseResourceTestCase(test.TestCase):
 class QuotaEngineTestCase(test.TestCase):
     def test_init(self):
         quota_obj = quota.QuotaEngine()
-
-        self.assertEqual(quota_obj._resources, {})
         self.assertIsInstance(quota_obj._driver, quota.DbQuotaDriver)
 
     def test_init_override_obj(self):
         quota_obj = quota.QuotaEngine(quota_driver=FakeDriver)
-
-        self.assertEqual(quota_obj._resources, {})
         self.assertEqual(quota_obj._driver, FakeDriver)
 
     def test_register_resource(self):
@@ -502,76 +406,19 @@ class QuotaEngineTestCase(test.TestCase):
 
         self.assertEqual(quota_obj._resources, dict(test_resource=resource))
 
-    def test_register_resources(self):
-        quota_obj = quota.QuotaEngine()
-        resources = [
-            quota.AbsoluteResource('test_resource1'),
-            quota.AbsoluteResource('test_resource2'),
-            quota.AbsoluteResource('test_resource3'),
-            ]
-        quota_obj.register_resources(resources)
-
-        self.assertEqual(quota_obj._resources, dict(
-                test_resource1=resources[0],
-                test_resource2=resources[1],
-                test_resource3=resources[2],
-                ))
-
-    def test_get_by_project_and_user(self):
-        context = FakeContext('test_project', 'test_class')
-        driver = FakeDriver(by_user=dict(
-                fake_user=dict(test_resource=42)))
-        quota_obj = quota.QuotaEngine(quota_driver=driver)
-        result = quota_obj.get_by_project_and_user(context, 'test_project',
-                                       'fake_user', 'test_resource')
-
-        self.assertEqual(driver.called, [
-                ('get_by_project_and_user', context, 'test_project',
-                 'fake_user', 'test_resource'),
-                ])
-        self.assertEqual(result, 42)
-
-    def test_get_by_project(self):
-        context = FakeContext('test_project', 'test_class')
-        driver = FakeDriver(by_project=dict(
-                test_project=dict(test_resource=42)))
-        quota_obj = quota.QuotaEngine(quota_driver=driver)
-        result = quota_obj.get_by_project(context, 'test_project',
-                                          'test_resource')
-
-        self.assertEqual(driver.called, [
-                ('get_by_project', context, 'test_project', 'test_resource'),
-                ])
-        self.assertEqual(result, 42)
-
-    def test_get_by_class(self):
-        context = FakeContext('test_project', 'test_class')
-        driver = FakeDriver(by_class=dict(
-                test_class=dict(test_resource=42)))
-        quota_obj = quota.QuotaEngine(quota_driver=driver)
-        result = quota_obj.get_by_class(context, 'test_class', 'test_resource')
-
-        self.assertEqual(driver.called, [
-                ('get_by_class', context, 'test_class', 'test_resource'),
-                ])
-        self.assertEqual(result, 42)
-
-    def _make_quota_obj(self, driver):
-        quota_obj = quota.QuotaEngine(quota_driver=driver)
-        resources = [
+    def _get_quota_engine(self, driver, resources=None):
+        resources = resources or [
             quota.AbsoluteResource('test_resource4'),
             quota.AbsoluteResource('test_resource3'),
             quota.AbsoluteResource('test_resource2'),
             quota.AbsoluteResource('test_resource1'),
-            ]
-        quota_obj.register_resources(resources)
-
-        return quota_obj
+        ]
+        return quota.QuotaEngine(quota_driver=driver, resources=resources)
 
     def test_get_defaults(self):
         context = FakeContext(None, None)
         driver = FakeDriver()
-        quota_obj = self._make_quota_obj(driver)
+        quota_obj = self._get_quota_engine(driver)
         result = quota_obj.get_defaults(context)
 
         self.assertEqual(driver.called, [
@@ -582,36 +429,31 @@ class QuotaEngineTestCase(test.TestCase):
     def test_get_class_quotas(self):
         context = FakeContext(None, None)
         driver = FakeDriver()
-        quota_obj = self._make_quota_obj(driver)
+        quota_obj = self._get_quota_engine(driver)
         result1 = quota_obj.get_class_quotas(context, 'test_class')
-        result2 = quota_obj.get_class_quotas(context, 'test_class', False)
 
         self.assertEqual(driver.called, [
                 ('get_class_quotas', context, quota_obj._resources,
-                 'test_class', True),
-                ('get_class_quotas', context, quota_obj._resources,
-                 'test_class', False),
+                 'test_class'),
                 ])
         self.assertEqual(result1, quota_obj._resources)
-        self.assertEqual(result2, quota_obj._resources)
 
     def test_get_user_quotas(self):
         context = FakeContext(None, None)
         driver = FakeDriver()
-        quota_obj = self._make_quota_obj(driver)
+        quota_obj = self._get_quota_engine(driver)
         result1 = quota_obj.get_user_quotas(context, 'test_project',
                                             'fake_user')
         result2 = quota_obj.get_user_quotas(context, 'test_project',
                                             'fake_user',
                                             quota_class='test_class',
-                                            defaults=False,
                                             usages=False)
 
         self.assertEqual(driver.called, [
                 ('get_user_quotas', context, quota_obj._resources,
-                 'test_project', 'fake_user', None, True, True),
+                 'test_project', 'fake_user', None, True),
                 ('get_user_quotas', context, quota_obj._resources,
-                 'test_project', 'fake_user', 'test_class', False, False),
+                 'test_project', 'fake_user', 'test_class', False),
                 ])
         self.assertEqual(result1, quota_obj._resources)
         self.assertEqual(result2, quota_obj._resources)
@@ -619,18 +461,17 @@ class QuotaEngineTestCase(test.TestCase):
     def test_get_project_quotas(self):
         context = FakeContext(None, None)
         driver = FakeDriver()
-        quota_obj = self._make_quota_obj(driver)
+        quota_obj = self._get_quota_engine(driver)
         result1 = quota_obj.get_project_quotas(context, 'test_project')
         result2 = quota_obj.get_project_quotas(context, 'test_project',
                                                quota_class='test_class',
-                                               defaults=False,
                                                usages=False)
 
         self.assertEqual(driver.called, [
                 ('get_project_quotas', context, quota_obj._resources,
-                 'test_project', None, True, True, False),
+                 'test_project', None, True, False),
                 ('get_project_quotas', context, quota_obj._resources,
-                 'test_project', 'test_class', False, False, False),
+                 'test_project', 'test_class', False, False),
                 ])
         self.assertEqual(result1, quota_obj._resources)
         self.assertEqual(result2, quota_obj._resources)
@@ -638,7 +479,7 @@ class QuotaEngineTestCase(test.TestCase):
     def test_count_as_dict_no_resource(self):
         context = FakeContext(None, None)
         driver = FakeDriver()
-        quota_obj = self._make_quota_obj(driver)
+        quota_obj = self._get_quota_engine(driver)
         self.assertRaises(exception.QuotaResourceUnknown,
                           quota_obj.count_as_dict, context, 'test_resource5',
                           True, foo='bar')
@@ -646,7 +487,7 @@ class QuotaEngineTestCase(test.TestCase):
     def test_count_as_dict_wrong_resource(self):
         context = FakeContext(None, None)
         driver = FakeDriver()
-        quota_obj = self._make_quota_obj(driver)
+        quota_obj = self._get_quota_engine(driver)
         self.assertRaises(exception.QuotaResourceUnknown,
                           quota_obj.count_as_dict, context, 'test_resource1',
                           True, foo='bar')
@@ -659,9 +500,10 @@ class QuotaEngineTestCase(test.TestCase):
 
         context = FakeContext(None, None)
         driver = FakeDriver()
-        quota_obj = self._make_quota_obj(driver)
-        quota_obj.register_resource(
-            quota.CountableResource('test_resource5', fake_count_as_dict))
+        resources = [
+            quota.CountableResource('test_resource5', fake_count_as_dict),
+        ]
+        quota_obj = self._get_quota_engine(driver, resources)
         result = quota_obj.count_as_dict(context, 'test_resource5', True,
                                          foo='bar')
 
@@ -670,7 +512,7 @@ class QuotaEngineTestCase(test.TestCase):
     def test_limit_check(self):
         context = FakeContext(None, None)
         driver = FakeDriver()
-        quota_obj = self._make_quota_obj(driver)
+        quota_obj = self._get_quota_engine(driver)
         quota_obj.limit_check(context, test_resource1=4, test_resource2=3,
                               test_resource3=2, test_resource4=1)
 
@@ -686,7 +528,7 @@ class QuotaEngineTestCase(test.TestCase):
     def test_limit_check_project_and_user(self):
         context = FakeContext(None, None)
         driver = FakeDriver()
-        quota_obj = self._make_quota_obj(driver)
+        quota_obj = self._get_quota_engine(driver)
         project_values = dict(test_resource1=4, test_resource2=3)
         user_values = dict(test_resource3=2, test_resource4=1)
         quota_obj.limit_check_project_and_user(context,
@@ -700,30 +542,8 @@ class QuotaEngineTestCase(test.TestCase):
                           None, None)],
                          driver.called)
 
-    def test_destroy_all_by_project_and_user(self):
-        context = FakeContext(None, None)
-        driver = FakeDriver()
-        quota_obj = self._make_quota_obj(driver)
-        quota_obj.destroy_all_by_project_and_user(context,
-                                                  'test_project', 'fake_user')
-
-        self.assertEqual(driver.called, [
-                ('destroy_all_by_project_and_user', context, 'test_project',
-                 'fake_user'),
-                ])
-
-    def test_destroy_all_by_project(self):
-        context = FakeContext(None, None)
-        driver = FakeDriver()
-        quota_obj = self._make_quota_obj(driver)
-        quota_obj.destroy_all_by_project(context, 'test_project')
-
-        self.assertEqual(driver.called, [
-                ('destroy_all_by_project', context, 'test_project'),
-                ])
-
     def test_resources(self):
-        quota_obj = self._make_quota_obj(None)
+        quota_obj = self._get_quota_engine(None)
 
         self.assertEqual(quota_obj.resources,
                          ['test_resource1', 'test_resource2',
@@ -747,9 +567,6 @@ class DbQuotaDriverTestCase(test.TestCase):
                    security_group_rules=20,
                    server_groups=10,
                    server_group_members=10,
-                   reservation_expire=86400,
-                   until_refresh=0,
-                   max_age=0,
                    group='quota'
                    )
 
@@ -829,19 +646,6 @@ class DbQuotaDriverTestCase(test.TestCase):
                 server_group_members=10,
                 ))
 
-    def test_get_class_quotas_no_defaults(self):
-        self._stub_quota_class_get_all_by_name()
-        result = self.driver.get_class_quotas(None, quota.QUOTAS._resources,
-                                              'test_class', False)
-
-        self.assertEqual(self.calls, ['quota_class_get_all_by_name'])
-        self.assertEqual(result, dict(
-                instances=5,
-                ram=25 * 1024,
-                metadata_items=64,
-                injected_file_content_bytes=5 * 1024,
-                ))
-
     def _stub_get_by_project_and_user(self):
         def fake_qgabpau(context, project_id, user_id):
             self.calls.append('quota_get_all_by_project_and_user')
@@ -1037,17 +841,6 @@ class DbQuotaDriverTestCase(test.TestCase):
                 )
         self.stub_out('nova.db.api.quota_get', fake_quota_get)
 
-    def test_get_by_project_and_user(self):
-        self._stub_get_by_project_and_user_specific()
-        result = self.driver.get_by_project_and_user(
-            FakeContext('test_project', 'test_class'),
-            'test_project', 'fake_user', 'test_resource')
-
-        self.assertEqual(self.calls, ['quota_get'])
-        self.assertEqual(result, dict(
-            test_resource=dict(in_use=20),
-            ))
-
     def _stub_get_by_project(self):
         def fake_qgabp(context, project_id):
             self.calls.append('quota_get_all_by_project')
@@ -1539,68 +1332,6 @@ class DbQuotaDriverTestCase(test.TestCase):
                     ),
                 ))
 
-    @mock.patch('nova.quota.DbQuotaDriver._get_usages',
-                side_effect=_get_fake_get_usages())
-    def test_get_user_quotas_no_defaults(self, mock_get_usages):
-        self._stub_get_by_project_and_user()
-        ctxt = FakeContext('test_project', 'test_class')
-        result = self.driver.get_user_quotas(
-            ctxt, quota.QUOTAS._resources, 'test_project', 'fake_user',
-            defaults=False)
-
-        self.assertEqual(self.calls, [
-                'quota_get_all_by_project_and_user',
-                'quota_get_all_by_project',
-                'quota_class_get_all_by_name',
-                ])
-        mock_get_usages.assert_called_once_with(ctxt, quota.QUOTAS._resources,
-                                                'test_project',
-                                                user_id='fake_user')
-        self.assertEqual(result, dict(
-                cores=dict(
-                    limit=10,
-                    in_use=4,
-                    ),
-               injected_files=dict(
-                    limit=2,
-                    in_use=0,
-                    ),
-                injected_file_path_bytes=dict(
-                    limit=127,
-                    in_use=0,
-                    ),
-                ))
-
-    @mock.patch('nova.quota.DbQuotaDriver._get_usages',
-                side_effect=_get_fake_get_usages())
-    def test_get_project_quotas_no_defaults(self, mock_get_usages):
-        self._stub_get_by_project()
-        ctxt = FakeContext('test_project', 'test_class')
-        result = self.driver.get_project_quotas(
-            ctxt, quota.QUOTAS._resources, 'test_project', defaults=False)
-
-        self.assertEqual(self.calls, [
-                'quota_get_all_by_project',
-                'quota_class_get_all_by_name',
-                'quota_class_get_default',
-                ])
-        mock_get_usages.assert_called_once_with(ctxt, quota.QUOTAS._resources,
-                                                'test_project')
-        self.assertEqual(result, dict(
-                cores=dict(
-                    limit=10,
-                    in_use=4,
-                    ),
-               injected_files=dict(
-                    limit=2,
-                    in_use=0,
-                    ),
-                injected_file_path_bytes=dict(
-                    limit=127,
-                    in_use=0,
-                    ),
-                ))
-
     def test_get_user_quotas_no_usages(self):
         self._stub_get_by_project_and_user()
         result = self.driver.get_user_quotas(
@@ -1720,7 +1451,7 @@ class DbQuotaDriverTestCase(test.TestCase):
             return {'floating_ips': 20}
 
         def fake_get_project_quotas(dbdrv, context, resources, project_id,
-                                    quota_class=None, defaults=True,
+                                    quota_class=None,
                                     usages=True, remains=False,
                                     project_quotas=None):
             self.calls.append('get_project_quotas')
@@ -1748,7 +1479,7 @@ class DbQuotaDriverTestCase(test.TestCase):
         def fake_process_quotas_in_get_user_quotas(dbdrv, context, resources,
                                                    project_id, quotas,
                                                    quota_class=None,
-                                                   defaults=True, usages=None,
+                                                   usages=None,
                                                    remains=False):
             self.calls.append('_process_quotas')
             result = {}
@@ -1989,7 +1720,7 @@ class DbQuotaDriverTestCase(test.TestCase):
 
     def _stub_get_project_quotas(self):
         def fake_get_project_quotas(dbdrv, context, resources, project_id,
-                                    quota_class=None, defaults=True,
+                                    quota_class=None,
                                     usages=True, remains=False,
                                     project_quotas=None):
             self.calls.append('get_project_quotas')
@@ -2163,9 +1894,6 @@ class NoopQuotaDriverTestCase(test.TestCase):
                    injected_file_path_length=255,
                    security_groups=10,
                    security_group_rules=20,
-                   reservation_expire=86400,
-                   until_refresh=0,
-                   max_age=0,
                    group='quota'
                    )
 
@@ -2193,13 +1921,6 @@ class NoopQuotaDriverTestCase(test.TestCase):
                                               'test_class')
         self.assertEqual(self.expected_without_dict, result)
 
-    def test_get_class_quotas_no_defaults(self):
-        result = self.driver.get_class_quotas(None,
-                                              quota.QUOTAS._resources,
-                                              'test_class',
-                                              False)
-        self.assertEqual(self.expected_without_dict, result)
-
     def test_get_project_quotas(self):
         result = self.driver.get_project_quotas(None,
                                                 quota.QUOTAS._resources,
@@ -2213,21 +1934,6 @@ class NoopQuotaDriverTestCase(test.TestCase):
                                              'fake_user')
         self.assertEqual(self.expected_with_usages, result)
 
-    def test_get_project_quotas_no_defaults(self):
-        result = self.driver.get_project_quotas(None,
-                                                quota.QUOTAS._resources,
-                                                'test_project',
-                                                defaults=False)
-        self.assertEqual(self.expected_with_usages, result)
-
-    def test_get_user_quotas_no_defaults(self):
-        result = self.driver.get_user_quotas(None,
-                                             quota.QUOTAS._resources,
-                                             'test_project',
-                                             'fake_user',
-                                             defaults=False)
-        self.assertEqual(self.expected_with_usages, result)
-
     def test_get_project_quotas_no_usages(self):
         result = self.driver.get_project_quotas(None,
                                                 quota.QUOTAS._resources,
