@@ -17,6 +17,7 @@ import copy
 
 import fixtures
 import mock
+from oslo_utils.fixture import uuidsentinel as uuids
 
 from nova import block_device
 from nova import context
@@ -27,7 +28,6 @@ from nova import test
 from nova.tests.unit import fake_block_device
 import nova.tests.unit.image.fake
 from nova.tests.unit.virt import fakelibosinfo
-from nova.tests import uuidsentinel as uuids
 from nova.virt import block_device as driver_block_device
 from nova.virt import driver
 from nova.virt.libvirt import blockinfo
@@ -210,14 +210,13 @@ class LibvirtBlockInfoTest(test.NoDBTestCase):
                                              "virtio", "ide",
                                              image_meta,
                                              block_device_info)
-
         expect = {
-            'disk': {'bus': 'scsi', 'dev': 'sda',
+            'disk': {'bus': 'virtio', 'dev': 'sda',
                      'type': 'disk', 'boot_index': '1'},
             'disk.local': {'bus': 'virtio', 'dev': 'vda', 'type': 'disk'},
-            'root': {'bus': 'scsi', 'dev': 'sda',
+            'root': {'bus': 'virtio', 'dev': 'sda',
                      'type': 'disk', 'boot_index': '1'}
-            }
+        }
         self.assertEqual(expect, mapping)
 
     def test_get_disk_mapping_rescue(self):
@@ -934,38 +933,29 @@ class LibvirtBlockInfoTest(test.NoDBTestCase):
 
     @mock.patch('nova.virt.libvirt.blockinfo.find_disk_dev_for_disk_bus',
                 return_value='vda')
-    @mock.patch('nova.virt.libvirt.blockinfo.get_disk_bus_for_disk_dev',
-                return_value='virtio')
-    def test_get_root_info_no_bdm(self, mock_get_bus, mock_find_dev):
+    def test_get_root_info_no_bdm(self, mock_find_dev):
         instance = objects.Instance(**self.test_instance)
         image_meta = objects.ImageMeta.from_dict(self.test_image_meta)
-        blockinfo.get_root_info(instance, 'kvm', image_meta, None,
+        info = blockinfo.get_root_info(instance, 'kvm', image_meta, None,
                                 'virtio', 'ide')
         mock_find_dev.assert_called_once_with({}, 'virtio')
 
-        blockinfo.get_root_info(instance, 'kvm', image_meta, None,
-                                'virtio', 'ide', root_device_name='/dev/vda')
-        mock_get_bus.assert_called_once_with('kvm', '/dev/vda')
+        self.assertEqual('virtio', info['bus'])
 
     @mock.patch('nova.virt.libvirt.blockinfo.find_disk_dev_for_disk_bus',
                 return_value='vda')
-    @mock.patch('nova.virt.libvirt.blockinfo.get_disk_bus_for_disk_dev',
-                return_value='virtio')
-    def test_get_root_info_no_bdm_empty_image_meta(self, mock_get_bus,
-                                                   mock_find_dev):
+    def test_get_root_info_no_bdm_empty_image_meta(self, mock_find_dev):
         # The evacuate operation passes image_ref=None to the compute node for
         # rebuild which then defaults image_meta to {}, so we don't have any
         # attributes in the ImageMeta object passed to get_root_info and we
         # need to make sure we don't try lazy-loading anything.
         instance = objects.Instance(**self.test_instance)
         image_meta = objects.ImageMeta.from_dict({})
-        blockinfo.get_root_info(instance, 'kvm', image_meta, None,
+        info = blockinfo.get_root_info(instance, 'kvm', image_meta, None,
                                 'virtio', 'ide')
         mock_find_dev.assert_called_once_with({}, 'virtio')
 
-        blockinfo.get_root_info(instance, 'kvm', image_meta, None,
-                                'virtio', 'ide', root_device_name='/dev/vda')
-        mock_get_bus.assert_called_once_with('kvm', '/dev/vda')
+        self.assertEqual('virtio', info['bus'])
 
     @mock.patch('nova.virt.libvirt.blockinfo.get_info_from_bdm')
     def test_get_root_info_bdm(self, mock_get_info):
