@@ -20,6 +20,7 @@ from eventlet import greenthread
 from lxml import etree
 import mock
 from oslo_concurrency.fixture import lockutils as lock_fixture
+from oslo_utils.fixture import uuidsentinel as uuids
 from oslo_utils import uuidutils
 
 from nova import exception
@@ -28,7 +29,6 @@ from nova import objects
 from nova import test
 from nova.tests.unit import fake_network
 from nova.tests.unit.virt.libvirt import fakelibvirt
-from nova.tests import uuidsentinel as uuids
 from nova.virt.libvirt import firewall
 from nova.virt.libvirt import host
 from nova.virt import netutils
@@ -149,9 +149,10 @@ class IptablesFirewallTestCase(test.NoDBTestCase):
         inst.info_cache.deleted = False
         return inst
 
+    @mock.patch.object(linux_net.iptables_manager, "execute")
     @mock.patch.object(objects.InstanceList, "get_by_security_group_id")
     @mock.patch.object(objects.SecurityGroupRuleList, "get_by_instance")
-    def test_static_filters(self, mock_secrule, mock_instlist):
+    def test_static_filters(self, mock_secrule, mock_instlist, fake_execute):
         UUID = "2674993b-6adb-4733-abd9-a7c10cc1f146"
         SRC_UUID = "0e0a76b2-7c52-4bc0-9a60-d83017e42c1a"
         instance_ref = self._create_instance_ref(UUID)
@@ -247,7 +248,7 @@ class IptablesFirewallTestCase(test.NoDBTestCase):
 
         network_model = _fake_network_info(self, 1)
 
-        linux_net.iptables_manager.execute = fake_iptables_execute
+        fake_execute.side_effect = fake_iptables_execute
 
         self.stub_out('nova.objects.Instance.get_network_info',
                       lambda instance: network_model)
