@@ -17,6 +17,7 @@
 import sys
 
 import mock
+from oslo_service import fixture as service_fixture
 from oslo_utils import encodeutils
 import six
 import testtools
@@ -34,8 +35,6 @@ if sys.version_info > (3,):
     long = int
 
 
-# Make RetryDecorator not actually sleep on retries
-@mock.patch('oslo_service.loopingcall.LoopingCallBase._sleep', new=mock.Mock())
 class GuestTestCase(test.NoDBTestCase):
 
     def setUp(self):
@@ -48,6 +47,9 @@ class GuestTestCase(test.NoDBTestCase):
         self.domain = mock.Mock(spec=fakelibvirt.virDomain)
         self.guest = libvirt_guest.Guest(self.domain)
 
+        # Make RetryDecorator not actually sleep on retries
+        self.useFixture(service_fixture.SleepFixture())
+
     def test_repr(self):
         self.domain.ID.return_value = 99
         self.domain.UUIDString.return_value = "UUID"
@@ -325,11 +327,6 @@ class GuestTestCase(test.NoDBTestCase):
             inc_sleep_time=.01, max_retry_count=3)
         # Some time later, we can do the wait/retry to ensure detach
         self.assertRaises(exception.DeviceNotFound, retry_detach)
-        # Check that the save_and_reraise_exception context manager didn't log
-        # a traceback when the libvirtError was caught and DeviceNotFound was
-        # raised.
-        self.assertNotIn('Original exception being dropped',
-                         self.stdlog.logger.output)
 
     @mock.patch.object(libvirt_guest.Guest, "detach_device")
     def test_detach_device_with_retry_operation_internal(self, mock_detach):
@@ -724,10 +721,6 @@ class GuestTestCase(test.NoDBTestCase):
         self.guest.migrate_configure_max_downtime(1000)
         self.domain.migrateSetMaxDowntime.assert_called_once_with(1000)
 
-    def test_migrate_configure_max_speed(self):
-        self.guest.migrate_configure_max_speed(1000)
-        self.domain.migrateSetMaxSpeed.assert_called_once_with(1000)
-
 
 class GuestBlockTestCase(test.NoDBTestCase):
 
@@ -874,6 +867,10 @@ class GuestBlockTestCase(test.NoDBTestCase):
         self.assertRaises(fakelibvirt.libvirtError,
                           self.gblock.is_job_complete)
 
+    def test_blockStats(self):
+        self.gblock.blockStats()
+        self.domain.blockStats.assert_called_once_with('vda')
+
 
 class JobInfoTestCase(test.NoDBTestCase):
 
