@@ -16,15 +16,16 @@ from collections import deque
 
 from lxml import etree
 import mock
+from oslo_utils.fixture import uuidsentinel as uuids
 from oslo_utils import units
-
+import six
 
 from nova.compute import power_state
+from nova import exception
+from nova.network import model as network_model
 from nova import objects
 from nova import test
-from nova.tests.unit import matchers
 from nova.tests.unit.virt.libvirt import fakelibvirt
-from nova.tests import uuidsentinel as uuids
 from nova.virt.libvirt import config as vconfig
 from nova.virt.libvirt import guest as libvirt_guest
 from nova.virt.libvirt import host
@@ -92,13 +93,14 @@ class UtilityMigrationTestCase(test.NoDBTestCase):
         self.assertEqual([], ports)
 
     @mock.patch('lxml.etree.tostring')
+    @mock.patch.object(migration, '_update_memory_backing_xml')
     @mock.patch.object(migration, '_update_perf_events_xml')
     @mock.patch.object(migration, '_update_graphics_xml')
     @mock.patch.object(migration, '_update_serial_xml')
     @mock.patch.object(migration, '_update_volume_xml')
     def test_get_updated_guest_xml(
             self, mock_volume, mock_serial, mock_graphics,
-            mock_perf_events_xml, mock_tostring):
+            mock_perf_events_xml, mock_memory_backing, mock_tostring):
         data = objects.LibvirtLiveMigrateData()
         mock_guest = mock.Mock(spec=libvirt_guest.Guest)
         get_volume_config = mock.MagicMock()
@@ -109,6 +111,7 @@ class UtilityMigrationTestCase(test.NoDBTestCase):
         mock_serial.assert_called_once_with(mock.ANY, data)
         mock_volume.assert_called_once_with(mock.ANY, data, get_volume_config)
         mock_perf_events_xml.assert_called_once_with(mock.ANY, data)
+        mock_memory_backing.assert_called_once_with(mock.ANY, data)
         self.assertEqual(1, mock_tostring.called)
 
     def test_update_serial_xml_serial(self):
@@ -128,7 +131,7 @@ class UtilityMigrationTestCase(test.NoDBTestCase):
                              encoding='unicode')
         new_xml = xml.replace("127.0.0.1", "127.0.0.100").replace(
             "2000", "2001")
-        self.assertThat(res, matchers.XMLMatches(new_xml))
+        self.assertXmlEqual(res, new_xml)
 
     def test_update_serial_xml_console(self):
         data = objects.LibvirtLiveMigrateData(
@@ -152,7 +155,7 @@ class UtilityMigrationTestCase(test.NoDBTestCase):
         new_xml = xml.replace("127.0.0.1", "127.0.0.100").replace(
             "2001", "299").replace("2002", "300")
 
-        self.assertThat(res, matchers.XMLMatches(new_xml))
+        self.assertXmlEqual(res, new_xml)
 
     def test_update_serial_xml_without_ports(self):
         # This test is for backwards compatibility when we don't
@@ -176,7 +179,7 @@ class UtilityMigrationTestCase(test.NoDBTestCase):
         res = etree.tostring(migration._update_serial_xml(doc, data),
                              encoding='unicode')
         new_xml = xml.replace("127.0.0.1", "127.0.0.100")
-        self.assertThat(res, matchers.XMLMatches(new_xml))
+        self.assertXmlEqual(res, new_xml)
 
     def test_update_graphics(self):
         data = objects.LibvirtLiveMigrateData(
@@ -197,7 +200,7 @@ class UtilityMigrationTestCase(test.NoDBTestCase):
                              encoding='unicode')
         new_xml = xml.replace("127.0.0.1", "127.0.0.100")
         new_xml = new_xml.replace("127.0.0.2", "127.0.0.200")
-        self.assertThat(res, matchers.XMLMatches(new_xml))
+        self.assertXmlEqual(res, new_xml)
 
     def test_update_volume_xml(self):
         connection_info = {
@@ -246,7 +249,7 @@ class UtilityMigrationTestCase(test.NoDBTestCase):
             doc, data, get_volume_config), encoding='unicode')
         new_xml = xml.replace('ip-1.2.3.4:3260-iqn.abc.12345.opst-lun-X',
                               'ip-1.2.3.4:3260-iqn.cde.67890.opst-lun-Z')
-        self.assertThat(res, matchers.XMLMatches(new_xml))
+        self.assertXmlEqual(res, new_xml)
 
     def test_update_volume_xml_keeps_address(self):
         # Now test to make sure address isn't altered for virtio-scsi and rbd
@@ -313,7 +316,7 @@ class UtilityMigrationTestCase(test.NoDBTestCase):
             doc, data, get_volume_config), encoding='unicode')
         new_xml = xml.replace('sdb',
                               'sdc')
-        self.assertThat(res, matchers.XMLMatches(new_xml))
+        self.assertXmlEqual(res, new_xml)
 
     def test_update_volume_xml_add_encryption(self):
         connection_info = {
@@ -400,7 +403,7 @@ class UtilityMigrationTestCase(test.NoDBTestCase):
         doc = etree.fromstring(xml)
         res = etree.tostring(migration._update_volume_xml(
             doc, data, get_volume_config), encoding='unicode')
-        self.assertThat(res, matchers.XMLMatches(new_xml))
+        self.assertXmlEqual(res, new_xml)
 
     def test_update_volume_xml_update_encryption(self):
         connection_info = {
@@ -470,7 +473,7 @@ class UtilityMigrationTestCase(test.NoDBTestCase):
             doc, data, get_volume_config), encoding='unicode')
         new_xml = xml.replace(uuids.encryption_secret_uuid_old,
                               uuids.encryption_secret_uuid_new)
-        self.assertThat(res, matchers.XMLMatches(new_xml))
+        self.assertXmlEqual(res, new_xml)
 
     def test_update_perf_events_xml(self):
         data = objects.LibvirtLiveMigrateData(
@@ -485,10 +488,10 @@ class UtilityMigrationTestCase(test.NoDBTestCase):
         res = etree.tostring(migration._update_perf_events_xml(doc, data),
                              encoding='unicode')
 
-        self.assertThat(res, matchers.XMLMatches("""<domain>
+        self.assertXmlEqual(res, """<domain>
   <perf>
     <event enabled="yes" name="cmt"/></perf>
-</domain>"""))
+</domain>""")
 
     def test_update_perf_events_xml_add_new_events(self):
         data = objects.LibvirtLiveMigrateData(
@@ -499,8 +502,8 @@ class UtilityMigrationTestCase(test.NoDBTestCase):
         res = etree.tostring(migration._update_perf_events_xml(doc, data),
                              encoding='unicode')
 
-        self.assertThat(res, matchers.XMLMatches("""<domain>
-<perf><event enabled="yes" name="cmt"/></perf></domain>"""))
+        self.assertXmlEqual(res, """<domain>
+<perf><event enabled="yes" name="cmt"/></perf></domain>""")
 
     def test_update_perf_events_xml_add_new_events1(self):
         data = objects.LibvirtLiveMigrateData(
@@ -514,10 +517,10 @@ class UtilityMigrationTestCase(test.NoDBTestCase):
         res = etree.tostring(migration._update_perf_events_xml(doc, data),
                              encoding='unicode')
 
-        self.assertThat(res, matchers.XMLMatches("""<domain>
+        self.assertXmlEqual(res, """<domain>
   <perf>
     <event enabled="yes" name="cmt"/><event enabled="yes" name="mbml"/></perf>
-</domain>"""))
+</domain>""")
 
     def test_update_perf_events_xml_remove_all_events(self):
         data = objects.LibvirtLiveMigrateData(
@@ -531,10 +534,355 @@ class UtilityMigrationTestCase(test.NoDBTestCase):
         res = etree.tostring(migration._update_perf_events_xml(doc, data),
                              encoding='unicode')
 
-        self.assertThat(res, matchers.XMLMatches("""<domain>
+        self.assertXmlEqual(res, """<domain>
   <perf>
     </perf>
-</domain>"""))
+</domain>""")
+
+    def test_update_memory_backing_xml_remove(self):
+        data = objects.LibvirtLiveMigrateData(
+            dst_wants_file_backed_memory=False)
+        xml = """<domain>
+  <memoryBacking>
+    <source type="file"/>
+    <access mode="shared"/>
+    <allocation mode="immediate"/>
+  </memoryBacking>
+</domain>"""
+        doc = etree.fromstring(xml)
+        res = etree.tostring(migration._update_memory_backing_xml(doc, data),
+                             encoding='unicode')
+
+        self.assertXmlEqual(res, """<domain>
+  <memoryBacking/>
+</domain>""")
+
+    def test_update_memory_backing_xml_add(self):
+        data = objects.LibvirtLiveMigrateData(
+            dst_wants_file_backed_memory=True)
+        xml = """<domain/>"""
+        doc = etree.fromstring(xml)
+        res = etree.tostring(migration._update_memory_backing_xml(doc, data),
+                             encoding='unicode')
+
+        self.assertXmlEqual(res, """<domain>
+  <memoryBacking>
+    <source type="file"/>
+    <access mode="shared"/>
+    <allocation mode="immediate"/>
+  </memoryBacking>
+</domain>""")
+
+    def test_update_memory_backing_xml_keep(self):
+        data = objects.LibvirtLiveMigrateData(
+            dst_wants_file_backed_memory=True)
+
+        xml = """<domain>
+  <memoryBacking>
+    <source type="file"/>
+    <access mode="shared"/>
+    <allocation mode="immediate"/>
+  </memoryBacking>
+</domain>"""
+        doc = etree.fromstring(xml)
+        res = etree.tostring(migration._update_memory_backing_xml(doc, data),
+                             encoding='unicode')
+
+        self.assertXmlEqual(res, """<domain>
+  <memoryBacking>
+    <source type="file"/>
+    <access mode="shared"/>
+    <allocation mode="immediate"/>
+  </memoryBacking>
+</domain>""")
+
+    def test_update_memory_backing_discard_add(self):
+        data = objects.LibvirtLiveMigrateData(
+            dst_wants_file_backed_memory=True, file_backed_memory_discard=True)
+
+        xml = """<domain>
+  <memoryBacking>
+    <source type="file"/>
+    <access mode="shared"/>
+    <allocation mode="immediate"/>
+  </memoryBacking>
+</domain>"""
+        doc = etree.fromstring(xml)
+        res = etree.tostring(migration._update_memory_backing_xml(doc, data),
+                             encoding='unicode')
+
+        self.assertXmlEqual(res, """<domain>
+  <memoryBacking>
+    <source type="file"/>
+    <access mode="shared"/>
+    <allocation mode="immediate"/>
+    <discard />
+  </memoryBacking>
+</domain>""")
+
+    def test_update_memory_backing_discard_remove(self):
+        data = objects.LibvirtLiveMigrateData(
+            dst_wants_file_backed_memory=True,
+            file_backed_memory_discard=False)
+
+        xml = """<domain>
+  <memoryBacking>
+    <source type="file"/>
+    <access mode="shared"/>
+    <allocation mode="immediate"/>
+    <discard />
+  </memoryBacking>
+</domain>"""
+        doc = etree.fromstring(xml)
+        res = etree.tostring(migration._update_memory_backing_xml(doc, data),
+                             encoding='unicode')
+
+        self.assertXmlEqual(res, """<domain>
+  <memoryBacking>
+    <source type="file"/>
+    <access mode="shared"/>
+    <allocation mode="immediate"/>
+  </memoryBacking>
+</domain>""")
+
+    def test_update_memory_backing_discard_keep(self):
+        data = objects.LibvirtLiveMigrateData(
+            dst_wants_file_backed_memory=True, file_backed_memory_discard=True)
+
+        xml = """<domain>
+  <memoryBacking>
+    <source type="file"/>
+    <access mode="shared"/>
+    <allocation mode="immediate"/>
+    <discard />
+  </memoryBacking>
+</domain>"""
+        doc = etree.fromstring(xml)
+        res = etree.tostring(migration._update_memory_backing_xml(doc, data),
+                             encoding='unicode')
+
+        self.assertXmlEqual(res, """<domain>
+  <memoryBacking>
+    <source type="file"/>
+    <access mode="shared"/>
+    <allocation mode="immediate"/>
+    <discard />
+  </memoryBacking>
+</domain>""")
+
+    def _test_update_vif_xml(self, conf, original_xml, expected_xml):
+        """Simulates updating the guest xml for live migrating from a host
+        using OVS to a host using vhostuser as the networking backend.
+        """
+        vif_ovs = network_model.VIF(id=uuids.vif,
+                                    address='DE:AD:BE:EF:CA:FE',
+                                    details={'port_filter': False},
+                                    devname='tap-xxx-yyy-zzz',
+                                    ovs_interfaceid=uuids.ovs)
+        source_vif = vif_ovs
+        migrate_vifs = [
+            objects.VIFMigrateData(
+                port_id=uuids.port_id,
+                vnic_type=network_model.VNIC_TYPE_NORMAL,
+                vif_type=network_model.VIF_TYPE_VHOSTUSER,
+                vif_details={
+                    'vhostuser_socket': '/vhost-user/test.sock'
+                },
+                profile={},
+                host='dest.host',
+                source_vif=source_vif)
+        ]
+        data = objects.LibvirtLiveMigrateData(vifs=migrate_vifs)
+
+        get_vif_config = mock.MagicMock(return_value=conf)
+        doc = etree.fromstring(original_xml)
+        updated_xml = etree.tostring(
+            migration._update_vif_xml(doc, data, get_vif_config),
+            encoding='unicode')
+        self.assertXmlEqual(updated_xml, expected_xml)
+
+    def test_update_vif_xml_to_vhostuser(self):
+        conf = vconfig.LibvirtConfigGuestInterface()
+        conf.net_type = "vhostuser"
+        conf.vhostuser_type = "unix"
+        conf.vhostuser_mode = "server"
+        conf.mac_addr = "DE:AD:BE:EF:CA:FE"
+        conf.vhostuser_path = "/vhost-user/test.sock"
+        conf.model = "virtio"
+
+        original_xml = """<domain>
+ <uuid>3de6550a-8596-4937-8046-9d862036bca5</uuid>
+ <devices>
+    <interface type="bridge">
+        <mac address="DE:AD:BE:EF:CA:FE"/>
+        <model type="virtio"/>
+        <source bridge="qbra188171c-ea"/>
+        <target dev="tapa188171c-ea"/>
+        <virtualport type="openvswitch">
+            <parameters interfaceid="%s"/>
+        </virtualport>
+        <address type='pci' domain='0x0000' bus='0x00' slot='0x04'
+                 function='0x0'/>
+    </interface>
+ </devices>
+</domain>""" % uuids.ovs
+        # Note that <target> and <virtualport> are dropped from the ovs source
+        # interface xml since they aren't applicable to the vhostuser
+        # destination interface xml. The type attribute value changes and the
+        # hardware address element is retained.
+        expected_xml = """<domain>
+ <uuid>3de6550a-8596-4937-8046-9d862036bca5</uuid>
+ <devices>
+    <interface type="vhostuser">
+        <mac address="DE:AD:BE:EF:CA:FE"/>
+        <model type="virtio"/>
+        <source mode="server" path="/vhost-user/test.sock" type="unix"/>
+        <address type='pci' domain='0x0000' bus='0x00' slot='0x04'
+                 function='0x0'/>
+    </interface>
+ </devices>
+</domain>"""
+        self._test_update_vif_xml(conf, original_xml, expected_xml)
+
+    def test_update_vif_xml_to_bridge_without_mtu(self):
+        """This test validates _update_vif_xml to make sure it does not add
+        MTU to the interface if it does not exist in the source XML.
+        """
+        conf = vconfig.LibvirtConfigGuestInterface()
+        conf.net_type = "bridge"
+        conf.source_dev = "qbra188171c-ea"
+        conf.target_dev = "tapa188171c-ea"
+        conf.mac_addr = "DE:AD:BE:EF:CA:FE"
+        conf.vhostuser_path = "/vhost-user/test.sock"
+        conf.model = "virtio"
+        conf.mtu = 9000
+
+        original_xml = """<domain>
+ <uuid>3de6550a-8596-4937-8046-9d862036bca5</uuid>
+ <devices>
+    <interface type="bridge">
+        <mac address="DE:AD:BE:EF:CA:FE"/>
+        <model type="virtio"/>
+        <source bridge="qbra188171c-ea"/>
+        <target dev="tapa188171c-ea"/>
+        <virtualport type="openvswitch">
+            <parameters interfaceid="%s"/>
+        </virtualport>
+        <address type='pci' domain='0x0000' bus='0x00' slot='0x04'
+                 function='0x0'/>
+    </interface>
+ </devices>
+</domain>""" % uuids.ovs
+        expected_xml = """<domain>
+ <uuid>3de6550a-8596-4937-8046-9d862036bca5</uuid>
+ <devices>
+    <interface type="bridge">
+        <mac address="DE:AD:BE:EF:CA:FE"/>
+        <model type="virtio"/>
+        <source bridge="qbra188171c-ea"/>
+        <target dev="tapa188171c-ea"/>
+        <address type='pci' domain='0x0000' bus='0x00' slot='0x04'
+                 function='0x0'/>
+    </interface>
+ </devices>
+</domain>"""
+        self._test_update_vif_xml(conf, original_xml, expected_xml)
+
+    def test_update_vif_xml_to_bridge_with_mtu(self):
+        """This test validates _update_vif_xml to make sure it maintains the
+        interface MTU if it exists in the source XML
+        """
+        conf = vconfig.LibvirtConfigGuestInterface()
+        conf.net_type = "bridge"
+        conf.source_dev = "qbra188171c-ea"
+        conf.target_dev = "tapa188171c-ea"
+        conf.mac_addr = "DE:AD:BE:EF:CA:FE"
+        conf.vhostuser_path = "/vhost-user/test.sock"
+        conf.model = "virtio"
+        conf.mtu = 9000
+
+        original_xml = """<domain>
+ <uuid>3de6550a-8596-4937-8046-9d862036bca5</uuid>
+ <devices>
+    <interface type="bridge">
+        <mac address="DE:AD:BE:EF:CA:FE"/>
+        <model type="virtio"/>
+        <source bridge="qbra188171c-ea"/>
+        <target dev="tapa188171c-ea"/>
+        <mtu size="9000"/>
+        <virtualport type="openvswitch">
+            <parameters interfaceid="%s"/>
+        </virtualport>
+        <address type='pci' domain='0x0000' bus='0x00' slot='0x04'
+                 function='0x0'/>
+    </interface>
+ </devices>
+</domain>""" % uuids.ovs
+        expected_xml = """<domain>
+ <uuid>3de6550a-8596-4937-8046-9d862036bca5</uuid>
+ <devices>
+    <interface type="bridge">
+        <mac address="DE:AD:BE:EF:CA:FE"/>
+        <model type="virtio"/>
+        <source bridge="qbra188171c-ea"/>
+        <mtu size="9000"/>
+        <target dev="tapa188171c-ea"/>
+        <address type='pci' domain='0x0000' bus='0x00' slot='0x04'
+                 function='0x0'/>
+    </interface>
+ </devices>
+</domain>"""
+        self._test_update_vif_xml(conf, original_xml, expected_xml)
+
+    def test_update_vif_xml_no_mac_address_in_xml(self):
+        """Tests that the <mac address> is not found in the <interface> XML
+        which results in an error.
+        """
+        data = objects.LibvirtLiveMigrateData(vifs=[
+            objects.VIFMigrateData(source_vif=network_model.VIF(
+                address="DE:AD:BE:EF:CA:FE"))])
+        original_xml = """<domain>
+         <uuid>3de6550a-8596-4937-8046-9d862036bca5</uuid>
+         <devices>
+            <interface type='direct'>
+                <source dev='eth0' mode='private'/>
+                <virtualport type='802.1Qbh'>
+                  <parameters profileid='finance'/>
+                </virtualport>
+            </interface>
+         </devices>
+        </domain>"""
+        get_vif_config = mock.MagicMock(new_callable=mock.NonCallableMock)
+        doc = etree.fromstring(original_xml)
+        ex = self.assertRaises(exception.NovaException,
+                               migration._update_vif_xml,
+                               doc, data, get_vif_config)
+        self.assertIn('Unable to find MAC address in interface XML',
+                      six.text_type(ex))
+
+    def test_update_vif_xml_no_matching_vif(self):
+        """Tests that the vif in the migrate data is not found in the existing
+        guest interfaces.
+        """
+        data = objects.LibvirtLiveMigrateData(vifs=[
+            objects.VIFMigrateData(source_vif=network_model.VIF(
+                address="DE:AD:BE:EF:CA:FE"))])
+        original_xml = """<domain>
+         <uuid>3de6550a-8596-4937-8046-9d862036bca5</uuid>
+         <devices>
+            <interface type="bridge">
+                <mac address="CA:FE:DE:AD:BE:EF"/>
+                <model type="virtio"/>
+                <source bridge="qbra188171c-ea"/>
+                <target dev="tapa188171c-ea"/>
+            </interface>
+         </devices>
+        </domain>"""
+        get_vif_config = mock.MagicMock(new_callable=mock.NonCallableMock)
+        doc = etree.fromstring(original_xml)
+        ex = self.assertRaises(KeyError, migration._update_vif_xml,
+                               doc, data, get_vif_config)
+        self.assertIn("CA:FE:DE:AD:BE:EF", six.text_type(ex))
 
 
 class MigrationMonitorTestCase(test.NoDBTestCase):
@@ -625,53 +973,20 @@ class MigrationMonitorTestCase(test.NoDBTestCase):
         self.assertEqual(migration.find_job_type(self.guest, self.instance),
                          fakelibvirt.VIR_DOMAIN_JOB_FAILED)
 
-    def test_live_migration_abort_stuck(self):
-        # Progress time exceeds progress timeout
-        self.assertTrue(migration.should_abort(self.instance,
-                                               5000,
-                                               1000, 2000,
-                                               4500, 9000,
-                                               "running"))
-
-    def test_live_migration_abort_no_prog_timeout(self):
-        # Progress timeout is disabled
-        self.assertFalse(migration.should_abort(self.instance,
-                                                5000,
-                                                1000, 0,
-                                                4500, 9000,
-                                                "running"))
-
-    def test_live_migration_abort_not_stuck(self):
-        # Progress time is less than progress timeout
-        self.assertFalse(migration.should_abort(self.instance,
-                                                5000,
-                                                4500, 2000,
-                                                4500, 9000,
-                                                "running"))
-
     def test_live_migration_abort_too_long(self):
         # Elapsed time is over completion timeout
-        self.assertTrue(migration.should_abort(self.instance,
-                                               5000,
-                                               4500, 2000,
-                                               4500, 2000,
-                                               "running"))
+        self.assertTrue(migration.should_trigger_timeout_action(
+            self.instance, 4500, 2000, "running"))
 
     def test_live_migration_abort_no_comp_timeout(self):
         # Completion timeout is disabled
-        self.assertFalse(migration.should_abort(self.instance,
-                                                5000,
-                                                4500, 2000,
-                                                4500, 0,
-                                                "running"))
+        self.assertFalse(migration.should_trigger_timeout_action(
+            self.instance, 4500, 0, "running"))
 
     def test_live_migration_abort_still_working(self):
         # Elapsed time is less than completion timeout
-        self.assertFalse(migration.should_abort(self.instance,
-                                                5000,
-                                                4500, 2000,
-                                                4500, 9000,
-                                                "running"))
+        self.assertFalse(migration.should_trigger_timeout_action(
+            self.instance, 4500, 9000, "running"))
 
     def test_live_migration_postcopy_switch(self):
         # Migration progress is not fast enough
