@@ -12,10 +12,12 @@
 #    License for the specific language governing permissions and limitations
 #    under the License.
 
-from oslo_config import cfg
+import mock
+from oslo_utils.fixture import uuidsentinel as uuids
 
 from nova import block_device
 from nova.compute import vm_states
+import nova.conf
 from nova import context
 from nova import objects
 from nova.objects import block_device as block_device_obj
@@ -23,10 +25,10 @@ from nova import test
 from nova.tests.unit import fake_instance
 from nova.virt import imagecache
 
-CONF = cfg.CONF
+CONF = nova.conf.CONF
 
 swap_bdm_128 = [block_device.BlockDeviceDict(
-        {'id': 1, 'instance_uuid': 'fake-instance',
+        {'id': 1, 'instance_uuid': uuids.instance,
          'device_name': '/dev/sdb1',
          'source_type': 'blank',
          'destination_type': 'local',
@@ -37,7 +39,7 @@ swap_bdm_128 = [block_device.BlockDeviceDict(
          'boot_index': -1})]
 
 swap_bdm_256 = [block_device.BlockDeviceDict(
-        {'id': 1, 'instance_uuid': 'fake-instance',
+        {'id': 1, 'instance_uuid': uuids.instance,
          'device_name': '/dev/sdb1',
          'source_type': 'blank',
          'destination_type': 'local',
@@ -64,24 +66,25 @@ class ImageCacheManagerTests(test.NoDBTestCase):
                           cache_manager.update, None, [])
         self.assertRaises(NotImplementedError,
                           cache_manager._get_base)
-        base_images = cache_manager._list_base_images(None)
-        self.assertEqual([], base_images['unexplained_images'])
-        self.assertEqual([], base_images['originals'])
+        self.assertRaises(NotImplementedError,
+                          cache_manager._scan_base_images, None)
         self.assertRaises(NotImplementedError,
                           cache_manager._age_and_verify_cached_images,
                           None, [], None)
 
-    def test_list_running_instances(self):
+    @mock.patch.object(objects.BlockDeviceMappingList,
+                       'bdms_by_instance_uuid')
+    def test_list_running_instances(self, mock_bdms_by_uuid):
         instances = [{'image_ref': '1',
                       'host': CONF.host,
                       'id': '1',
-                      'uuid': '123',
+                      'uuid': uuids.instance_1,
                       'vm_state': '',
                       'task_state': ''},
                      {'image_ref': '2',
                       'host': CONF.host,
                       'id': '2',
-                      'uuid': '456',
+                      'uuid': uuids.instance_2,
                       'vm_state': '',
                       'task_state': ''},
                      {'image_ref': '2',
@@ -89,7 +92,7 @@ class ImageCacheManagerTests(test.NoDBTestCase):
                       'ramdisk_id': '22',
                       'host': 'remotehost',
                       'id': '3',
-                      'uuid': '789',
+                      'uuid': uuids.instance_3,
                       'vm_state': '',
                       'task_state': ''}]
 
@@ -98,27 +101,20 @@ class ImageCacheManagerTests(test.NoDBTestCase):
 
         image_cache_manager = imagecache.ImageCacheManager()
 
-        self.mox.StubOutWithMock(objects.block_device.BlockDeviceMappingList,
-                   'get_by_instance_uuid')
-
         ctxt = context.get_admin_context()
         swap_bdm_256_list = block_device_obj.block_device_make_list_from_dicts(
             ctxt, swap_bdm_256)
         swap_bdm_128_list = block_device_obj.block_device_make_list_from_dicts(
             ctxt, swap_bdm_128)
-        objects.block_device.BlockDeviceMappingList.get_by_instance_uuid(
-                ctxt, '123').AndReturn(swap_bdm_256_list)
-        objects.block_device.BlockDeviceMappingList.get_by_instance_uuid(
-                ctxt, '456').AndReturn(swap_bdm_128_list)
-        objects.block_device.BlockDeviceMappingList.get_by_instance_uuid(
-                ctxt, '789').AndReturn(swap_bdm_128_list)
-
-        self.mox.ReplayAll()
-
+        mock_bdms_by_uuid.return_value = {uuids.instance_1: swap_bdm_256_list,
+                                          uuids.instance_2: swap_bdm_128_list,
+                                          uuids.instance_3: swap_bdm_128_list}
         # The argument here should be a context, but it's mocked out
         running = image_cache_manager._list_running_instances(ctxt,
             all_instances)
 
+        mock_bdms_by_uuid.assert_called_once_with(ctxt,
+                     [uuids.instance_1, uuids.instance_2, uuids.instance_3])
         self.assertEqual(4, len(running['used_images']))
         self.assertEqual((1, 0, ['instance-00000001']),
                          running['used_images']['1'])
@@ -131,23 +127,19 @@ class ImageCacheManagerTests(test.NoDBTestCase):
                          running['used_images']['22'])
 
         self.assertIn('instance-00000001', running['instance_names'])
-        self.assertIn('123', running['instance_names'])
-
-        self.assertEqual(4, len(running['image_popularity']))
-        self.assertEqual(1, running['image_popularity']['1'])
-        self.assertEqual(2, running['image_popularity']['2'])
-        self.assertEqual(1, running['image_popularity']['21'])
-        self.assertEqual(1, running['image_popularity']['22'])
+        self.assertIn(uuids.instance_1, running['instance_names'])
 
         self.assertEqual(len(running['used_swap_images']), 2)
         self.assertIn('swap_128', running['used_swap_images'])
         self.assertIn('swap_256', running['used_swap_images'])
 
-    def test_list_resizing_instances(self):
+    @mock.patch.object(objects.BlockDeviceMappingList,
+                       'bdms_by_instance_uuid')
+    def test_list_resizing_instances(self, mock_bdms_by_uuid):
         instances = [{'image_ref': '1',
                       'host': CONF.host,
                       'id': '1',
-                      'uuid': '123',
+                      'uuid': uuids.instance,
                       'vm_state': vm_states.RESIZED,
                       'task_state': None}]
 
@@ -155,25 +147,20 @@ class ImageCacheManagerTests(test.NoDBTestCase):
                          for instance in instances]
 
         image_cache_manager = imagecache.ImageCacheManager()
-        self.mox.StubOutWithMock(objects.block_device.BlockDeviceMappingList,
-                   'get_by_instance_uuid')
 
         ctxt = context.get_admin_context()
         bdms = block_device_obj.block_device_make_list_from_dicts(
             ctxt, swap_bdm_256)
-        objects.block_device.BlockDeviceMappingList.get_by_instance_uuid(
-                ctxt, '123').AndReturn(bdms)
+        mock_bdms_by_uuid.return_value = {uuids.instance: bdms}
 
-        self.mox.ReplayAll()
         running = image_cache_manager._list_running_instances(ctxt,
             all_instances)
 
+        mock_bdms_by_uuid.assert_called_once_with(ctxt, [uuids.instance])
         self.assertEqual(1, len(running['used_images']))
         self.assertEqual((1, 0, ['instance-00000001']),
                          running['used_images']['1'])
-        self.assertEqual(set(['instance-00000001', '123',
-                              'instance-00000001_resize', '123_resize']),
+        self.assertEqual(set(['instance-00000001', uuids.instance,
+                              'instance-00000001_resize',
+                              '%s_resize' % uuids.instance]),
                          running['instance_names'])
-
-        self.assertEqual(1, len(running['image_popularity']))
-        self.assertEqual(1, running['image_popularity']['1'])
