@@ -39,7 +39,6 @@ from nova import exception
 from nova.i18n import _
 import nova.privsep.fs
 import nova.privsep.libvirt
-from nova import utils
 from nova.virt.disk.mount import api as mount
 from nova.virt.disk.vfs import api as vfs
 from nova.virt.image import model as imgmodel
@@ -50,81 +49,43 @@ LOG = logging.getLogger(__name__)
 
 CONF = nova.conf.CONF
 
-_MKFS_COMMAND = {}
-_DEFAULT_MKFS_COMMAND = None
 
-FS_FORMAT_EXT2 = "ext2"
-FS_FORMAT_EXT3 = "ext3"
-FS_FORMAT_EXT4 = "ext4"
-FS_FORMAT_XFS = "xfs"
-FS_FORMAT_NTFS = "ntfs"
-FS_FORMAT_VFAT = "vfat"
+# NOTE(mikal): Here as a transition step
+SUPPORTED_FS_TO_EXTEND = nova.privsep.fs.SUPPORTED_FS_TO_EXTEND
 
-SUPPORTED_FS_TO_EXTEND = (
-    FS_FORMAT_EXT2,
-    FS_FORMAT_EXT3,
-    FS_FORMAT_EXT4)
-
-_DEFAULT_FILE_SYSTEM = FS_FORMAT_VFAT
-_DEFAULT_FS_BY_OSTYPE = {'linux': FS_FORMAT_EXT4,
-                         'windows': FS_FORMAT_NTFS}
 
 for s in CONF.virt_mkfs:
     # NOTE(yamahata): mkfs command may includes '=' for its options.
     #                 So item.partition('=') doesn't work here
     os_type, mkfs_command = s.split('=', 1)
     if os_type:
-        _MKFS_COMMAND[os_type] = mkfs_command
-    if os_type == 'default':
-        _DEFAULT_MKFS_COMMAND = mkfs_command
-
-
-def get_fs_type_for_os_type(os_type):
-    return os_type if _MKFS_COMMAND.get(os_type) else 'default'
-
-
-def get_file_extension_for_os_type(os_type, specified_fs=None):
-    mkfs_command = _MKFS_COMMAND.get(os_type, _DEFAULT_MKFS_COMMAND)
-    if mkfs_command:
-        extension = mkfs_command
-    else:
-        if not specified_fs:
-            specified_fs = CONF.default_ephemeral_format
-            if not specified_fs:
-                specified_fs = _DEFAULT_FS_BY_OSTYPE.get(os_type,
-                                                         _DEFAULT_FILE_SYSTEM)
-        extension = specified_fs
-    return utils.get_hash_str(extension)[:7]
+        nova.privsep.fs.load_mkfs_command(os_type, mkfs_command)
 
 
 def mkfs(os_type, fs_label, target, run_as_root=True, specified_fs=None):
-    """Format a file or block device using
-       a user provided command for each os type.
-       If user has not provided any configuration,
-       format type will be used according to a
-       default_ephemeral_format configuration
-       or a system defaults.
-    """
-
-    mkfs_command = (_MKFS_COMMAND.get(os_type, _DEFAULT_MKFS_COMMAND) or
-                    '') % {'fs_label': fs_label, 'target': target}
-    if mkfs_command:
-        utils.execute(*mkfs_command.split(), run_as_root=run_as_root)
-    else:
-        if not specified_fs:
-            specified_fs = CONF.default_ephemeral_format
-            if not specified_fs:
-                specified_fs = _DEFAULT_FS_BY_OSTYPE.get(os_type,
-                                                         _DEFAULT_FILE_SYSTEM)
-
-        utils.mkfs(specified_fs, target, fs_label, run_as_root=run_as_root)
+    nova.privsep.fs.configurable_mkfs(
+        os_type, fs_label, target, run_as_root,
+        CONF.default_ephemeral_format, specified_fs)
 
 
 def resize2fs(image, check_exit_code=False, run_as_root=False):
-    if run_as_root:
-        nova.privsep.fs.resize2fs(image, check_exit_code)
+    # NOTE(mikal): note that the check_exit_code kwarg here only refers to
+    # resize2fs, not the precursor e2fsck. Yes, I agree it's confusing.
+    try:
+        if run_as_root:
+            nova.privsep.fs.e2fsck(image)
+        else:
+            nova.privsep.fs.unprivileged_e2fsck(image)
+
+    except processutils.ProcessExecutionError as exc:
+        LOG.debug("Checking the file system with e2fsck has failed, "
+                  "the resize will be aborted. (%s)", exc)
+
     else:
-        nova.privsep.fs.unprivileged_resize2fs(image, check_exit_code)
+        if run_as_root:
+            nova.privsep.fs.resize2fs(image, check_exit_code)
+        else:
+            nova.privsep.fs.unprivileged_resize2fs(image, check_exit_code)
 
 
 def get_disk_info(path):
@@ -174,7 +135,7 @@ def extend(image, size):
         nova.privsep.libvirt.ploop_resize(image.path, size)
         return
 
-    utils.execute('qemu-img', 'resize', image.path, size)
+    processutils.execute('qemu-img', 'resize', image.path, size)
 
     if (image.format != imgmodel.FORMAT_RAW and
         not CONF.resize_fs_using_block_device):
@@ -256,7 +217,7 @@ def is_image_extendable(image):
     else:
         # For raw, we can directly inspect the file system
         try:
-            utils.execute('e2label', image.path)
+            processutils.execute('e2label', image.path)
         except processutils.ProcessExecutionError as e:
             LOG.debug('Unable to determine label for image %(image)s with '
                       'error %(error)s. Cannot resize.',
