@@ -269,8 +269,8 @@ class Guest(object):
                 flags |= libvirt.VIR_DOMAIN_UNDEFINE_NVRAM
             self._domain.undefineFlags(flags)
         except libvirt.libvirtError:
-            LOG.debug("Error from libvirt during undefineFlags. %d"
-                      "Retrying with undefine", self.id)
+            LOG.debug("Error from libvirt during undefineFlags for guest "
+                      "%d. Retrying with undefine", self.id)
             self._domain.undefine()
         except AttributeError:
             # Older versions of libvirt don't support undefine flags,
@@ -404,7 +404,7 @@ class Guest(object):
                               device, persistent, live)
 
             except libvirt.libvirtError as ex:
-                with excutils.save_and_reraise_exception(reraise=False) as ctx:
+                with excutils.save_and_reraise_exception():
                     errcode = ex.get_error_code()
                     if errcode in (libvirt.VIR_ERR_OPERATION_FAILED,
                                    libvirt.VIR_ERR_INTERNAL_ERROR):
@@ -421,10 +421,6 @@ class Guest(object):
                             # detach fails because the device is not found
                             raise exception.DeviceNotFound(
                                 device=alternative_device_name)
-                    # Re-raise the original exception if we're not raising
-                    # DeviceNotFound instead. This will avoid logging of a
-                    # "Original exception being dropped" traceback.
-                    ctx.reraise = True
 
         conf = get_device_conf_func(device)
         if conf is None:
@@ -644,6 +640,7 @@ class Guest(object):
            VIR_MIGRATE_UNSAFE Force migration even if it is considered
                               unsafe.
            VIR_MIGRATE_OFFLINE Migrate offline
+           VIR_MIGRATE_TLS Use QEMU-native TLS
         :param bandwidth: The maximum bandwidth in MiB/s
         """
         params = {}
@@ -691,13 +688,6 @@ class Guest(object):
         """
         self._domain.migrateSetMaxDowntime(mstime)
 
-    def migrate_configure_max_speed(self, bandwidth):
-        """The maximum bandwidth that will be used to do migration
-
-        :param bw: Bandwidth in MiB/s
-        """
-        self._domain.migrateSetMaxSpeed(bandwidth)
-
     def migrate_start_postcopy(self):
         """Switch running live migration to post-copy mode"""
         self._domain.migrateStartPostCopy()
@@ -869,6 +859,10 @@ class BlockDevice(object):
 
         return False
 
+    def blockStats(self):
+        """Extracts block device statistics for a domain"""
+        return self._guest._domain.blockStats(self._disk)
+
 
 class VCPUInfo(object):
     def __init__(self, id, cpu, state, time):
