@@ -145,24 +145,23 @@ class Image(object):
         """
         pass
 
-    def libvirt_info(self, disk_bus, disk_dev, device_type, cache_mode,
+    def libvirt_info(self, disk_info, cache_mode,
                      extra_specs, hypervisor_version, boot_order=None,
                      disk_unit=None):
         """Get `LibvirtConfigGuestDisk` filled for this image.
 
-        :disk_dev: Disk bus device name
-        :disk_bus: Disk bus type
-        :device_type: Device type for this image.
+        :disk_info: Metadata generated by libvirt.blockinfo.get_disk_mapping
         :cache_mode: Caching mode for this image
         :extra_specs: Instance type extra specs dict.
         :hypervisor_version: the hypervisor version
         :boot_order: Disk device boot order
         """
+        disk_bus = disk_info['bus']
         info = vconfig.LibvirtConfigGuestDisk()
         info.source_type = self.source_type
-        info.source_device = device_type
+        info.source_device = disk_info['type']
         info.target_bus = disk_bus
-        info.target_dev = disk_dev
+        info.target_dev = disk_info['dev']
         info.driver_cache = cache_mode
         info.driver_discard = self.discard_mode
         info.driver_io = self.driver_io
@@ -283,7 +282,7 @@ class Image(object):
 
             if (self.preallocate and self._can_fallocate() and
                     os.access(self.path, os.W_OK)):
-                utils.execute('fallocate', '-n', '-l', size, self.path)
+                processutils.execute('fallocate', '-n', '-l', size, self.path)
 
     def _can_fallocate(self):
         """Check once per class, whether fallocate(1) is available,
@@ -431,14 +430,6 @@ class Image(object):
         raise exception.ImageUnacceptable(image_id=image_id_or_uri,
                                           reason=reason)
 
-    def flatten(self):
-        """Flatten an image.
-
-        The implementation of this method is optional and therefore is
-        not an abstractmethod.
-        """
-        raise NotImplementedError('flatten() is not implemented')
-
     def direct_snapshot(self, context, snapshot_name, image_format, image_id,
                         base_image_id):
         """Prepare a snapshot for direct reference from glance.
@@ -865,28 +856,27 @@ class Rbd(Image):
 
         self.discard_mode = CONF.libvirt.hw_disk_discard
 
-    def libvirt_info(self, disk_bus, disk_dev, device_type, cache_mode,
+    def libvirt_info(self, disk_info, cache_mode,
                      extra_specs, hypervisor_version, boot_order=None,
                      disk_unit=None):
         """Get `LibvirtConfigGuestDisk` filled for this image.
 
-        :disk_dev: Disk bus device name
-        :disk_bus: Disk bus type
-        :device_type: Device type for this image.
+        :disk_info: Metadata generated by libvirt.blockinfo.get_disk_mapping
         :cache_mode: Caching mode for this image
         :extra_specs: Instance type extra specs dict.
         :hypervisor_version: the hypervisor version
         :boot_order: Disk device boot order
         """
         info = vconfig.LibvirtConfigGuestDisk()
+        disk_bus = disk_info['bus']
 
         hosts, ports = self.driver.get_mon_addrs()
-        info.source_device = device_type
+        info.source_device = disk_info['type']
         info.driver_format = 'raw'
         info.driver_cache = cache_mode
         info.driver_discard = self.discard_mode
         info.target_bus = disk_bus
-        info.target_dev = disk_dev
+        info.target_dev = disk_info['dev']
         info.source_type = 'network'
         info.source_protocol = 'rbd'
         info.source_name = '%s/%s' % (self.pool, self.rbd_name)
@@ -969,9 +959,6 @@ class Rbd(Image):
         raise exception.ImageUnacceptable(image_id=image_id_or_uri,
                                           reason=reason)
 
-    def flatten(self):
-        self.driver.flatten(self.rbd_name, pool=self.pool)
-
     def get_model(self, connection):
         secret = None
         if CONF.libvirt.rbd_secret_uuid:
